/*
 * Bitmaps.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <unistd.h>

#include <hr_defines.h>
#include <hr_general.h>
#include <hr_bobs.h>
#include <hr_ilbm.h>

struct hr_bitmap{
	int flags;
	int width;
	int height;
	int colors;
	int color_offset;
	unsigned char *data;
	unsigned char *palette;
};


/* Creates a bitmap.*/
struct hr_bitmap *hr_create_bitmap (unsigned char *data,
												unsigned char *palette,
												int width,int height,
												int colors,int flags)
{
	struct hr_bitmap *b;
	b=malloc(sizeof(struct hr_bitmap));
	b->flags=flags;
	b->width=width;
	b->height=height;
	b->colors=colors;
	b->color_offset=0;
	b->data=data;
	/* rgbrgbrgbrgbrgb-form (24bit) */
	b->palette=palette;
	return b;
}


/* Deletes a bitmap, its data and palette.*/
void hr_delete_bitmap(struct hr_bitmap *b) {
	if(b!=NULL){
		free(b->data);
		free(b->palette);
		free(b);
	}
}


/* Recolors a bitmap. Color 0 is left untouched unless GFX_INCLUDEZERO flag
 * is given. The func only works for color bmaps.*/
void hr_recolor_bitmap(struct hr_bitmap *p,int color_offset,int flags) {
	int l;
	unsigned char *tp;
	if((p!=NULL) && (color_offset!=0)){
		if(p->flags & GFX_256COLOR){	
			tp=p->data;
			for(l=0;l<((p->width)*(p->height));l++){
				if((*tp)||(flags & GFX_INCLUDEZERO))
				  (*tp)+=color_offset;
				++tp;
			}
			p->color_offset+=color_offset;
		}
	}
}


/* Draws a bit-per-pixel image on screen. SLOW! */
void hr_monodraw(struct hr_bitmap *b,int x,int y,int color) {
	int xx,yy;
	int pixel_offset=0;
	int byte_offset,bit_offset;
	if(b!=NULL)
	  for(yy=0;yy<(b->height);yy++) {
		  for(xx=0;xx<(b->width);xx++) {
			  byte_offset=pixel_offset/8;
			  bit_offset=pixel_offset%8;
			  if(*(b->data+byte_offset) & (128>>bit_offset))
				 gl_setpixel(x+xx,y+yy,color);
			  ++pixel_offset;
		  }
	  }
}

void hr_use_bitmap_palette(struct hr_bitmap *bmap,
									int mincol,int maxcol,int flags)
{
	int c,d,offset;
	if(bmap!=NULL){
		if(bmap->palette!=NULL){
			if(bmap->flags & GFX_24BPALETTE)
			  d=4;
			else
			  d=1;
			if(flags & GFX_INCLUDEZERO)
			  offset=0;
			else
			  offset=3;
			for(c=0;c<(bmap->colors);c++){
				/* The values are divided with 4 to get
				 * 6+6+6 bit color resolution */
				gl_setpalettecolor(mincol+c,
										 *(bmap->palette+offset+(c*3))/d,
										 *(bmap->palette+offset+(c*3+1))/d,
										 *(bmap->palette+offset+(c*3+2))/d);
				/* Break away if maxcol is reached.*/
				if((mincol+c)==maxcol)
				  break;
			}
		}
	}
}



/* FIX FIX FIX FIX FIX FIX FIX FIX */
/* Returns index of brightest/darkest color
 * in either bmap palette or current vga palette.
 * -1 == error. */
int hr_color_extreme(struct hr_bitmap *b,int mincol,int maxcol,int flags){
	int extreme=-1,c,rr,gg,bb;
	double v,mv;
	/* Check bmap palette boundaries.
	 * if mincol>maxcol, -1 is returned sooner or later.*/
	if(flags & GFX_NOBMAPPALETTE){
		if(maxcol>255)
		  maxcol=255;
	}
	else if(b!=NULL){
		if((b->palette)!=NULL){
			if(maxcol>=(b->colors))
			  maxcol=(b->colors)-1;
		}
		else
		  return -1;							/* Error. NULL bmap palette. */
	}
	else
	  return -1;								/* Error. NULL bmap. */
	/**/
	if(mincol<0)
	  mincol=0;
	if(flags & GFX_BRITEST)
	  mv=0;
	else 
	  mv=666.0;
	for(c=mincol;c<=maxcol;c++){
		/* Get RGB components from palette.
		 * No need to check whether they are 24bit, 18bit or whatever.*/
		if(flags & GFX_NOBMAPPALETTE){
			/* Search color from current vga palette */
			gl_getpalettecolor(c,&rr,&gg,&bb);
		}
		else {
			/* Get color from bmap palette.*/
			rr=(int)(*((b->palette)+c*3));
			gg=(int)(*((b->palette)+c*3+1));
			bb=(int)(*((b->palette)+c*3+2));
		}
		/* brightness */
		/*v=(rr-(gg+0.509*rr+0.194*bb)/1.704)/1.140;*/
		v=MAX(rr,MAX(gg,bb));
		if(flags & GFX_BRITEST){
			if(v>mv){
				mv=v;
				extreme=c;
			}
		}
		else{
			if(v<mv){
				mv=v;
				extreme=c;
			}
		}
	}
	return extreme;
}
