/*
 *	Generic svgalib moving objects.
 * Note that .bitmap is just a pointer to bitmap malloced elsewhere.
 * First create [&recolor] a bitmap, then create a bob using it.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <unistd.h>

#include <hr_defines.h>
#include <hr_bitmaps.h>
#include <hr_general.h>


struct hr_bob{
	int flags;
	unsigned int index;
	int x;
	int y;
	int hot_x;
	int hot_y;
	struct hr_bitmap *bitmap;
	struct hr_bitmap *tempmap;
};


/* Deletes a bob. Tempmap is freed wholly, bitmap data is not touched.*/
void hr_delete_bob(struct hr_bob *b) {
   if(b!=NULL) {
		hr_delete_bitmap(b->tempmap);
		free(b);
	}
}


/* Creates a bob and returns pointer to the new button. */
struct hr_bob *hr_create_bob(unsigned int index,
									  int x,int y,int hot_x,int hot_y,
									  struct hr_bitmap *bitmap,int flags)
{
	struct hr_bob *b=NULL;
	unsigned char *t;
	if(bitmap->flags & GFX_256COLOR){
		b=malloc(sizeof(struct hr_bob));
		b->flags=flags;
		b->index=index;
		b->x=x;
		b->y=y;
		b->hot_x=hot_x;
		b->hot_y=hot_y;
		b->bitmap=bitmap;
		/* This is needed to save the background gfx.*/
		t=malloc((bitmap->width)*(bitmap->height));
		b->tempmap=hr_create_bitmap(t,NULL,bitmap->width,bitmap->height,
											 256,GFX_256COLOR);
	}
	return b;
}


/* Updates a bob. */
void hr_update_bob(struct hr_bob *b,int x,int y,
						 struct hr_bitmap *bitmap,int mode)
{
	int ox,oy;
	/**/
	if(b!=NULL) {
		ox=b->x;
		oy=b->y;
		/* Do not change bob's position if it is inited/removed ONLY */
		switch(mode){
		 case BOB_INITONLY:
		 case BOB_REMOVEONLY:
			x=b->x;
			y=b->y;
			break;
		 default:
			b->x=x;
			b->y=y;
			break;
		}
		/* If the bob is wanted to be animated...*/
		if(bitmap!=NULL)
		  b->bitmap=bitmap;
		/* case BOB_NOIMG is ignored.. */
		switch(mode) {
		 case BOB_INIT:
		 case BOB_INITONLY:
			/* Initalize bob after some gfx is changed.*/
			if(!(b->flags & BOB_INITIALIZED)) {
				b->flags |= BOB_INITIALIZED;
				gl_getbox(x-(b->hot_x),y-(b->hot_y),
							 b->tempmap->width,b->tempmap->height,b->tempmap->data);
				gl_putboxmask(x-(b->hot_x),y-(b->hot_y),
								  b->bitmap->width,b->bitmap->height,b->bitmap->data);
				break;
			}
		 case BOB_SMART:
			/* Normal mode. Bob moved on top of unchanging background. */
			/* Now save the pointer background and paste pointer to screen.*/
			if((b->flags & BOB_INITIALIZED) &&
				((ox!=x)||(oy!=y))) {
				/* Normal mode. Bob moved on top of unchanging background.
				 * First restores old background, then saves new background
				 * and draws Bob pointer. */
				gl_putbox(ox-(b->hot_x),oy-(b->hot_y),
							 b->tempmap->width,b->tempmap->height,
							 b->tempmap->data);
				gl_getbox(x-(b->hot_x),y-(b->hot_y),
							 b->tempmap->width,b->tempmap->height,
							 b->tempmap->data);
				gl_putboxmask(x-(b->hot_x),y-(b->hot_y),
								  b->bitmap->width,b->bitmap->height,
								  b->bitmap->data);
			}
			break;
		 case BOB_DUMB:
			/* This is used when the whole screen is constantly redrawn. */
			gl_putboxmask(x-(b->hot_x),y-(b->hot_y),
							  b->bitmap->width,b->bitmap->height,
							  b->bitmap->data);
			break;
		 case BOB_REMOVE:
		 case BOB_REMOVEONLY:
			/* Remove Bob from screen */
			if(b->flags & BOB_INITIALIZED) {
				b->flags &= ~(BOB_INITIALIZED);
				gl_putbox(ox-(b->hot_x),oy-(b->hot_y),
							 b->tempmap->width,b->tempmap->height,
							 b->tempmap->data);
			}
			break;
		}
	}
}
