/*
 *	Button functions
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include <hr_defines.h>
#include <hr_fonts.h>
#include <hr_mouse.h>
#include <hr_bobs.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_keyboard.h>
#include <hr_string_edit.h>
#include <hr_windows.h>

/* When setting up sliders that are only horiz/vert and not both,
 * x/y variables can be ignored.*/
struct hr_slider{
	int flags;
	int x_items;
	int x_visible;
	int x_position;
	int x_minval;
	int *x_valint;
	int y_items;
	int y_visible;
	int y_position;
	int y_minval;
	int *y_valint;
};

struct hr_button{
	struct hr_button *next;
	unsigned int index;
	int flags;
	struct hr_window *parent_window;
	struct hr_font  *font;
	struct hr_color *colors;
	unsigned char  *text;
	unsigned char  *text2;
	struct hr_bitmap *bitmap;
	struct hr_bitmap *bitmap2;
	int x1;
	int y1;
	int x2;
	int y2;
	int hotkey;
	void *extras;
};


/* Returns 1 if button has STRING_EDITED-flag set.
 * The flag is unset before returning. */
int hr_button_string_edited(struct hr_button *b){
	if(b!=NULL){
		if(b->flags & STRING_EDITED){
			b->flags &= ~(STRING_EDITED);
			return 1;
		}
	}
	return 0;
}



/* Returns button under coordinates x,y or -1 if no button is there,
 * OR button that has "index" if its >0 ... blagasdsadsad!*/
struct hr_button *hr_get_button(int x,int y,unsigned int index)  {	
	struct hr_button *p=hr.firstbutton;
	while(p!=NULL) {
		if(p->parent_window == hr.active_window){
			if(index){
				if(p->index==index)
				  return p;
			}
			else{
				if((p->flags & ENABLED) &&
					(p->x1 <=x) &&
					(p->x2 >=x) &&
					(p->y1 <=y) &&
					(p->y2 >=y))
				  return p;
			}
		}
		p=p->next;
	}
	return NULL;
}


/* Shades a rectangular area */
void hr_shade(int x1,int y1,int x2,int y2,struct hr_color *colors){
	int x,y;
	for(y=y1;y<=y2;y++)  {
		for(x=x1;x<=x2;x++)  {
			if((x%2)==(y%2))
			  gl_setpixel(x,y,colors->dark);
		}
	}
}


/* Draws a nice bevelbox ("3D" box, light source topleft.) */
void hr_bevelbox(int x1,int y1,int x2,int y2,
					  struct hr_color *colors,int flags)
{
	int c1,c2;
	/* Set colors */
	if(flags & PRESSED) {
		c1=(colors->dark);
		c2=(colors->light);
	}
	else{
		c1=(colors->light);
		c2=(colors->dark);
	}
	/* Draw the bevelbox. */
	gl_hline(x1,y1,x2,c1);
	gl_line(x1,y1,x1,y2,c1);
	gl_hline(x1+1,y2,x2,c2);
	gl_line(x2,y1+1,x2,y2-1,c2);
}


/* If pressed==1, the textbox is darker gray,
 * otherwise brighter gray (compared to color 0).
 * Text that doesn't fit in the button is cropped.
 * NOTE! Font height may exceed height of button - no checking is made! */
void hr_textbox(int x1,int y1,int x2,int y2,
					 struct hr_font *font,struct hr_color *colors,
					 unsigned char *text,int flags)
{
	unsigned char *p;
	int w;
	/* Clear the background.*/
	gl_fillbox(x1+1,y1+1,abs(x2-x1)-1,abs(y2-y1)-1,
				  (flags & PRESSED)?(colors->pressed):(colors->raised));
	/* Draw the 3D bevelbox*/
	hr_bevelbox(x1,y1,x2,y2,colors,flags);
	/* Print the text. */
	if(text!=NULL)  {
		w=abs(x2-x1)+1;
		p=malloc(strlen(text)+1);
		strcpy(p,text);
		if(strlen(p)>w/(font->width))
		  *(p+w/(font->width))=0;
		hr_monowrite((x1+x2)/2-strlen(p)*(font->width/2),
						 (y1+y2)/2-(font->height/2)+1,
						 ((flags & PRESSED)?(colors->color2):(colors->color)),
						 colors->bgr,p,font,NONE);
		free(p);
	}
	/* Shade the textbox a la Amiga if it is disabled */
	if(!(flags & ENABLED))
	  hr_shade(x1,y1,x2,y2,colors);
}


/* Prints a text with accompanied textbox with a string in it.
 *                            +-----+
 * --->                  text |value|
 *                            +-----+ */
void hr_value_gadget(int x,int y,int w,int h,int offset,
							struct hr_font *font,struct hr_color *colors,
							unsigned char *text,
							unsigned char *valuetext,int value)
{
	unsigned char tmp[HR_BUFFER_SIZE];
	if(text!=NULL){
		strncpy(tmp,text,HR_BUFFER_SIZE-1);
		tmp[offset/8]=0;
		hr_monowrite(x,y+(h/2)-4,colors->color,colors->bgr,tmp,font,NONE);
	}
	if(valuetext==NULL) {
		sprintf(tmp,"%d",value);
		valuetext=tmp;
	}
	hr_textbox(x+offset,y,x+w-1,y+h-1,
				  font,colors,
				  valuetext,PRESSED|ENABLED);
}


/* Resets all buttons. Note! Bitmaps are not freed.*/
void hr_delete_all_buttons() {
	struct hr_button *op,*p;
	p=hr.firstbutton;
	while(p!=NULL)  {		
		op=p;
		p=p->next;
		free(op->text);
		free(op->text2);
		free(op);
	}
	hr.firstbutton=NULL;
	hr.default_button=NULL;
}


/* Unneccessary func, but makes source look nicer.*/
struct hr_slider *hr_create_slider(int flags,
											  int x_items,int x_visible,int x_position,
											  int x_minval,int *x_valint,
											  int y_items,int y_visible,int y_position,
											  int y_minval,int *y_valint)
{
	struct hr_slider *s=NULL;
	s=malloc(sizeof(struct hr_slider));
	s->flags=flags;
	s->x_items=MAX(1,x_items);
	s->x_visible=MIN(x_visible,x_items);/* Just to be sure :-) */
	s->x_position=x_position;
	s->x_minval=x_minval;
	s->x_valint=x_valint;
	s->y_items=MAX(1,y_items);
	s->y_visible=MIN(y_visible,y_items);/* Just to be sure :-) */
	s->y_position=y_position;
	s->y_minval=y_minval;
	s->y_valint=y_valint;
	return s;
}


/* This adds a 3D slider bar to a button that has its
 * HSLIDER or/and VSLIDER flags set.
 * Should be called from within update_button() only.*/
void hr_slider_bar(struct hr_button *b){
	struct hr_slider *s=(struct hr_slider *)(b->extras);
	int x1,y1,x2,y2,l;
	if(s!=NULL){
		/* Set shitloads of checks! FIX! FIX! FIX! FIX!
		 * w and/or h may be negative!
		 * default positions of topleft and bottomright corners of the
		 * slider bar */
		x1=(b->x1)+1;
		y1=(b->y1)+1;
		x2=(b->x2)-1;
	   y2=(b->y2)-1;
		/* If the slider button is set to watch either xvalint or yvalint,
		 * set new x/y_positions. This allows very easy updating
		 * of interactive sliders.*/
		if(s->flags & SLIDER_GET_XVALINT){
			if(s->x_valint!=NULL){
				if(s->flags & SLIDER_XNEG)
				  s->x_position=(s->x_minval)-*(s->x_valint);
				else
				  s->x_position=*(s->x_valint)-(s->x_minval);
			}
		}
		if(s->flags & SLIDER_GET_YVALINT){
			if(s->y_valint!=NULL){
				if(s->flags & SLIDER_YNEG)
				  s->y_position=(s->y_minval)-*(s->y_valint);
				else
				  s->y_position=*(s->y_valint)-(s->y_minval);
			}
		}
		if(b->flags & HSLIDER){
			if((s->x_items)>1){
				/* Calc width of bar */
				l=((s->x_visible)*((b->x2)-(b->x1)-1))/(s->x_items);
				/* Calc x1/x2.*/
				x1=(b->x1)+1+(((s->x_position)*
									((b->x2)-(b->x1)-1-l))/((s->x_items)-1));
				x2=x1+l;
				if(x1<=(b->x1))
				  x1=(b->x1)+1;
				if(x2>=(b->x2))
				  x2=(b->x2)-1;
			}
		}
		if(b->flags & VSLIDER){
			if((s->y_items)>1){
				/* Calc height of bar */
				l=((s->y_visible)*((b->y2)-(b->y1)-1))/(s->y_items);
				/* Calc y1/y2*/
				y1=(b->y1)+1+(((s->y_position)*
									((b->y2)-(b->y1)-1-l))/((s->y_items)-1));
				y2=y1+l;
				if(y1<=(b->y1))
				  y1=(b->y1)+1;
				if(y2>=(b->y2))
				  y2=(b->y2)-1;
			}
		}
		/* Draw it.*/
		gl_fillbox(x1,y1,(x2-x1+1),(y2-y1+1),b->colors->bgr);
		hr_bevelbox(x1,y1,x2-1,y2-1,b->colors,NONE);
	}
}


/* Updates button condition and possibly changes texts.
 * Bitmaps can be changed too using this func...
 * So animated buttons are possible :-) */
void hr_update_button(unsigned int index,struct hr_button *p,
							 int mode,void *data)
{
	int w,l,underscore_printed;
	struct hr_bitmap *tb;
	unsigned char *tt,*tmp;
	/* If the used wishes to access by index and not address...
	 * Find the address.*/
	if(p==NULL)
	  p=hr_get_button(0,0,index);
	if(p!=NULL) {
		if(data!=NULL){
			/* If a new string is given, a copy of the text is made.
			 * Otherwise just pointers are set.*/
			switch(mode){
#if 0
			 case UPDATE_GFXONLY:
				break;
#endif
			 case UPDATE_FLAGS:
				p->flags=(*(int *)data);
				break;
			 case UPDATE_X1:
				p->x1=(*(int *)data);
				break;
			 case UPDATE_Y1:
				p->y1=(*(int *)data);
				break;
			 case UPDATE_X2:
				p->x2=(*(int *)data);
				break;
			 case UPDATE_Y2:
				p->y2=(*(int *)data);
				break;
			 case UPDATE_TEXT:
				if((unsigned char *)data!=(p->text)) {
					free(p->text);
					p->text=malloc(strlen((unsigned char *)data)+1);
					strcpy(p->text,(unsigned char *)data);	
				}
				break;
			 case UPDATE_TEXT2:
				if((unsigned char *)data!=(p->text2)) {
					free(p->text2);
					p->text2=malloc(strlen((unsigned char *)data)+1);
					strcpy(p->text2,(unsigned char *)data);
				}
				break;
			 case UPDATE_BITMAP:
				/* Note that "old" bitmaps are not freed!*/
				p->bitmap=(struct hr_bitmap *)data;
				break;
			 case UPDATE_BITMAP2:
				p->bitmap2=(struct hr_bitmap *)data;
				break;
			 case UPDATE_FONT:
				/* Font modifying. */
				p->font=(struct hr_font *)data;
				break;
			 case UPDATE_COLORS:
				/* Color modifying */
				p->colors=(struct hr_color *)data;
				break;
			 case UPDATE_PARENT:
				/* Change "owner window" of button */
				p->parent_window=(struct hr_window *)data;
				break;
			 case UPDATE_HOTKEY:
				/* Change hotkey for button */
				p->hotkey=(int)(*((int *)data));
				break;
			 case UPDATE_EXTRAS:
				/* Set extras-pointer.*/
				p->extras=data;
				break;
			}
		}
		/* Draw the button if it is not invisible.
		 * The check is made here because it makes no sense to create
		 * invisible textboxes :-) */
		if(!(p->flags & INVISIBLE)){
			/* Clear the background.*/
			gl_fillbox((p->x1)+1,(p->y1)+1,
						  abs((p->x2)-(p->x1))-1,abs((p->y2)-(p->y1))-1,
						  (p->flags & PRESSED)?
						  (p->colors->pressed):
						  (p->colors->raised));
			/* Draw the bitmap.*/
			tb=(p->flags & PRESSED)?(p->bitmap2):(p->bitmap);
			if(tb!=NULL)
			  gl_putboxmask(((p->x1)+(p->x2))/2-(tb->width)/2,
								 ((p->y1)+(p->y2))/2-(tb->height)/2,
								 tb->width,tb->height,tb->data);
			/* Hilite? */
			if((hr.default_button==p) &&
				(p->flags & ENABLED))
			  gl_hline((p->x1)+1,(p->y2)-1,(p->x2)-1,p->colors->hilite);
			/* Print the text. If text2 does not exist, text is always used.*/
			if((p->text2)==NULL)
			  tt=p->text;
			else
			  tt=(p->flags & PRESSED)?(p->text2):(p->text);
			if(tt!=NULL){
				w=abs((p->x2)-(p->x1))+1;
				tmp=malloc(strlen(tt)+1);
				strcpy(tmp,tt);
				if(strlen(tmp)>w/(p->font->width))
				  *(tmp+w/(p->font->width))=0;
				hr_monowrite(((p->x1)+(p->x2))/2-strlen(tmp)*(p->font->width/2),
								 ((p->y1)+(p->y2))/2-(p->font->height/2)+1,
								 ((p->flags & PRESSED)?(p->colors->color2):
								  (p->colors->color)),
								 p->colors->bgr,tmp,p->font,NONE);
				/* If the button has both text and a hotkey, underline
				 * first hotkey character.*/
				if(((p->hotkey)>' ') && !(p->flags & INVISIBLE_HOTKEY)){
					/* Now convert tmp-string into spaces and underlines
					 * and print it over the original text. SLOW crap, but works.*/
					underscore_printed=0;
					for(l=0;l<strlen(tmp);l++){
						if(tolower((int)*(tt+l))==tolower((int)(p->hotkey)) &&
							(!underscore_printed)){
							*(tmp+l)='_';
							underscore_printed=1;
						}
						else
						  *(tmp+l)=' ';
					}
					hr_monowrite(((p->x1)+(p->x2))/2-strlen(tmp)*(p->font->width/2),
									 ((p->y1)+(p->y2))/2-(p->font->height/2)+2,
									 ((p->flags & PRESSED)?(p->colors->color2):
									  (p->colors->color)),
									 p->colors->bgr,tmp,p->font,NONE);
				}
				free(tmp);
			}
			/* If the button is a slider, draw the bar.*/
			if((p->flags & HSLIDER)||(p->flags & VSLIDER))
			  hr_slider_bar(p);
			/* Draw the 3D bevelbox */
			hr_bevelbox((p->x1),(p->y1),
							(p->x2),(p->y2),p->colors,p->flags);
			/* Shade the textbox a la Amiga if it is disabled */
			if(!(p->flags & ENABLED))
			  hr_shade(p->x1,p->y1,p->x2,p->y2,p->colors);
		}
	}
}


/* Creates a button and draws it to screen.
 * Returns pointer to the new button.
 * Note! If you create 1*1 pixel buttons, some other func may crash :-) */
struct hr_button *hr_create_button(unsigned int index,
											  int x1,int y1,int x2,int y2,
											  struct hr_font *font,
											  struct hr_color *colors,
											  unsigned char *text,
											  unsigned char *text2,
											  int flags){
	struct hr_button *p=hr.firstbutton;
	unsigned char *tt;
	if(p==NULL)
	  hr.firstbutton=p=malloc(sizeof(struct hr_button));
	else  {			  
		while(p->next!=NULL) {			
			p=p->next;
		}
		p->next=malloc(sizeof(struct hr_button));
		p=p->next;
	}
	/* This is absolutely necessary!*/
	p->parent_window=hr.active_window;
	/* The rest...*/
	p->next=NULL;
	p->bitmap=NULL;
	p->bitmap2=NULL;
	p->extras=NULL;
	p->hotkey=0;
	p->flags=flags;
	p->index=index;
	p->font=font;	
	p->colors=colors;
	p->x1=MIN(x1,x2);
	p->x2=MAX(x1,x2);
	p->y1=MIN(y1,y2);
	p->y2=MAX(y1,y2);
	/* Make copy of texts accompanied with a button.
	 * This space is automatically freed when either the text is changed
	 * or the button removed.*/
	(p->text)=(p->text2)=NULL;
	if(text!=NULL) {
		tt=malloc(strlen(text)+1);
		strcpy(tt,text);
		p->text=tt;
	}
	if(text2!=NULL) {
		tt=malloc(strlen(text2)+1);
		strcpy(tt,text2);
		p->text2=tt;
	}
	/* Drawing is done in update_button */
	hr_update_button(0,p,UPDATE_GFXONLY,NULL);
	return p;
}


/* Removes a button from the button list (but NOT from screen!). */
void hr_remove_button(struct hr_button *p) {
	struct hr_button *p2;
	if(p!=NULL) {		
		/* Set pointers so that list is not screwed.*/
		if(p==hr.firstbutton)
		  hr.firstbutton=p->next;
		else {		
			p2=hr.firstbutton;
			while(p2->next != p)
			  p2=p2->next;
			p2->next=p->next;						/* Jump "over" p  */
		}
		/* Button string(s) are freed.
		 * Bitmaps, font and colors are left untouched.
		 * free(NULL) is ok. */
		free(p->text);
		free(p->text2);
		free(p->extras);
		free(p);
		/* Final check to avoid nasty things */
		if(hr.default_button==p)
		  hr.default_button=NULL;
	}
}
