/*
 *	General functions.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <math.h>

#include <hr_defines.h>
#include <hr_bobs.h>
#include <hr_fonts.h>
#include <hr_bitmaps.h>
#include <hr_mouse.h>
#include <hr_keyboard.h>
#include <hr_zones.h>
#include <hr_buttons.h>
#include <hr_vconsole.h>
#include <hr_windows.h>
#include <hr_screens.h>


struct hr_color{
	unsigned char color;
	unsigned char color2;
	unsigned char hilite;
	unsigned char bgr;
	unsigned char light;
	unsigned char dark;
	unsigned char raised;
	unsigned char pressed;
};


/* This struct contains most HobbiRewl globals...
 * easier to hunt em down this way.*/
struct {
	unsigned int flags;
	unsigned char    *datapath;
	struct hr_color  *def_colors;
	struct hr_button *firstbutton;
	struct hr_button *default_button;
	struct hr_font   *font8x8;
	struct hr_menu   *firstmenu;
	struct hr_screen *active_screen;
	GraphicsContext  *physicalscreen;
	struct hr_window *firstwindow;
	struct hr_window *active_window;
	struct hr_zone   *firstzone;
   int  def_hotkey;
   int  (*def_func)();
} hr;

int pcos[360];
int psin[360];

/* Prints an error and exits. FIX! */
void hr_error_exit(unsigned char *text) {
	if(text!=NULL)
	  printf(":\t%s\n",text);
	/* Print the error.*/
	printf(":\t%d:\t%s\n",errno,strerror(errno));
	exit(1);
}


/* Call this before exiting.*/
void hr_cleanup() {
	hr_screen_close(hr.active_screen);
#if 0
	vga_setmode(TEXT);
	usleep(50000);
#endif
	hr_vc_switchback();
	free(hr.def_colors);
}



/* Always call this first.*/
void hr_init() {
	int a;
	unsigned char tmp[HR_BUFFER_SIZE];
	/* Run in free vconsole.*/
	hr_get_vconsole();
	atexit(hr_cleanup);	
#if 0
	vga_safety_fork(&hr_cleanup);
#endif
#if 1
	vga_disabledriverreport();
#endif
	/* Give up root privileges and init svgalib stuff..*/
	vga_init();	
	if(!(hr.datapath=getenv("HOBBIREWL_DATAPATH"))) {
		printf("HOBBIREWL_DATAPATH env variable not found.\n");
		exit(0);
	}
	srandom((unsigned int)time(NULL));
	/* Precalc sin/cos tables and convert from radians to degrees.*/
	for(a=0;a<360;a++){
		psin[a]=(int)(sin((double)((a*2.0*M_PI)/360.0))*((double)(1<<SCALE)));
		pcos[a]=(int)(cos((double)((a*2.0*M_PI)/360.0))*((double)(1<<SCALE)));
	}
	/* Clear the flags.*/
	hr.flags=0;
	/* Nullify some pointers */
   hr.firstzone=NULL;
	hr.firstbutton=NULL;
	hr.firstwindow=NULL;
   hr.def_hotkey=0;
	hr.def_func=NULL;
	/* NULL-active window means the "root" window.*/
	hr.active_window=NULL;
	/* Allocate gfx context for physical screen and nullify
	 * active screen pointer.*/
	hr.physicalscreen=gl_allocatecontext();
	hr.active_screen=NULL;
	/* No buttons yet.*/
	hr.default_button=NULL;
	/* Default color structure */
	hr.def_colors=malloc(sizeof(struct hr_color));
	hr.def_colors->color  =1;
	hr.def_colors->color2 =1;
	hr.def_colors->hilite =3;
	hr.def_colors->bgr    =0;
	hr.def_colors->light  =1;
	hr.def_colors->dark   =2;
	hr.def_colors->raised =4;
	hr.def_colors->pressed=5;
	/* Default font.*/
	sprintf(tmp,"%s%s",hr.datapath,FONT8X8);
	hr.font8x8=hr_create_font(tmp,GFX_MONO);
	/* Init mouse */
	hr_init_mouse();
	/* Clear keyboard input stream...*/
	hr_clearkey();
};


/* Floodfill. Uses recursion. FIX! int flags-->####*/
void hr_floodfill(int x,int y,int c,int rc,int mode) {
	gl_setpixel(x,y,c);
	switch(mode)   {
	 case 0:
		/* Mode 0 - Recolor color rc */
		if(vga_getpixel(x-1,y)==rc)
		  hr_floodfill(x-1,y,c,rc,mode);
		if(vga_getpixel(x+1,y)==rc)
		  hr_floodfill(x+1,y,c,rc,mode);
		if(vga_getpixel(x,y-1)==rc)
		  hr_floodfill(x,y-1,c,rc,mode);
		if(vga_getpixel(x,y+1)==rc)
		  hr_floodfill(x,y+1,c,rc,mode);
		break;
	 case 1:
		/* Mode 1 - Stop coloring at color rc */
		if(vga_getpixel(x-1,y)!=rc)
		  hr_floodfill(x-1,y,c,rc,mode);
		if(vga_getpixel(x+1,y)!=rc)
		  hr_floodfill(x+1,y,c,rc,mode);
		if(vga_getpixel(x,y-1)!=rc)
		  hr_floodfill(x,y-1,c,rc,mode);
		if(vga_getpixel(x,y+1)!=rc)
		  hr_floodfill(x,y+1,c,rc,mode);
		break;
	}
}


/* Replaces colors on range c1-c2 with nc.*/
void hr_replace_colors(int x1,int y1,int x2,int y2,int c1,int c2,int nc) {
	int x,y,c;
	for(y=y1;y<=y2;y++)   {
		for(x=x1;x<=x2;x++)   {
			c=vga_getpixel(x,y);
			if((c>=c1) && (c<=c2))
			  gl_setpixel(x,y,nc);
		}
	}
}


/* Ugly shit.*/
void hr_debug_log(unsigned char *text) {
	unsigned char *cmd_line;
	cmd_line=malloc(strlen(text)+26);
	sprintf(cmd_line,"echo \"%s\" >>/tmp/debug.log\n",text);
	system(cmd_line);
	free(cmd_line);
};


	
/* 16 first colors in palette are reserved.
 * These are never changed (at least, should not be changed).
 * The color style is taken from AmigaOs 2.0 (default)
 * and modified a bit.*/
void hr_default_palette()  {
	/* 4 AmigaOs 2.0-style colors */
	/* With ET4000 (15,15,15) was ok for color 0.
	 * CL-GD5422-80QC-D seems to output darker colors.*/
	gl_setpalettecolor(0,19,19,19);       /* Gray background. */
	gl_setpalettecolor(1,50,50,50);       /* White */
	gl_setpalettecolor(2,0,0,0);          /* Black */
	gl_setpalettecolor(3,0,10,47);       /* Blue */
	/* "pressed" and "raised" gray */
	gl_setpalettecolor(4,20,20,20);       /* Lighter Gray */
	gl_setpalettecolor(5,18,18,18);       /* Darker  Gray */
	/* 12-15 Pointer */
	hr_use_bitmap_palette(hr_mouse.bob->bitmap,12,15,NONE);
}



/* Circle.... useless...
 * Calcs 1/8 of dots, mirrors the rest.*/
#define CIRCLE_STEP 3
void hr_circle(int x,int y,int r,int c){
	int a,xp,yp,div;
	if(r>0){
		div=(1<<SCALE)/r;
		for(a=0;a<=45;a+=CIRCLE_STEP){
			xp=pcos[a]/div;
			yp=psin[a]/div;
			gl_setpixel(x+xp,y+yp,c);
			gl_setpixel(x-xp,y+yp,c);
			gl_setpixel(x+xp,y-yp,c);
			gl_setpixel(x-xp,y-yp,c);
			/* Swap x and y.*/
			gl_setpixel(x+yp,y+xp,c);
			gl_setpixel(x-yp,y+xp,c);
			gl_setpixel(x+yp,y-xp,c);
			gl_setpixel(x-yp,y-xp,c);
		}
	}
}


/* Draws a filled disc.*/
void hr_disc(int x,int y,int r,int c){
	int xx,yy,r2;
	if(r>0){
		r2=r*r;
		gl_hline(x-r,y,x+r,c);
		for(yy=1;yy<=r;yy++){
			xx=(int)(sqrt((double)(r2-(yy*yy))));
			gl_hline(x-xx,y-yy,x+xx,c);
			gl_hline(x-xx,y+yy,x+xx,c);
		}
	}
}

