/*
 *	Input handler.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include <hr_defines.h>
#include <hr_fonts.h>
#include <hr_mouse.h>
#include <hr_bobs.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_keyboard.h>
#include <hr_buttons.h>
#include <hr_string_edit.h>
#include <hr_windows.h>


/* The generic input handler. */
struct hr_button *hr_input_handler(struct hr_button *b,
											  int (*func)(),int flags)
{
	struct hr_slider *s=NULL;
	struct hr_button *ob,*ab;
	int f=0,key=0,bflags,ox,oy,xx,yy,func_ret=0;
	int hotkey_pressed=0,key_pressed=0,ok,l;

	/* Wait for a button press.
	 * NOTE! "AMB" means "Any Mouse Button" in this context! */
	if((flags & LMB_PRESSED)||
		(flags & RMB_PRESSED)||
		(flags & AMB_PRESSED)) {
		/* Return pointer to button that was clicked on,
		 * or NULL if mouse was clicked on "empty" space.
		 * Value of b is ignored.*/
		hr_update_mouse(NONE,BOB_INIT);
		/*Loop, until the required button is pressed */
		while((((flags & LMB_PRESSED) && !(HR_LMB)) ||
				 ((flags & RMB_PRESSED) && !(HR_RMB))||
				 ((flags & AMB_PRESSED) && !((HR_LMB)||
													  (HR_RMB))))&&
				!(key_pressed) &&
				!(hotkey_pressed) &&
				!(func_ret))
			{
				if((key=hr_getkey())){
					hr_update_mouse(NONE,BOB_REMOVE);
					if(!(flags & IGNORE_HOTKEYS)){
						/* Default button-related stuff.*/
						if(hr.default_button!=NULL){
							if((hr.default_button->parent_window)==hr.active_window){
								switch(key){
								 case 9:
									/* Tabulator. Cycles default button.*/
									hr.default_button->flags &= ~(NO_TAB_CYCLE);
									ob=hr.default_button;
									ok=0;
									do{
										if((hr.default_button->next)==NULL)
										  hr.default_button=hr.firstbutton;
										else
										  hr.default_button=hr.default_button->next;
										if((!(hr.default_button->flags & NO_TAB_CYCLE) &&
											 !(hr.default_button->flags & INVISIBLE) &&
											 (hr.default_button->flags & ENABLED) &&
											 ((hr.default_button->parent_window)==
											  hr.active_window))||
											/* Cycled through? */
											(hr.default_button==ob))
										  ok=1;
									}while(!ok);
									/* Update the original and new default button
									 * gfx.*/
									if(ob!=hr.default_button){
										hr_update_button(0,ob,UPDATE_GFXONLY,NULL);
										hr_update_button(0,hr.default_button,
															  UPDATE_GFXONLY,NULL);
									}
									break;
								 case 10:
									/* "Default" button selected (enter pressed) */
									if(hr.default_button->flags & ENABLED){
										b=hr.default_button;
										hotkey_pressed=1;
									}
									break;
								}
							}
						}
						/* Check hotkey */
						if(hr.firstbutton!=NULL){
							ob=hr.firstbutton;
							while(ob!=NULL){
								if((ob->flags & ENABLED) &&
									((ob->parent_window)==hr.active_window) &&
									(tolower(ob->hotkey)==tolower(key))){
									b=ob;
									b->flags |= HOTKEY_PRESSED;
									hotkey_pressed=1;
								}
								ob=ob->next;
							}
						}
					}
					/* Any key pressed */
					if(flags & KBD_PRESSED)
					  key_pressed=1;
					hr_update_mouse(NONE,BOB_INIT);
				}
				/* Check if func() returns true */
				if(func!=NULL)
				  func_ret=func();
				/* Update mouse */
#ifdef WHY_DOES_WAIT_FOR_EVENT_NEVER_WORK_WITH_MOUSE_OR_KEYBOARD
				if((func!=NULL) || (flags & KBD_PRESSED))
				  hr_update_mouse(NONE,BOB_SMART);
				else
				  hr_update_mouse(WAIT_EVENT,BOB_SMART);
#else
				hr_update_mouse(NONE,BOB_SMART);
#endif
			}
		hr_update_mouse(NONE,BOB_REMOVE);
		/* If hotkey was pressed, momentarily push/pop button in question.*/
		if(hotkey_pressed){
			if(b!=NULL){
				if(!(b->flags & NO_DELAY) &&
					!(b->flags & ACCEPTS_INPUT)){
					/* Get current button flags, store originals,
					 * toggle PRESSED, update button, wait.*/
					f=bflags=(b->flags);
					bflags ^= PRESSED;
					hr_update_button(0,b,UPDATE_FLAGS,&bflags);
#define DELAY 250000
					usleep(DELAY);	
					/* Restore original flags, update image.*/
					hr_update_button(0,b,UPDATE_FLAGS,&f);
				}
			}
		}
		/* KBD_PRESSes and nonzero func_ret values never
		 * modify the value of b.*/
		if(((key_pressed)||(func_ret))&&(!hotkey_pressed))
		  return b;
		else{
			/* If the button that was clicked on is a string gadget,
			 * edit the string, then return b, which at that time
			 * points either to the string gadget (Return was pressed)
			 * or to another button (editing was cancelled by clicking
   		 * outside the string gadget).*/
			if(!hotkey_pressed)
			  b=hr_get_button(hr_mouse.x,hr_mouse.y,0);
			if(b!=NULL){
				if((b->flags & ACCEPTS_INPUT) &&
					(!(flags & NO_EDITING)))
				  b=hr_string_edit(b,func,NONE);
			}
			return b;
		}
	}
	
	/* Ask for key to be released.------------------------------------------
	 * NOTE! "AMB" means "All Mouse Buttons" in this context! */
	if((flags & LMB_RELEASED)||
		(flags & RMB_RELEASED)||
		(flags & AMB_RELEASED)||
		(flags & KBD_RELEASED)) {
		/* Wait until mouse key is released, and while waiting,
		 * update the button that b points to.
		 * If b is NULL, this loop will not do anything visible.
		 * Return value is b, if mouse button is released on button b,
		 * NULL otherwise. */
		if(b!=NULL){
			/* Store flags */
			f=(b->flags);
			/* If the button is a slider... s points to that slider.*/
			if((b->flags & HSLIDER)||(b->flags & VSLIDER))
			  s=(struct hr_slider *)(b->extras);
		}
		/* Value of b is not touched. b contains the original button
		 * address all the time. ox,oy are set to -1 to make sure
		 * the slider is updated at least once. */
		ab=b;ob=ab-1;
		ox=oy=-1;
		hr_update_mouse(NONE,BOB_INIT);
		while((((flags & LMB_RELEASED) && (HR_LMB)) ||
				 ((flags & RMB_RELEASED) && (HR_RMB))||
				 ((flags & AMB_RELEASED) && ((HR_LMB)||
													  (HR_RMB)))
				 /*||((flags & KBD_RELEASED) && (hr_getkey()))*/)&&
				(!func_ret))
			{
				if(b!=NULL) {
					/* Sliders.*/
					if((b->flags & HSLIDER)||
						(b->flags & VSLIDER)){
						if((s!=NULL)&&
							((ox!=hr_mouse.x)||
							 (oy!=hr_mouse.y))) {						
							/* Remove mouse before changing gfx on screen */
							hr_update_mouse(NONE,BOB_REMOVE);
							/* Calc new x/y positions for slider bar.*
							 * and update slider structure data.*/
							if(b->flags & HSLIDER){
								/* Calc width of slider bar.
								 * Needed to know width of the "zones"
								 * on left and right that equal to minimum/
								 * maximum position. */
								l=((s->x_visible)*((b->x2)-(b->x1)-1))/(s->x_items);
								xx=hr_mouse.x-(b->x1)-(l/2);
								if(xx<0)
								  xx=0;
								if(xx>((b->x2)-(b->x1)-l))
								  xx=(b->x2)-(b->x1)-l;
								s->x_position=
								  ((xx*(s->x_items-1))/((b->x2)-(b->x1)-l));
								/* Set new value to sdfsdfasfas FIX? */
								if(s->x_valint!=NULL){
									if(s->flags & SLIDER_SET_XVALINT){
										if(s->flags & SLIDER_XNEG)
										  *(s->x_valint)=(s->x_minval)-(s->x_position);
										else
										  *(s->x_valint)=(s->x_minval)+(s->x_position);
									}
								}
							}
							if(b->flags & VSLIDER){
								l=((s->y_visible)*((b->y2)-(b->y1)-1))/(s->y_items);
								yy=hr_mouse.y-(b->y1)-(l/2);
								if(yy<0)
								  yy=0;
								if(yy>((b->y2)-(b->y1)-l))
								  yy=((b->y2)-(b->y1)-l);
								s->y_position=
								  ((yy*(s->y_items-1))/((b->y2)-(b->y1)-l));
								/**/
								if(s->y_valint!=NULL){
									if(s->flags & SLIDER_SET_YVALINT){
										if(s->flags & SLIDER_YNEG)
										  *(s->y_valint)=(s->y_minval)-(s->y_position);
										else
										  *(s->y_valint)=(s->y_minval)+(s->y_position);
									}
								}
							}
							/* Redraw button. Nothing is changed in the
							 * button structure, only within the slider structure,
							 * so no UPDATE_?-flag is needed.*/
							hr_update_button(0,b,NONE,NULL);
							/* Re-init mouse */
							hr_update_mouse(NONE,BOB_INIT);
							/* Store mew mouse coordinates.*/
							ox=hr_mouse.x;
							oy=hr_mouse.y;
						}
					}
					/* Check whether pointer is moved on top of different
					 * button, and whether either ab or ob points to
					 * button being checked (b).*/
					else if((ob!=ab) && ((ob==b)||(ab==b))) {
						ob=ab;
						/* Other buttons, EXCEPT string gadgets,
						 * are popped up/down if need be.*/
						if(!(b->flags & ACCEPTS_INPUT)){
							/* Remove mouse before changing gfx on screen */
							hr_update_mouse(NONE,BOB_REMOVE);
							/* If ab==b, switch PRESSED flag, otherwise give original
							 * flags. Strings/bitmaps are left untouched. */
							bflags=(ab==b)?(f ^ PRESSED):(f);
							hr_update_button(0,b,UPDATE_FLAGS,&bflags);
							/* Re-init mouse */
							hr_update_mouse(NONE,BOB_INIT);
						}
					}
				}
				if(func!=NULL)
				  func_ret=func();
				/* Update mouse structure data and get button under mouse
				 * pointer.*/
				hr_update_mouse(NONE,BOB_SMART);
				ab=hr_get_button(hr_mouse.x,hr_mouse.y,0);
			}
		hr_update_mouse(NONE,BOB_REMOVE);
		if(b!=NULL){
			/* If this button's hotkey was pressed,
			 * make sure switches etc work too.*/
			if(b->flags & HOTKEY_PRESSED){
				b->flags &= ~(HOTKEY_PRESSED);
				ab=b;
			}
		}
		/* If the jump out of input handler wa triggered by func(),
		 * do nothing and return NULL.*/
		if((!func_ret)){
			/* Note! SWITCH|ACCEPTS_INPUT --> Weirdness. Do not use that
			 * combination.*/
			if(b!=NULL) {
				/* Restore original flags */
				b->flags=f;
				/* If the button acts as a switch with on/off positions, and 
				 * if the button was selected, the state of PRESSED flag will be
				 * inverted.*/
				if((b->flags & SWITCH) && (ab==b))
				  (b->flags) ^= PRESSED;
				hr_update_button(0,b,UPDATE_FLAGS,&(b->flags));
				/* Mouse must not necessarily be released on a slider
				 * for the slider state to be updated.*/
				if((b->flags & HSLIDER)||(b->flags & VSLIDER))
				  return b;
			}
			/* Mouse must be released on the same button it was pressed on
			 * in case of normal buttons.*/
			if(ab==b)
			  return b;
		}
		
	}
	/* Blech. */
	return NULL;
}
