/*
 * PBM support.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <unistd.h>
#include <string.h>

#include <hr_defines.h>
#include <hr_general.h>
#include <hr_bobs.h>
#include <hr_bitmaps.h>

#define MAX_COLORS 256


/* Reads a PBM file from disk,
 * and creates a mono bitmap out of it.*/
struct hr_bitmap *hr_read_pbm(unsigned char *name,int flags){
	FILE *fp;
	int width,height,error=0,wbytes,x,y,l;
	unsigned char tmp[HR_BUFFER_SIZE],byte;
	unsigned char *ppmdata;
	struct hr_bitmap *bmap=NULL;

	/* Open the file. The file is parsed in 2 passes:
	 * first the palette is created, then the bitmap.*/
	if((fp=fopen(name,"r"))){
		/* Is this file a "raw" PBM? */
		fgets(tmp,HR_BUFFER_SIZE,fp);
		if(strncmp("P4",tmp,2)==0){
			/* Create bitmap struct.*/
			bmap=hr_create_bitmap(NULL,NULL,0,0,0,NONE);
			/* Skip comments.*/
			do{
				fgets(tmp,HR_BUFFER_SIZE,fp);
			}while(tmp[0]=='#');
			/* Ok, now get width and height info from tmp[].*/
			sscanf(tmp,"%d %d",&width,&height);
			
			/* Unga Bunga! */
			bmap->width=width;
			bmap->height=height;
			bmap->flags=GFX_MONO;
			
			/* Number of bytes per scanline.*/
			wbytes=width/8;
			if(width%8)
			  ++wbytes;
			/* Read the image data and close file.*/			
			ppmdata=malloc(wbytes*height);
			fread(ppmdata,1,wbytes*height,fp);
			fclose(fp);
			
			/* Copy palette to bitmap struct.*/
			bmap->colors=1;
			bmap->palette=malloc(3);
			*(bmap->palette)=255;
			*(bmap->palette+1)=255;
			*(bmap->palette+2)=255;
			
			/* Create bitmap */
			bmap->data=malloc((width*height)/8+1);
			/* Scan all pixels.*/
			l=0;
			for(y=0;y<(bmap->height);y++){
				for(x=0;x<(bmap->width);x++){
					byte=*(ppmdata+(y*wbytes)+x/8);
					/* WTF? Are PBM bytes inverted? */
					if(!(byte & (128>>(x%8))))
					  *((bmap->data)+l/8) |= (128>>(l%8));
					++l;
				}
			}
			free(ppmdata);
		}
		else{
			printf("\"P4\" identifier not found in %s.\n",name);
			error=2;
		}
	}
	else
	  printf("Error opening PBM file %s.\n",name);
	/* Kewlness */
	if(error)
	  return NULL;
	else
	  return bmap;
}
