/*
 * PPM support.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <unistd.h>
#include <string.h>

#include <hr_defines.h>
#include <hr_general.h>
#include <hr_bobs.h>
#include <hr_bitmaps.h>

#define MAX_COLORS 256


/* Reads a PPM file from disk, creates a palette for it
 * and creates an 256 color image (or b/w image) out of it.*/
/* FIX! */
struct hr_bitmap *hr_read_ppm(unsigned char *name,int flags){
	FILE *fp;
	int width,height,colors,error=0,l,p,newcolor,c;
	unsigned char tmp[HR_BUFFER_SIZE];
	unsigned char *image,*tp;
	unsigned char rr[MAX_COLORS],gg[MAX_COLORS],bb[MAX_COLORS];
	unsigned char *dp0,*dp1,*dp2,*ppmdata;
	struct hr_bitmap *bmap=NULL;

	/* Open the file. The file is parsed in 2 passes:
	 * first the palette is created, then the pixmap.*/
	if((fp=fopen(name,"r"))){
		/* Is this file a portable pixmap? */
		fgets(tmp,HR_BUFFER_SIZE,fp);
		if(strncmp("P6",tmp,2)==0){
			/* Skip comments.*/
			do{
				fgets(tmp,HR_BUFFER_SIZE,fp);
			}while(tmp[0]=='#');
			/* Get width and height.*/
			sscanf(tmp,"%d %d",&width,&height);
			/* Forget maxval. FIX! */
			fgets(tmp,HR_BUFFER_SIZE,fp);
			
			/* Malloc mem for converted bitmap */
			tp=image=malloc(width*height);
			
			/* Malloc space for a "pixel buffer".
			 * Hmm. Should this be bigger to speed things up? Blah...*/
#define BUFFERED_PIXELS 1024
			ppmdata=malloc(BUFFERED_PIXELS*3);
			/* This is just to get rid of a warning about using
			 * these possibly being used uninitialized..*/
			dp0=dp1=dp2=NULL;
			/* Scan all 24bit pixels,
			 * Create color table and create 256 color image.*/
			c=colors=newcolor=0;
			for(p=0;p<(width*height);p++){
				/* Read some more stuff if necessary */
				if((p%BUFFERED_PIXELS)==0){
					fread(ppmdata,3,BUFFERED_PIXELS,fp);
					/* Reset r,g,b pointers.*/
					dp0=ppmdata;
					dp1=ppmdata+1;
					dp2=ppmdata+2;
				}
				/* First check if this pixel is of same color
				 * than the one checked before. If yes,
				 * Skip past all the slow palette searchings etc...
				 * Speeds things up with most images */
				if(!((c>0)&&
					  (rr[c-1]==*dp0)&&
					  (gg[c-1]==*dp1)&&
					  (bb[c-1]==*dp2))){
					/* Is this an "old" color? */
					newcolor=1;
					for(l=0;l<colors;l++){
						if((rr[l]==*dp0)&&
							(gg[l]==*dp1)&&
							(bb[l]==*dp2)){
							/* Old color found.*/
							newcolor=0;
							/* Set pixel color.*/
							c=l+1;
							break;
						}
					}
					/* New color found! Create new color map entry */
					if(newcolor){
						if(colors<MAX_COLORS){
							/* Copy the 3 new colors bytes to palette.*/
							rr[colors]=*dp0;
							gg[colors]=*dp1;
							bb[colors]=*dp2;
							/* Set pixel color and inc amount of colors.*/
							c=(++colors);
						}
					}
				}
				/* Put new pixel to converted 256color bitmap
				 * and move tp pointer to next pixel.*/
				*(tp++)=(unsigned char)(c);
				/* Move rgb pointers to next rgb triplet.*/
				dp0+=3; dp1+=3; dp2+=3;
			}
			/* Close file.*/
			fclose(fp);
			/* Create pixmap struct. */
			bmap=hr_create_bitmap(image,malloc(colors*3),width,height,colors,
										 GFX_256COLOR|GFX_24BPALETTE);
			/* Copy palette.*/
			for(l=0;l<colors;l++){
				*((bmap->palette)+l*3)=(unsigned char)rr[l];
				*((bmap->palette)+l*3+1)=(unsigned char)gg[l];
				*((bmap->palette)+l*3+2)=(unsigned char)bb[l];
			}
		}
		else{
			printf("\"P6\" identifier not found in %s.\n",name);
			error=2;
		}
	}
	else
	  printf("Error opening PPM file %s.\n",name);
	/* Kewlness */
	if(error)
	  return NULL;
	else
	  return bmap;
}


/* Saves current frame buffer to disk in ppm format.
 * Returns 1 if everything went ok, 0 otherwise.*/
int hr_save_ppm(int x,int y,int w,int h,unsigned char *name,int flags){
	FILE *fp;
	int rr[256],gg[256],bb[256],xx,yy,c;
	if((x>=0)&&(x<WIDTH)&&(y>=0)&&(y<HEIGHT)){
		if((fp=fopen(name,"w"))){
			w=MIN(WIDTH,w);
			h=MIN(HEIGHT,h);
			/* Get palette */
			for(c=0;c<256;c++)
			  gl_getpalettecolor(c,&(rr[c]),&(gg[c]),&(bb[c]));
			/* Header */
			fprintf(fp,"P6\n#Screen dump\n%d %d\n255\n",WIDTH,HEIGHT);
			/* Image data */
			for(yy=y;yy<y+h;yy++){
				for(xx=x;xx<x+w;xx++){
					c=vga_getpixel(xx,yy);
					fputc(4*rr[c],fp);
					fputc(4*gg[c],fp);
					fputc(4*bb[c],fp);
				}
			}
			fclose(fp);
			return 1;
		}
		else
		  printf("Error opening %s to save a PPM file.\n",name);
	}
	return 0;
}
