/*
 *	General functions.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <math.h>

#include <hr_general.h>
#include <hr_defines.h>
#include <hr_bobs.h>
#include <hr_fonts.h>
#include <hr_bitmaps.h>
#include <hr_mouse.h>
#include <hr_keyboard.h>
#include <hr_zones.h>
#include <hr_buttons.h>
#include <hr_vconsole.h>
#include <hr_windows.h>


struct hr_screen {
	int flags;
	int mode;
	GraphicsContext *logical;
};


/* Opens a screen, and recalcs/sets mouse position if necessary.
 * Does not print any error messages. */
struct hr_screen *hr_screen_open(int mode,int flags){
	int ow,oh;
	hr.default_button=NULL;
	hr.active_window=NULL;

#ifdef SWITCH_KLUDGE
	/* Hmmm... if screenmode is changed from, say, 320x200 to 640x480,
	 * ctrl-c just hangs the program. Bug in svgalib?
	 * This temporary change to TEXT mode is necessary */
	usleep(SWITCH_KLUDGE);
	vga_setmode(TEXT);
	usleep(SWITCH_KLUDGE);
#endif

	/* Change to correct gfx mode.*/
	if (!vga_hasmode(mode))
	  return NULL;
	else {
		hr.active_screen=malloc(sizeof(struct hr_screen));
		ow=WIDTH;oh=HEIGHT;
		/* Make sure pointer is ready to be re-initialized */
		hr_update_mouse(NONE,BOB_REMOVE);
		/* Set mode, gl context and enable clipping.*/
		vga_setmode(mode);
		gl_setcontextvga(mode);
		if(flags & SCREEN_NOCLIP)
		  gl_disableclipping();
		else
		  gl_enableclipping();
		/**/
		hr.active_screen->flags=flags;
		hr.active_screen->mode=mode;
		/* set physical gfx context */
		gl_getcontext(hr.physicalscreen);
		/* set logical gfx context if required.*/
		if(flags & SCREEN_VIRTUAL){
			/* Virtual gfx context */
			gl_setcontextvgavirtual(mode);
			if(flags & SCREEN_NOCLIP)
			  gl_disableclipping();
			else
			  gl_enableclipping();
			hr.active_screen->logical = gl_allocatecontext();
			gl_getcontext(hr.active_screen->logical);
		}
		/* Reposition mouse pointer */
		if((WIDTH!=ow) && (HEIGHT!=oh) && (ow>0) && (oh>0))  {
			mouse_setposition((hr_mouse.x*WIDTH)/ow,(hr_mouse.y*HEIGHT)/oh);
			/* Update new coordinates in mouse structure */
			hr_update_mouse(NONE,BOB_NOIMG);
		}
		return hr.active_screen;
	}
}


/* "Closes" a screen (deallocs mem used by it) */
void hr_screen_close(struct hr_screen *screen){
	if(screen!=NULL){
		if(screen->flags & SCREEN_VIRTUAL){
			if((screen->logical)!=NULL)
			  gl_freecontext(screen->logical);
		}
		free(screen);
	}
}
