/*
 * Sound support.
 * Code taken from SndKit 2.5 recplay.c/help.c
 * (Copyright by Hannu Savolainen 1993)
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <unistd.h>

#include <hr_defines.h>
#include <hr_sound.h>

#include <malloc.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <sys/soundcard.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#ifndef __STDC__
#include <getopt.h>
#endif /* __STDC__ */


#define SOUND_DEVICE       "/dev/dsp1"
#define DEFAULT_DSP_SPEED  22000
#define DEFAULT_DSP_STEREO 0
#define DEFAULT_SAMPLESIZE 8
#define PLAY   1


#ifdef SOUND_SUPPORT
int audio;
struct sample_s {
	unsigned char *data;
	unsigned int  length;
};



int dsp_speed = DEFAULT_DSP_SPEED;
int dsp_stereo = DEFAULT_DSP_STEREO;
int samplesize = DEFAULT_SAMPLESIZE;
int abuf_size;

char *audiobuf, c;

void play (char *name);


/* Some error printing routines. */
void describe_error(void) {
	switch (errno){
	 case ENXIO:
	 case ENODEV:
		fprintf(stderr,	"The device file was found in /dev but\n"
				  "there is no driver for it in the kernel.\n"
				  "\n"
				  "There is several possible reasons:\n"
				  "- The sound driver is not present\n"
				  "  in the kernel you are currently running.\n"
				  "- The driver is in the kernel but there is\n"
				  "  no soundcard on your system.\n"
				  "- Configuration of the sound driver doesn't\n"
				  "  match the hardware.\n"
				  "\n"
				  "Try cat /dev/sndstat for further info.\n");
		break;
		
	 case ENOSPC:
		fprintf(stderr,	"The soundcard driver was not installed\n"
				  "properly. The sound_mem_init() routine\n"
				  "was not called during the boot procedure.\n");
		break;
		
	 case ENOENT:
		fprintf(stderr,	"The device file is missing from /dev.\n"
				  "Run the script at the end of file\n"
				  "sound/linux/Readme (distributed with\n"
				  "the sound driver.\n");
		break;
		
	 default: ;
	}
}


/* Inits samples defined in SAMPLE_LIST and opens audio device.*/
void init_sounds() {
	int tmp;

	printf("Initalizing sound...\n");
	/* Open audio device for playback.*/
	audio = open (SOUND_DEVICE, /*O_NONBLOCK|*/O_WRONLY, 0);
	if (audio == -1) {
		perror (SOUND_DEVICE);
		describe_error();
		exit (-1);
	}
	
	/* Set 8/16 "bitness".*/
	tmp = samplesize;
	ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &samplesize);
	if (tmp != samplesize) {
		fprintf(stderr, "Unable to set the sample size\n");
		exit(-1);
	}
	
	/* Set mono/stereo mode.*/
	if (ioctl (audio, SNDCTL_DSP_STEREO, &dsp_stereo)==-1) {
		fprintf (stderr, "Unable to set mono/stereo\n");
		perror (SOUND_DEVICE);
		exit (-1);
	}
	
	/* Set playback speed. Note! Actual speed may not be what was
	 * requested. SB sucks.*/
	if (ioctl (audio, SNDCTL_DSP_SPEED, &dsp_speed) == -1) {
		fprintf (stderr, "Unable to set audio speed\n");
		perror (SOUND_DEVICE);
		exit (-1);
	}
	
	/* Get current audio buffer size. */
	ioctl (audio, SNDCTL_DSP_GETBLKSIZE, &abuf_size);
	if (abuf_size < 1024 || abuf_size > (2*65536)) {
		if (abuf_size == -1)
		  perror (SOUND_DEVICE);
		else
		  fprintf (stderr, "Invalid audio buffers size %d\n", abuf_size);
		exit (-1);
	}
	
	
	/* Malloc space for audio buffer.*/
	if ((audiobuf = malloc (abuf_size)) == NULL) {
		fprintf (stderr, "Unable to allocate input/output buffer\n");
		exit (-1);
	}
}
/* close(audio) is in exit_stuff()! */


/* This one plays a sample. */
void play_sample(int number) {
/**********************************************************************/
	unsigned char *name="/tmp/sadfsfdasdfafdssfdsdfsdfsdafsadfsadfdsaFIX";
	int fd;
/**********************************************************************/
	int l;
	int count=0x7fffffff, c;
	
/***********************************************************************/
	if ((fd = open (name, O_RDONLY, 0)) == -1) {
		perror (name);
		exit (-1);
	}
/***********************************************************************/
	
	/* Copy blocks size of audiobuf until whole sample is played. */
	while (count) {
		c = count;
		
		if (c > abuf_size)
		  c = abuf_size;
		
		if ((l = read (fd, audiobuf, c)) > 0) {
			if (write (audio, audiobuf, l) != l) {
				perror (SOUND_DEVICE);
				exit (-1);
			}
			count -= l;
		}
		else {
			if (l == -1) {
				perror (name);
				exit (-1);
			}
			count = 0;	/* Stop */
		}
		
	}			/* while (count) */
/*************************************************************************/
	if (fd != 0)
	  close (fd);
/*************************************************************************/
}
#endif SOUND_SUPPORT
