/*
 *	Button functions
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>

#include <hr_defines.h>
#include <hr_fonts.h>
#include <hr_mouse.h>
#include <hr_general.h>
#include <hr_keyboard.h>
#include <hr_buttons.h>
#include <hr_bobs.h>
#include <hr_bitmaps.h>
#include <hr_input_handler.h>

/* Updates button and draws cursor.*/
void hr_editor_button_update(struct hr_button *b,
									  unsigned char *text,
									  int offset,int loc)
{
	int xx,yy,x1,y1,c,visible_characters,bflags;
	unsigned char tmp[HR_BUFFER_SIZE];
	/* Calc number of visible characters. */
	visible_characters=((b->x2)-(b->x1)+1)/(b->font->width)-1;
	/* Copy all characters to tmp[]*/
	if((offset<0) || (loc<0))
	  strncpy(tmp,text,HR_BUFFER_SIZE);
	/* Copy visible characters to tmp[].*/
	else{
		strncpy(tmp,text+offset,visible_characters+1);
		/* Force end of line.*/
		tmp[visible_characters]=0;
	}
	/* Redraw button gfx and set STRING_EDITED flag.*/
	bflags=(b->flags | STRING_EDITED);
	hr_update_button(0,b,UPDATE_FLAGS,&bflags);
	hr_update_button(0,b,((bflags & PRESSED)?
								 (UPDATE_TEXT2):(UPDATE_TEXT)),
						  tmp);
	/* Draw cursor, but only if it fits in the button.
	 * Note that no scrolling of text within the button is supported at the
	 * moment */
	if((loc>=0)/* && (loc<visible_characters)*/){
		/* Topleft coordinates of the cursor.*/
		x1=((b->x1)+(b->x2))/2-strlen(tmp)*((b->font->width)/2)+
		  (loc*(b->font->width));
		y1=((b->y1)+(b->y2))/2-((b->font->height)/2);
		/* Draw cursor */
		for(yy=y1;yy<y1+(b->font->height);yy++){
			for(xx=x1;xx<x1+(b->font->width);xx++){
				c=vga_getpixel(xx,yy);
				if((c==(b->colors->color)) ||
					(c==(b->colors->color)))
				  gl_setpixel(xx,yy,2);
				else
				  gl_setpixel(xx,yy,1);
			}
		}
	}
	
}


/* Lets one edit a string within a button interactively.
 * Very useful for making text-entry gadgets.
 * It looks better if text-entries are within an Amiga-style
 * double-bevelbox, that is, a raised bevelbox that contains a
 * pressed "button" with the text in it.
 * Note! The cursor is part of the string in buffer.
 * Therefor, strlen(buffer) should always be >=1.
 * Note2: Since no graphics are drawn by this func, one does not
 * need to worry about updating mouse pointer, buttons and such...
 * everything is automagical via hr_input_handler().*/
struct hr_button *hr_string_edit(struct hr_button *b,
											int (*func)(),int flags)
{
	struct hr_button *newb;
	int loc=0,nloc,offset=0,l,xx,visible_characters;
	int another_one,enter_received=0,first_time;
	unsigned char tmp[HR_BUFFER_SIZE];
	/* Do nothing if b points to NULL.*/
	newb=b;
	if(b!=NULL){
		do {
			/* Calc the amount of visible characters,
			 * and only loop if at least one character fits on screen.*/
			visible_characters=((b->x2)-(b->x1)+1)/(b->font->width)-1;
			if(visible_characters>0) {
				/* Copy the string to be edited */
				strncpy(tmp,
						  ((b->flags & PRESSED)?(b->text2):(b->text)),
						  HR_BUFFER_SIZE-2);
				/* Just to make sure keyboard.lastkey is 0.*/
				hr_clearkey();
				/* Loop while linefeed not received, AND
				 * left button not clicked outside this button. */
				enter_received=0;
				first_time=1;
				newb=b;
				while((first_time)||
						(!(enter_received) &&
						 (newb=hr_input_handler(b,func,
														KBD_PRESSED|LMB_PRESSED|
														NO_EDITING|IGNORE_HOTKEYS)) &&
						 (newb==b)))
					{
						/* Needs to be done for common good :-) */
						if(first_time){
							first_time=0;
							/* Calc initial offset and cursor location */
							if(strlen(tmp)>=visible_characters){
								offset=strlen(tmp)-visible_characters+1;
								loc=visible_characters-1;
							}
							else{
								offset=0;
								loc=strlen(tmp);
							}
							/* Redraw button gfx.*/
							hr_editor_button_update(b,tmp,offset,loc);
							/* Make sure no unnecessary repositioning is done.*/
							hr_input_handler(NULL,func,AMB_RELEASED);
						}
						/* Mouse event (LMB was clicked on the string gadget b).*/
						else if(HR_LMB){
							/* Calc new cursor location.*/
							xx=((b->x1)+(b->x2))/2-
							  (MIN(strlen(tmp),visible_characters)*
								(b->font->width))/2;
							if(hr_mouse.x<xx)
							  nloc=0;
							else
							  nloc=(hr_mouse.x-xx)/(b->font->width);
							/* Update loc and redraw button gfx
							 * if location was changed.*/
							if(nloc<0)
							  nloc=0;
							if(nloc>strlen(tmp))
							  nloc=strlen(tmp);
							if((nloc!=loc) && (nloc<=visible_characters-1)){
								loc=nloc;
								/* Redraw button gfx.*/
								hr_editor_button_update(b,tmp,offset,loc);
							}
							hr_input_handler(NULL,func,AMB_RELEASED);
						}
						/* Keyboard event.*/
						else if(hr_keyboard.lastkey!=0){
							switch(hr_keyboard.lastkey) {
							 case 10:
								/* Awayyyyyyyyy.*/
								enter_received=1;
								break;
							 case 27:
								/* Escape sequences. */
								if(hr_getkey()==91){
									switch(hr_getkey()) {
									 case 68:
										/* Curs left */
										if(loc>0)
										  --loc;
										else if(offset>0)
										  --offset;
										break;
									 case 67:
										/* Curs right */
										if(offset+loc<strlen(tmp)){
											if(loc<visible_characters-1)
											  ++loc;
											else
											  ++offset;
										}
										break;
									}
								}
								break;
							 case 4:
								/* Delete (Ctrl-D)*/
								if(offset+loc<strlen(tmp)) {
									for(l=offset+loc;l<strlen(tmp);l++)
									  tmp[l]=tmp[l+1];
									/* Move offset towards beginning if
									 * the string is longer than fits in the gadget
									 * AND if we are near the end of text.*/
									if((offset>0)&&
										((strlen(tmp)-offset-loc)<visible_characters)){
										--offset;
										++loc;
									}
								}
								break;
							 case 127:
								/* Backspace */
								if(offset+loc>0){
								if(loc>0){
									if((offset>0) &&
										(strlen(tmp)>=visible_characters))
									  --offset;
									else
									  --loc;
								}
									else
									  --offset;
									for(l=offset+loc;l<strlen(tmp);l++)
									  tmp[l]=tmp[l+1];
								}
								break;
							 default:
								/* Insert a character in buffer[], if it fits.*/
								if(strlen(tmp)<HR_BUFFER_SIZE-2) {
									/* Make space for a new char */
									for(l=strlen(tmp);l>=(offset+loc);l--)
									  tmp[l+1]=tmp[l];
									/* Insert the char */
									tmp[offset+loc]=hr_keyboard.lastkey;
									if(loc<visible_characters-1)
									  ++loc;
									else
									  ++offset;
								}
								break;
							}
							/* Redraw button gfx.*/
							hr_editor_button_update(b,tmp,offset,loc);
						}
					}
				/* Remove spaces */
				/*if(flags & REMOVE_B_SPACES){  FIX!*/
				while((strlen(tmp)>0) && (tmp[0]<33))
				  memcpy(tmp,tmp+1,strlen(tmp)-1);
				/*}FIX!*/
				/*if(flags & REMOVE_E_SPACES){FIX!*/
				while((strlen(tmp)>0) && (tmp[strlen(tmp)-1]<33))
				  tmp[strlen(tmp)-1]=0;
				/*}FIX!*/
				/* Replace weird characters within the string with spaces.*/
				for(l=0;l<strlen(tmp);l++){
					if(tmp[l]<32)
					  tmp[l]=32;
				}
				/* Redraw button gfx and copy whole tmp to button
				 * text/text2.*/
				hr_editor_button_update(b,tmp,-1,-1);
				/* Not really necessary...*/
				hr_clearkey();
			}
			/* Edit another string, if the used clicked in another
			 * string gadget. Kinda kludgish, but should work fine,
			 * and does not use recursion :-) */
			another_one=0;
			if((newb!=NULL)&&(!(enter_received))){
				if(newb->flags & ACCEPTS_INPUT){
					b=newb;
					another_one=1;
				}
			}
		}while(another_one);
	}
	return newb;
}
