/*
 * Vconsole stuff.
 * Everything taken from zgv 2.7 source by Russell Marks <mr216@gre.ac.uk>
 */


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/vt.h>
#include <sys/stat.h>

#include <hr_general.h>

int hr_original_vt,hr_separate_vt=0;
int hr_vt,hr_ttyfd;


/* Switch vconsole back to normal mode.*/
void hr_vc_switchback() {
	struct vt_mode vtm;
	/* also change back to stdin blocking;
	 * some versions of bash seem to be a little sensitive to it being
	 * left non-blocking.
	 */
	fcntl(hr_ttyfd,F_SETFL,0);
	if(hr_separate_vt) {
		fprintf(stderr,"%c[H%c[J",27,27);     /* seems to get junk-filled... */
		ioctl(hr_ttyfd,VT_GETMODE,&vtm);
		vtm.mode=VT_AUTO;
		ioctl(hr_ttyfd,VT_SETMODE,&vtm);
		ioctl(hr_ttyfd,VT_ACTIVATE,hr_original_vt);
	}
}


/* if we're not running directly on a console, try to find a free console
 * and move us to it. Notes old VT so we can switch back to it when finished,
 * if we ran on a different one to start off with.
 */
int hr_fixvt() {
   struct stat sbuf;
   struct vt_stat vts;
   int major,minor;
   int fd;
   int num;
   char vt_filename[20];
   /* see if terminal is a console */
   fd=dup(fileno(stderr));
   fstat(fd,&sbuf);
   major=sbuf.st_rdev>>8;
   hr_vt=minor=sbuf.st_rdev&0xff;
   close(fd);
	/* if on a console, already ok */
   if(major==4 && minor<64)
     return(1);
	/* otherwise we need to look for a free VT, redirect std{in,out,err},
	 * and switch to it. If there's no free VTs, give up now.*/
	hr_separate_vt=1;
	/* still root perms, so this shouldn't be a problem... */
	if((fd=open("/dev/console",O_RDONLY))<0) return(0);
	ioctl(fd,VT_GETSTATE,&vts);
	hr_original_vt=vts.v_active;
	ioctl(fd,VT_OPENQRY,&num);
	if(num==-1) return(0);  /* no VTs free */
	/* now, before we go for it, we test the *current* VT to see if RW access
	 * is allowed. If so, the user's probably 'genuine'.*/
	sprintf(vt_filename,"/dev/tty%d",hr_original_vt);
	stat(vt_filename,&sbuf);
	if(getuid()!=sbuf.st_uid)
	  printf("You must be the owner of the current console.\n");
	hr_vt=num;
	sprintf(vt_filename,"/dev/tty%d",num);
	setsid();
	/* "dontchangestdin"-check removed.*/
	if(freopen(vt_filename,"r",stdin)==NULL)
	  return(0);
	if(freopen(vt_filename,"w",stderr)==NULL)
	  return(0);
	/* we keep stdout - doesn't need to be changed */
	/* switch to the new VT */
	ioctl(fd,VT_ACTIVATE,num);
	close(fd);
	/* ok, done it. */
	return(1);
}


/* Blih blah. */
int hr_get_vconsole() {
	/* Get vc */
	if(hr_fixvt()==0){
		printf("Not running on console and no free VTs found.\n");
		return 0;
	}
	return 1;
}

#if 0
void sdfsdfdsf(){
	/* Nonblocking mode */
	hr_ttyfd=fileno(stdin);
	fcntl(hr_ttyfd,F_SETFL,O_NONBLOCK);
}
#endif
