/*
 *	Window functions. Slow slow slow slow mem hungry & slow...
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>

#include <hr_defines.h>
#include <hr_bobs.h>
#include <hr_mouse.h>
#include <hr_buttons.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_input_handler.h>
#include <hr_fonts.h>


struct hr_window_gadgets{
	int button_n;
	struct hr_button **buttons;
};


struct hr_window {
	struct hr_window *next;
	int flags;
	struct hr_color *colors;
	struct hr_bitmap *background;
	struct hr_bitmap *foreground;
	int x;
	int y;
	int width;
	int height;
	struct hr_window_gadgets *gadgets;
};



/* Returns pointer to frontmost (last) window in window list.
 * If there are no windows open, NULL is returned.*/
struct hr_window *hr_frontmost_window(){
	struct hr_window *wp;
	wp=hr.firstwindow;
	if(wp!=NULL){
		while(wp->next!=NULL)
		  wp=wp->next;
	}
	return wp;
}


/* Removes all windows from-frontmost-to-wp from screen.
 * This must be called before moving,
 * resizing or in any way modifying windows. It is ok to modify CONTENTS
 * of frontmost window tho.*/
void hr_get_windows(struct hr_window *wp){
	struct hr_window *t;
	int gotcha;
	/* In reverse order, "get" windows.*/
	if(wp!=NULL){
		while(!(wp->flags & WINDOW_STORED)){
			t=wp;
			gotcha=0;
			while((t->next!=NULL) && (!gotcha)){
				if(t->next->flags & WINDOW_STORED)
				  gotcha=1;
				t=t->next;
			}
			t->flags |= WINDOW_STORED;
			/* Save the window contents. */
			hr_delete_bitmap(t->foreground);
			t->foreground=hr_create_bitmap(malloc((t->width)*(t->height)),
													 NULL,(t->width),(t->height),
													 0,GFX_256COLOR);
			gl_getbox((t->x),(t->y),
						 (t->width),(t->height),(t->foreground->data));
			/* Restore gfx under window, free mem.*/
			if(t->background!=NULL){
				gl_putbox((t->x),(t->y),(t->width),(t->height),
							 (t->background->data));
				hr_delete_bitmap(t->background);
				t->background=NULL;
			}
		}
	}	
}
	

/* Puts all windows back to their places, frees foregrounds. Call this
 * After moving, resizing or otherwise modifying any non-frontmost
 * window.*/
void hr_put_windows(){
	struct hr_window *wp;
	wp=hr.firstwindow;
	while(wp!=NULL){
		if(wp->flags & WINDOW_STORED){
			wp->flags &= ~(WINDOW_STORED);
			/* Save the background */
			hr_delete_bitmap(wp->background);
			wp->background=hr_create_bitmap(malloc((wp->width)*(wp->height)),
													  NULL,(wp->width),(wp->height),
													  0,GFX_256COLOR);
			gl_getbox((wp->x),(wp->y),(wp->width),(wp->height),
						 (wp->background->data));
			/* Draw window contents and free foreground bitmap.
			 * It is ok for a window to be resized.*/
			if(wp->foreground!=NULL){
				if(((wp->width)==(wp->foreground->width))&&
					((wp->height)==(wp->foreground->height)))
				  gl_putbox((wp->x),(wp->y),
								(wp->width),(wp->height),(wp->foreground->data));
				else
				  gl_putboxpart((wp->x),(wp->y),
									 (wp->width),(wp->height),
									 (wp->foreground->width),
									 (wp->foreground->height),
									 (wp->foreground->data),0,0);
				hr_delete_bitmap(wp->foreground);
				wp->foreground=NULL;
			}
		}
		/* Neeeeeeext! */
		wp=wp->next;
	}
}


/* Redraws bevelbox, titlebac and stuff. Only works for frontmost window.*/
void hr_window_refresh(struct hr_window *wp){
	int width;
	int bflags,l;
	int x1=0,x2=0,y1=0,y2=0;
	struct hr_button *b;
	if(wp!=NULL){
		/* Remove windows on top of this one.*/
		if(wp!=hr_frontmost_window())
		  hr_get_windows(wp->next);
		/* Clear window area. */
		gl_fillbox((wp->x)+1,(wp->y)+1,
					  (wp->width)-2,(wp->height)-2,(wp->colors->bgr));
		if(!(wp->flags & WINDOW_NOBORDER)){
			/* Draw edge bevelbox.*/
			hr_bevelbox(wp->x,wp->y,
							(wp->x)+(wp->width)-1,(wp->y)+(wp->height)-1,
							wp->colors,NONE);
		}
		if(wp->flags & WINDOW_INNERBOX){
			x1=(wp->x)+2;
			x2=(wp->x)+(wp->width)-3;
			y1=(wp->y)+2;
			y2=(wp->y)+(wp->height)-3;
		}
		/* Set the window button positions and turn the buttons visible.*/
		if((wp->gadgets)!=NULL){
			if(wp->gadgets->button_n>0){
				width=(((wp->width)-3)/(wp->gadgets->button_n));
				/* Now put all the buttons in a row.*/
				for(l=0;l<(wp->gadgets->button_n);l++){
					b=(wp->gadgets->buttons)[l];
					if(b!=NULL) {
						b->x1=(wp->x)+2+l*width;
						b->x2=(wp->x)+2+(l+1)*width-2;
						b->y1=(wp->y)+(wp->height)-7-(b->font->height);
						if(wp->flags & WINDOW_INNERBOX){
							if((b->y1)<(y2+2))
							  y2=(b->y1)-2;
						}
						b->y2=(wp->y)+(wp->height)-3;
						/* Make the button visible */
						bflags=(b->flags &= ~(INVISIBLE));
						hr_update_button(0,b,UPDATE_FLAGS,&bflags);
					}
				}
			}
		}
		if(wp->flags & WINDOW_INNERBOX){
			/* Draw inner bevelbox.*/
			hr_bevelbox(x1,y1,x2,y2,wp->colors,PRESSED);
		}
		/* Restore windows on top of this one */
		hr_put_windows();
	}
}


/* Opens (creates) a window */
struct hr_window *hr_window_open(int x,int y,int w,int h,
											struct hr_color *colors,
											int flags)
{
	struct hr_window *wp;
	struct hr_button *b;
	/* Get rid of old default button, but set the flag
	 * so that it's easy to restore previous defbutton when the window
	 * is closed.*/
	if((b=hr.default_button)){
		b->flags |= OLD_DEFAULT_BUTTON;
		hr.default_button=NULL;
		hr_update_button(0,b,UPDATE_GFXONLY,NULL);
	}
	/* Create new window struct */
	wp=malloc(sizeof(struct hr_window));
	hr.active_window=wp;
	wp->flags=flags;
	wp->x=x;
	wp->y=y;
	wp->width=w;
	wp->height=h;
	wp->colors=colors;
	/* Nullify foreground pointer */
	wp->foreground=NULL;
	/* Save the background */
	wp->background=hr_create_bitmap(malloc(w*h),NULL,w,h,0,GFX_256COLOR);
	gl_getbox(x,y,w,h,wp->background->data);	
	/* Link buttons to window.*/
	wp->gadgets=NULL;
	/* Set some pointers.*/
	if(hr.firstwindow==NULL)
	  hr.firstwindow=wp;
	else
	  hr_frontmost_window()->next=wp;
	wp->next=NULL;
	/* Draw it.*/
	hr_window_refresh(wp);
	/* Awayyy......*/
	return wp;
}


/* Closes a window. Since possible window buttons were (hopefully!)
 * initialized as INVISIBLE, the background is redrawn correctly.*/
void hr_window_close(struct hr_window *wp) {
	struct hr_window *t;
	struct hr_button *b;
	int l;
	if(wp!=NULL) {
		/* Remove windows on top of wp.*/
		hr_get_windows(wp);
		/* Delete the window buttons. It is ok to remove NULL buttons.*/
		if(wp->gadgets!=NULL){
			for(l=0;l<(wp->gadgets->button_n);l++)
			  hr_remove_button((wp->gadgets->buttons)[l]);
			free(wp->gadgets->buttons);
			free(wp->gadgets);
		}
		/* it is ok to free/delete NULLs...*/
		hr_delete_bitmap(wp->background);
		hr_delete_bitmap(wp->foreground);
		/* Next-pointers must "jump over" *wp.*/
		if(wp==hr.firstwindow)
		  hr.firstwindow=wp->next;
		else{
			t=hr.firstwindow;
			while((t->next)!=wp)
			  t=t->next;
			t->next=(wp->next);
		}
		free(wp);
		/* Redraw windows on top of *wp if necessary */
		hr_put_windows();
	}
	/* Set active window */
	hr.active_window=hr_frontmost_window();
	/* Set default button. First appropriate button is selected
	 * (there should be only one). the OLD_DEFAULT_BUTTON-flag should
	 * ONLY be set within window_open().*/
	b=hr.firstbutton;
	while(b!=NULL){
		if((b->flags & OLD_DEFAULT_BUTTON) &&
			((b->parent_window)==hr.active_window) &&
			(b->flags & ENABLED) &&
			!(b->flags & INVISIBLE) &&
			!(b->flags & NO_TAB_CYCLE))
			{
				b->flags &= ~(OLD_DEFAULT_BUTTON);
				hr.default_button=b;
				hr_update_button(0,b,UPDATE_GFXONLY,NULL);
				break;
			}
		b=b->next;
	}
}


/* Moves a window */
void hr_window_move(struct hr_window *wp,int nx,int ny){
	if(wp!=NULL){
		hr_get_windows(wp);
		wp->x=nx;
		wp->y=ny;
		hr_put_windows();
	}
}

/* Resizes a window */
void hr_window_resize(struct hr_window *wp,int nwidth,int nheight){
	if(wp!=NULL){
		hr_get_windows(wp);
		wp->width=nwidth;
		wp->height=nheight;
		hr_put_windows();
	}
}


/* Moves a window to front.*/
void hr_window_to_front(struct hr_window *wp){
	struct hr_window *t;
	if((wp!=NULL) && (wp!=hr_frontmost_window())){
		hr_get_windows(wp);
		if(wp==hr.firstwindow)
		  hr.firstwindow=hr.firstwindow->next;
		else{
			t=hr.firstwindow;
			while(t->next!=wp)
			  t=t->next;
			t->next=(t->next->next);
		}
		hr_frontmost_window()->next=wp;
		wp->next=NULL;
		hr_put_windows();
	}
}


/* Moves a window to back */
void hr_window_to_back(struct hr_window *wp){
	struct hr_window *t;
	if((wp!=NULL)&&(wp!=hr.firstwindow)){
		t=hr.firstwindow;
		while(t->next!=wp)
		  t=t->next;
		t->next=t->next->next;
		wp->next=hr.firstwindow;
		hr.firstwindow=wp;
	}
}


/* Creates one window gadget struct and zeroes it.*/
struct hr_window_gadgets *hr_create_window_gadgets(int button_n){
	struct hr_window_gadgets *ggg;
	ggg=malloc(sizeof(struct hr_window_gadgets));
	ggg->button_n=button_n;
	if(button_n>0)
	  ggg->buttons=malloc(button_n*sizeof(struct hr_button *));
	else
	  ggg->buttons=NULL;
	return ggg;
}
