/*
 *	Tykkipeli 2
 * DESTRUCTZION!
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <math.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <weapons.h>
#include <guns.h>
#include <terrain.h>
#include <explode.h>
#include <shields.h>
#include <control_panel.h>
#include <info_windows.h>
#include <colors.h>
#include <weapon_defines.h>
#include <shield_defines.h>

#include <hr_defines.h>
#include <hr_buttons.h>
#include <hr_general.h>
#include <hr_keyboard.h>


struct bullet{
	struct bullet *next;
	int  flags;
	struct player  *owner;
	struct warhead *type;
	int  counter;
	long x;
	long y;
	long xs;
	long ys;
	int  color;
};

/* These two return the origin coordinates of a bullet that is to be
 * fired from gun barrel of some player.*/
#define BARREL_X(p) ( ((p)->x)+((pcos[(p)->angle]*(BARREL_LEN+2))>>SCALE) )
#define BARREL_Y(p) ( ((p)->y)-BARREL_OFFSET- \
                      ((psin[(p)->angle]*(BARREL_LEN+2))>>SCALE) )


/* Inits a bullet at _SCREEN_ coordinates x,y.
 * Speed xs,ys MUST be scaled up.
 * Returns a pointer to the bullet.
 * NOTE! No 0/0/MAX_X/MAX_Y-checking for boundaries is done, so it is ok
 * to init a bullet outside the visible area.
 */
struct bullet *init_bullet(struct player *p,
									struct warhead *w,
									int x,int y,int xs,int ys,
									int counter,int flags)
{
	struct bullet *b=firstbullet;
	if(b==NULL)
	  firstbullet=b=malloc(sizeof(struct bullet));
	else {
		while((b->next)!=NULL)
		  b=b->next;
		b->next=malloc(sizeof(struct bullet));
		b=b->next;
	}
	b->next=NULL;
	/* About 50% of bullets have BULLET_RANDOMFLAG set.*/
	b->flags=flags|((random()%2)?(BULLET_RANDOMFLAG):(NONE));
	b->owner=p;
	b->type=w;									/* Pointer to a warhead */
	if(flags & BULLET_EXPLODING){
		b->x=x;
		b->y=y;
	}
	else{
		b->x=(x<<SCALE);
		b->y=(y<<SCALE);
	}
	b->xs=xs;
	b->ys=ys;
	b->color=MINCOL_BULLET+(b->type->color);
	if(flags & FIRE_INSTANTANEOUS)
	  b->counter=0;
	else
	  b->counter=counter;
	return b;
};


/* Throws some particles up in the air... */
void init_particle(int x,int y) {
	int a,pwr;
	if((game_flags & PARTICLES) && (y<=MAX_Y)){
		a=45+random()%90;
		pwr=20+random()%300;
		init_bullet(NULL,&particle_s,
						x,y,(pcos[a]*pwr)>>(SCALE/2),
						-(psin[a]*pwr)>>(SCALE/2),0,
						ENABLED|BULLET_PENETRATES_SHIELDS|BULLET_DOES_NOT_FLASH);
	}
}


/* Disables all bullets */
void delete_all_bullets() {
	struct bullet *b=firstbullet,*ob;
	firstbullet=NULL;
	while(b!=NULL) {
		ob=b;
		b=b->next;
		free(ob);
	}
}


/* Deletes one bullet, frees the mem etc. */
void delete_bullet(struct bullet *b)  {
	struct bullet *b2=NULL;
	if(b!=NULL) {		
		if(b==firstbullet)
		  firstbullet=b->next;
		else  {		
			b2=firstbullet;
			while(b2->next != b)
			  b2=b2->next;
			b2->next=b->next;					/* Jump "over" b  */
		}
		free(b);
	}
}


/* This func does the work of calculating start position and
 * speed for each warhead in a weapon. The weapon that is fired
 * does not have to be the current active_weapon of player p.
 * IMPORTANT NOTE! If the bullet foes not fire immediately ,
 * the x/y positions WILL be recalculated in fly_bullets when
 * the bullet gets enabled. */
void fire(struct player *p,struct weapon *w,
			 int x,int y,int angle,int power,int flags)
{
	struct warhead *wh;
	struct warhead_list *whl;
	struct weapon_list *wl;
	int l,s,power2,heads;
	int angle2,avar;
	/* Decrement one weapon of selected type, IF selected
	 * weapon is not the default weapon. If a weapon that was not owned
	 * by player p was fired, return.
	 * Note! It is legal to fire a weapon owned by no-one,
	 * and even if a player fires a weapon, he/she must not necessarily
	 * have any :-) This allows a weapon to "contain" other weapons
	 * (for example, Spread Bomb contains a MegaShot).*/
	if((p!=NULL)&&
		!(flags & FIRE_NO_DECREMENT)){
		if((w->default_amount)!=-1) {
			wl=p->weapons;
			while((wl->type)!=(p->active_weapon))
			  wl=wl->next;
			if((wl->amount)>0)
			  --(wl->amount);
			else
			  /* Hmmm...? BUGZEN! Someone tried to use a weapon
				* that is not owned by player p.*/
			  /* Add some clever thing here :-) FIX */
			  return;
		}
	}
	/* Total $$$ wasted to ammo */
	ammo_money+=(w->cost);
	/* Init all bullets of every shot */
	for(s=0;s<(w->shots);s++) {
		avar=0;
		l=0;
		whl=(w->warheads)[s];
		heads=warheads_in_list(whl);
		while(whl!=NULL){
			wh=(whl->type);
			/* Angle Variance */
			if(((w->variance)[s])>0)
			  avar=(-((w->variance)[s])/2
					  +random()%((w->variance)[s]));
			if(heads>1)
			  angle2=avar+angle-(((w->spread)[s])/2)+(((w->spread)[s])/
																	(heads-1))*l;
			else
			  angle2=angle+avar;
			/* Make sure angle2 is on legal range [0-359].*/
			if(angle2<0)
			  angle2+=360;
			else if(angle2>359)
			  angle2-=360;
			/* Power. The heavier the bullet, the smaller the power.
			 * Weightless projectiles are always fired at preset power.*/
			if(wh->weight>0)
			  power2=(power-BULLET_WEIGHT_SDEC*(wh->weight));
			else
			  power2=SCALE*64;
			/* Note! If FIRE_INSTANTANEOUS flag is set,
			 * init_bullet() will set all speeds to zero.*/
			init_bullet(p,wh,
							x,y,
							(pcos[angle2]*power2)>>(SCALE/2),
							(-psin[angle2]*power2)>>(SCALE/2),
							-((w->speed)[s]),flags);
			l++;
			whl=whl->next;
		}
	}	
}


/* Fires the weapon of player p.
 * NOTE! weapon[].warheads contains string of (warhead_type+1).
 */
void fire_handler(struct player *p) {
	shots_fired++;								 /* Global counter */
	/* Fire the active weapon normally.*/
	fire(p,p->active_weapon,
		  0,0, /* No need to calc these, because FIRE_FROM_BARREL flag is set */
		  p->angle,p->power,FIRE_FROM_BARREL|ENABLED);
}


/* Move the bullets until they are all hit or disabled */
void fly_bullets() {
	struct bullet *b,*ob;
	struct player *p,*p2;
	int all_gone=0,edgetemp=edges,tc,update_panelbars;
	hr_clearkey();
	while((!all_gone) && !(game_flags & GAME_ABORTED)) {
		/* Watch keyboard..*/
		switch(hr_getkey()){
		 case 27:
			if(ask_confirmation("Abort game?"))
			  game_flags |= GAME_ABORTED;
			break;
		}
		all_gone=1;
		update_panelbars=0;
		++gfx_counter;
		vga_waitretrace();
		if(collapse())								/* Nice realtime collapsing... */
		  all_gone=0;
		if(drop_guns())							/* Nice realtime gun drop...   */
		  all_gone=0;
		color_fx();
		
		b=firstbullet;
		while(b!=NULL) {
			if(b->flags & ENABLED) {
				all_gone=0;
				/* Inc counter */
				/* Only do blah blah if the counter is at least equal to 1.*/
				if(++(b->counter)>0) {					
					/* Nice realtime explosions... */
					if(b->flags & BULLET_EXPLODING)
					  explode(b);
					/* Bullets.*/
					else
					  if(!(b->flags & BULLET_HIT))  {
						  /* If the bullet was fired from gun barrel, recalc
							* its starting position. This is done because
							* a gun may move between clicking Fire and the bullet
							* actually flying outta the barrel.*/
						  if(b->flags & FIRE_FROM_BARREL) {
							  /* Unset the flag so that this is done only once */
							  b->flags &= ~(FIRE_FROM_BARREL);
							  if(b->owner!=NULL){
								  b->x=(BARREL_X(b->owner)<<SCALE);
								  b->y=(BARREL_Y(b->owner)<<SCALE);
							  }
						  }
						  
						  /* Remove old bullet image if necessary.*/
						  if((b->counter>1) && (b->flags & BULLET_IS_DRAWN)){
							  b->flags &= ~(BULLET_IS_DRAWN);
							  if(game_flags & TRACERS)
								 /* Tracer dot */
								 gl_setpixel(((b->x)>>SCALE),((b->y)>>SCALE),
												 sky_color(((b->x)>>SCALE),
															  ((b->y)>>SCALE)+33));
							  else
								 gl_setpixel(((b->x)>>SCALE),((b->y)>>SCALE),
												 sky_color(((b->x)>>SCALE),
															  ((b->y)>>SCALE)));
						  }
						  
						  /* Draw flight effect, if any */
						  if(b->type->flight_effect)
							 flite_effect(b);
						  
						  /* Modify bullet speeds */
						  if(b->type->weight>0) {
							  /* Gravity */
							  if(b->flags & BULLET_DIGMODE) {
								  /* Update tertemp[]-array */
								  quick_lower(((b->x)>>SCALE),((b->y)>>SCALE));
								  /* Bounce towards earth */
								  (b->ys)-=(g*(b->type->weight))<<2;
							  }
							  else {
								  /* Accelerate in normal gravity direction */
								  (b->ys)+=(g*(b->type->weight))<<2;
								  /* Wind only has effect in airborne */
								  (b->xs)+=(wind*64)/(b->type->weight);
							  }
						  }
						  
						  /* Move the bullet */
						  (b->x)+=(b->xs);
						  (b->y)+=(b->ys);
						  
						  /* Path-specific stuff. */
						  switch(b->type->path) {
#if 0
							case PATH_NORMAL:
							  /* Do nothing special.
								* This is used by "normal" bullets and phaser
								* tails and such.*/
							  break;
#endif
							case PATH_SUICIDE:
							  /* These explode in the middle of shooting gun */
							  if(b->owner!=NULL){
								  b->x=((b->owner->x)<<SCALE);
								  b->y=(((b->owner->y)-BARREL_OFFSET)<<SCALE);
							  }
							  b->flags |= BULLET_HIT;
							  break;
							case PATH_PHASER:
							  /* Phaser. Phaser bullet creates a "tail" for it.
								* Since the counter of tail particles is 15,
								* this does not create infinite number of bullets.*/
							  if(b->owner!=NULL){
								  if((b->owner->flags & ALIVE) &&
									  ((b->counter)>1) &&
									  ((b->counter)<(b->type->trigger_step)))
									 init_bullet(b->owner,b->type,
													 BARREL_X(b->owner),BARREL_Y(b->owner),
													 b->xs,b->ys,
													 (b->type->trigger_step)+1,ENABLED);
							  }
							  break;
							case PATH_DROP:
							  /* "Drops" when y speed is small enough.*/
							  if(((b->ys)>0) && !(b->flags & BULLET_FLAG1)) {
								  b->flags |= BULLET_FLAG1;
								  b->xs=0;
							  }
							  break;
							case PATH_SEEKDROP:
							  /* blaaah FIX SLOW! */
							  p=firstplayer;
							  while(p!=NULL) {
								  if((p!=(b->owner)) &&
									  !(b->flags & BULLET_FLAG1) &&
									  (abs(((b->x)>>SCALE)-(p->x))<5)) {
										  b->flags |= BULLET_FLAG1;
										  b->xs=0;
									  }
								  p=p->next;
							  }
							  break;
							case PATH_FIREWORKS:
							  /* Activates when y speed goes positive.*/
							  if((b->ys)>0)
								 b->flags |= BULLET_HIT;
							  break;
							case PATH_DIGGER:
							  /* Digger bullet surfaced? */
							  if((b->flags & BULLET_DIGMODE) &&
								  (pixel_type(PIX_SKY,
												  vga_getpixel(((b->x)>>SCALE),
																	((b->y)>>SCALE)))))
							  	 b->flags |= BULLET_HIT;
							  break;
							case PATH_CRUISER:
							  /* Cruiser. Stays on top of terrain.*/
							  if(pixel_type(PIX_GROUND,
												 vga_getpixel(((b->x)>>SCALE),
																  ((b->y)>>SCALE)))||
								  (((b->y)>>SCALE)>MAX_Y))
								 {
									 b->y=((terrain[((b->x)>>SCALE)]-1)<<SCALE);
									 b->ys=0;
								 }
							  break;
							case PATH_BOUNCER:
							  /* Bouncer. Bounces on top of terrain.*/
							  if(pixel_type(PIX_GROUND,
												 vga_getpixel(((b->x)>>SCALE),
																  ((b->y)>>SCALE)))||
								  (((b->y)>>SCALE)>MAX_Y))
								 {
									 b->y=((terrain[((b->x)>>SCALE)]-1)<<SCALE);
									 b->ys=-(b->ys);
								 }
							  break;
						  }
						  
						  /* Flew outta screen horizontally?
							* If edge-mode is non-standard, blah blah.*/
						  if((b->x)<0 || ((b->x)>>SCALE)>MAX_X )  {
							  if(edges==EDGES_RANDOM)
							  	 edgetemp=random()%MAX_EDGEMODE;
							  switch(edgetemp) {
								case EDGES_NORMAL:
								  /*Normal*/
								  b->flags &= ~(ENABLED);
								  break;
								case EDGES_WRAP:
								  /*Wrap*/
								  if(b->flags & BULLET_EDGEFLAG)
									 b->flags &= ~(ENABLED);
								  if(((b->x)>>SCALE)>MAX_X)
								  	 (b->x)=0;
								  else
								  	 (b->x)=(MAX_X<<SCALE);
								  b->flags |= BULLET_EDGEFLAG;
								  /* Collided? */
								  tc=vga_getpixel(((b->x)>>SCALE),((b->y)>>SCALE));
								  if(pixel_type(PIX_GROUND,tc) ||
									  is_gun(((b->x)>>SCALE),((b->y)>>SCALE)))
									 b->flags |= BULLET_HIT;
								  break;
								case EDGES_BOUNCY:
								  /*Bouncy*/
								  if(b->flags & BULLET_EDGEFLAG)
									 b->flags &= ~(ENABLED);
								  (b->xs)=-(b->xs);
								  b->flags |= BULLET_EDGEFLAG;
								  /* Collided? */
								  tc=vga_getpixel(((b->x)>>SCALE),((b->y)>>SCALE));
								  if(pixel_type(PIX_GROUND,tc) ||
									  is_gun(((b->x)>>SCALE),((b->y)>>SCALE)))
									 b->flags |= BULLET_HIT;
								  break;
							  }
						  }
						  
						  /* Flew too high? This must be checked - otherwise
							* phasers etc would cause quite long delays :-) */
						  else if(((b->y)>>SCALE)<ORBIT_Y )
						  	 b->flags &= ~(ENABLED);
						  
						  /* Dug/flew too deep? */
						  else if(((b->y)>>SCALE)>MAX_Y)
						  	 b->flags |= BULLET_HIT;
						  
						  /* Check collision between bullet and ground */
						  else if(pixel_type(PIX_GROUND,
													vga_getpixel(((b->x)>>SCALE),
																	 ((b->y)>>SCALE))) ||
									 (((b->y)>>SCALE)>MAX_Y)) {
							  switch(b->type->path) {
								case PATH_DIGGER:
								  /* Digger hit ground? Enable digging mode!*/
								  b->flags |= BULLET_DIGMODE;
								  break;
								default:
								  /* Normal bullet collided with ground? */
								  b->flags |= BULLET_HIT;
								  break;
							  }
						  }
						  
						  /* Collided with shield?
							* A player cannot shoot his/her own shield.
							* Note! is_shield() only returns non-NULL
							* if the shield is not all used up.*/
						  else if((p2=is_shield(((b->x)>>SCALE),((b->y)>>SCALE))) &&
									 (p2!=(b->owner))){
							  update_panelbars=shield_effect(p2,b);
						  }
						  
						  /* Collided with gun??? */
						  else if(is_gun(((b->x)>>SCALE),((b->y)>>SCALE)))
						  	 b->flags |= BULLET_HIT;
						  
						  /* If the bullet was marked to be HIT, init an explosion
							* and zero the counter.*/
						  if(b->flags & BULLET_HIT)
							 init_explosion(b);
						  else {
							  /* Draw the bullet.*/
							  if((b->y)>=0){
								  gl_setpixel(((b->x)>>SCALE),((b->y)>>SCALE),
												  b->color);
								  b->flags |= BULLET_IS_DRAWN;
							  }
						  }
					  }
				  }
			}
			ob=b;
			b=b->next;
			/* Delete bullet if it's useless. */
			if(!(ob->flags & ENABLED)) 
			  delete_bullet(ob);
		}
		/* If any bullet hit a shield, update panel bars.*/
		if(update_panelbars)
		  panel_bars();
	}
}
