/*
 *	Tykkipeli 2
 *	Explosions.
 */
  
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <bullets.h>
#include <weapons.h>
#include <terrain.h>
#include <guns.h>
#include <control_panel.h>
#include <explosion_types.h>
#include <weapon_defines.h>
#include <shield_defines.h>
#include <colors.h>

#include <hr_defines.h>
#include <hr_bobs.h>
#include <hr_general.h>


int flash_counter;
struct{
	int r;
	int g;
	int b;
}explosion_color[MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1];


/* Draws a dot and gives damage. */
void dot(int x,int y,int color,struct bullet *e) {
	struct player *p;
	gl_setpixel(x,y,color);
	if(e!=NULL){
		if((p=is_gun(x,y))){
			hurt(p,e->owner,e->type->efficiency,DAMAGE_DOT);
		}
	}
}


/* Quick hack to minimize need of vga_getpixel() when removing
 * balls_of_fire from screen.*/
#define CIRCLE_STEP 3
void explo_circle_remove(int x,int y,int r){
	int a,xp,yp,div;	
	div=(1<<SCALE)/r;
	for(a=0;a<=45;a+=CIRCLE_STEP){
		xp=pcos[a]/div;
		yp=psin[a]/div;
		gl_setpixel(x+xp,y+yp,sky_color(x+xp,y+yp));
		gl_setpixel(x-xp,y+yp,sky_color(x-xp,y+yp));
		gl_setpixel(x+xp,y-yp,sky_color(x+xp,y-yp));
		gl_setpixel(x-xp,y-yp,sky_color(x-xp,y-yp));
		quick_lower(x+xp,y+yp);
		quick_lower(x-xp,y+yp);
		/* Swap x/y */
		gl_setpixel(x+yp,y+xp,sky_color(x+yp,y+xp));
		gl_setpixel(x-yp,y+xp,sky_color(x-yp,y+xp));
		gl_setpixel(x+yp,y-xp,sky_color(x+yp,y-xp));
		gl_setpixel(x-yp,y-xp,sky_color(x-yp,y-xp));
		quick_lower(x+yp,y+xp);
		quick_lower(x-yp,y+xp);
	}
}


/* Draws a nice expanding round explosion */
int ball_of_fire(struct bullet *e) {
	int xx,yy,distance;
	struct player *p;	
	/* The initial dot and particles */
	if(e->counter == 1)  {
		gl_setpixel(e->x,e->y,sky_color(e->x,e->y));
		xx=random()%(e->type->xrange);
		for(yy=0;yy<xx;yy++)
		  init_particle((e->x)-10+random()%21,
							 (e->y)-random()%10);		
	}
	/* Draw one circle of explosion */
	if((e->counter)<=(e->type->xrange))
	  hr_circle(e->x,e->y,e->counter,
					MINCOL_EXPLOSION+((e->counter)%
											(MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1)));
	/* Draw one circle of sky...*/
	if((e->counter)>EXPLO_THICKNESS)  {
		distance=(e->counter)-EXPLO_THICKNESS;
		explo_circle_remove(e->x,e->y,distance);
	}
	/* Give damage if explosion is over. */
	if(e->counter >= (e->type->xrange+EXPLO_THICKNESS))  {		
		p=firstplayer;
		while(p!=NULL) {
			distance=DISTANCE(e->x,e->y,p->x,p->y);
			if(distance<=(e->type->xrange))  {
				hurt(p,e->owner,
					  ((e->type->efficiency)*
						(e->type->xrange)-distance)/(e->type->xrange),
					  DAMAGE_EXPLO);
			}
			p=p->next;
		}
		return 1;
	}
	return 0;
}


void explode(struct bullet *e){
	int xx,yy,zz=0,x1,x2,y1,y2;
	int gx1,gx2;
	struct player *p;

	if(e!=NULL) {
		/* Only check bullets that are in exploding phase.*/
		if((e->flags & ENABLED) && (e->flags & BULLET_EXPLODING)) {
			
			/* Remove gunzen, if necessary */
			gx1=(e->x)-(e->type->xrange);
			gx2=(e->x)+(e->type->xrange);
			update_guns_on_xrange(gx1,gx2,BOB_REMOVE,NONE);
		
			/* Ok, now do the explotrick.*/
			switch(e->type->explosion_type) {
			 case EXPLO_PARTICLE:
				/* A particle hits ground
				 * or smoke vanishes up the air.*/
				if(pixel_type(PIX_GROUND,
								  vga_getpixel((e->x),(e->y))))
				  /* Add some gray stuff and do nothing else.
					* Looks a bit like ash.*/
				  gl_setpixel((e->x),(e->y),128);
				else
				  /* Replace sky.*/
				  gl_setpixel((e->x),(e->y),sky_color(e->x,e->y));
				e->flags &= ~(ENABLED);
				break;
			 case EXPLO_ROUND:
				/* Normal round explosions */
				/* If the explosion is over, give damage and waste the explosion.*/
				if(ball_of_fire(e))
				  e->flags &= ~(ENABLED);
				break;
			 case EXPLO_SPREAD:
				/* Spread bomb. First init a small expanding fireball,
				 * then fire a MegaShot(tm).*/
				if(ball_of_fire(e))
				  e->flags &= ~(ENABLED);
				if(e->counter==1)
				  fire((e->owner),get_weapon(CHARS2INT("Mega")),
						 (e->x),(e->y)-2,70+random()%40,
						 555+random()%222,ENABLED|FIRE_NO_DECREMENT);
				break;
			 case EXPLO_FIREWORKS:
				/* Create some copies.*/
				if(ball_of_fire(e))
				  e->flags &= ~(ENABLED);
				else if((e->counter)==1){
					for(xx=0;xx<360;xx+=30)
					  fire(e->owner,get_weapon(CHARS2INT("Shot")),
							 e->x,e->y,xx,
							 MAX_POWER,ENABLED|FIRE_NO_DECREMENT);
				}
				break;
			 case EXPLO_VAPO:
				/* VapoBeam(tm) */
				/* Width and y offset */
				zz=random()%((e->type->xrange)+1);
				y1=(e->y)-(e->counter)+1;
				y2=(e->y)+(e->counter)-1;
				x1=(e->x)-zz;
				x2=(e->x)+zz;
				/* Draw*/
				if(y1>=((e->y)-(e->type->yrange))){
					gl_hline(x1,y1,x2,MINCOL_EXPLOSION);   /* FIX COLOR */
					gl_hline(x1,y2,x2,MINCOL_EXPLOSION);   /* FIX COLOR */
					for(xx=x1;xx<=x2;xx++)
					  quick_lower(xx,y2);
				}
				else
				  e->flags &= ~(ENABLED);
				/* Remove "previous" explosion lines.*/
				if((e->counter)>1){
					replace_with_sky((e->x)-(e->type->xrange),y1+1,
										  (e->x)+(e->type->xrange),y1+1,
										  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
					replace_with_sky((e->x)-(e->type->xrange),y2-1,
										  (e->x)+(e->type->xrange),y2-1,
										  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
				}
				break;
			 case EXPLO_ENBANK:
#define ENBWDTH 10
#define TERRAIN_COLORS (MAXCOL_GROUND-MINCOL_GROUND+1)
				/* Builds walls around explo-x,y */
				/* First yy is above the explosion y coordinate */
				yy=(e->y)-(e->counter)+1;
				if(yy>=((e->y)-(e->type->yrange))){
					for(zz=0;zz<=1;zz++){
						for(x2=0;x2<ENBWDTH;x2++){
							for(x1=0;x1<=1;x1++){
								if(x1)
								  xx=(e->x)-(e->type->xrange)+x2;
								else
								  xx=(e->x)+(e->type->xrange)-x2;
								if(pixel_type(PIX_SKY,vga_getpixel(xx,yy))){
									dot(xx,yy,
										 MINCOL_GROUND+random()%TERRAIN_COLORS,e);
									if(zz)
									  quick_lower(xx,MAX_Y);
									else
									  quick_raise(xx,yy);
								}
							}
						}
						/* Now yy is below explo-y */
						yy=(e->y)+(e->counter)-1;
					}
				}
				else
				  e->flags &= ~(ENABLED);
				break;
			 case EXPLO_EXCAVATE:
				/* Digs a moat arond the gun */
				/* Width and y offset */
				y1=(e->y)-(e->counter)+1;
				y2=(e->y)+(e->counter)-1;
				x1=(e->x)-(e->type->xrange);
				x2=(e->x)+(e->type->xrange);
				/* Draw*/
#define EXCWDTH 10
				if(y1>=((e->y)-(e->type->yrange))){
					gl_hline(x1,y1,x1+EXCWDTH,MINCOL_EXPLOSION);   /* FIX COLOR */
					gl_hline(x2-EXCWDTH,y1,x2,MINCOL_EXPLOSION);   /* FIX COLOR */
					gl_hline(x1,y2,x1+EXCWDTH,MINCOL_EXPLOSION);   /* FIX COLOR */
					gl_hline(x2-EXCWDTH,y2,x2,MINCOL_EXPLOSION);   /* FIX COLOR */
					temp_lower(x1,x1+EXCWDTH,y2);
					temp_lower(x2-EXCWDTH,x2,y2);
				}
				else
				  e->flags &= ~(ENABLED);
				/* Remove "previous" explosion lines.*/
				if((e->counter)>1){
					/* Left*/
					replace_with_sky(x1,y1+1,
										  x1+EXCWDTH,y1+1,
										  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
					replace_with_sky(x2-EXCWDTH,y1+1,
										  x2,y1+1,
										  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
					/* Right*/
					replace_with_sky(x1,y2-1,
										  x1+EXCWDTH,y2-1,
										  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
					replace_with_sky(x2-EXCWDTH,y2-1,
										  x2,y2-1,
										  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
}
				break;
			 case EXPLO_MOLE:
				dot(e->x,e->y,sky_color(e->x,e->y),e);
				quick_lower(e->x,e->y);
				if((e->counter)<(e->type->xrange)){
					(e->x)+= 1-random()%3;
					(e->y)+= 1-random()%3;
					gl_setpixel(e->x,e->y,
									(MINCOL_EXPLOSION+
									 random()%(MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1)));
				}
				else
				  e->flags &= ~(ENABLED);
				break;
#if 0
				/* Some Amos basic source */
			 case 6:
				/* Mkiboxi */
				/*       Z=0 : S=0 */
				/*       Repeat  */
				/*          Inc Z */
				/*          If Ma Point(X-Z,Y)=0 and X-Z>=0 */
				/*             S=-1 */
				/*          Else If Ma Point(X+Z,Y)=0 and X+Z<640 */
				/*             S=1 */
				/*          End If  */
				/*       Until S<>0 or Z=RANGE */
				/*       _LYD_SYVIN[X,X+RANGE*S] : Y1=Param */
				/*       If S<>0 */
				/*          For XX=X To X+RANGE*S Step S */
				/*             For YY=Y To Y+YRANGE */
				/*                If Ma Point(XX,YY)=0 */
				/*                   _DOT[XX,YY,4,EFFICIENCY] */
				/*                End If  */
				/*             Next  */
				/*          Next  */
				/*       End If  */
				break;
			 case 7:
				/* Cone of CRAP */
				/*       For YY=Y To Y-YRANGE Step -1 */
				/*          XD=((Y-YY)*RANGE)/YRANGE */
				/*          For XX=X-XD To X+XD Step 2 */
				/*             If Ma Point(XX,YY)=0 */
				/*                _DOT[XX,YY,4,EFFICIENCY] */
				/*             End If  */
				/*          Next  */
				/*       Next  */
				break;
			 case 9:
				/* Napalm */
				/*       For Z=0 To 1 */
				/*          For XX=1 To RANGE-YRANGE */
				/*             Wait Vbl  */
				/*             For ZZ=-XX To XX Step 2*XX */
				/*                YY=Peek(A1+X+ZZ)+1 */
				/*                If Z=0 */
				/*                   _DOT[X+ZZ,YY,7,EFFICIENCY/4] */
				/*                Else  */
				/*                   _PALLO[X+ZZ,YY,YRANGE,EFFICIENCY] */
				/*                End If  */
				/*             Next  */
				/*          Next  */
				/*       Next  */
				break;
			 case 10:
				/* Cruiser Mine */
				/*       Y1=Peek(A1+X) */
				/*       For YY=Y1-YRANGE To Y1 */
				/*          For XX=X-RANGE To X+RANGE */
				/*             _DOT[XX,YY,5,EFFICIENCY] */
				/*          Next  */
				/*       Next  */
				break;
			 case 11:
				/* Teleport the gun if the new location is ok */
				/*       _TYKIN_PAIKKA_CHECK[PELAAJA,X] */
				/*       If Param */
				/*          _TYKKI_TELEPORT[PELAAJA,-1] */
				/*          Amreg(1+PELAAJA)=X */
				/*          _TYKKI_TELEPORT[PELAAJA,1] */
				/*       End If  */
				break;
#endif
			 case EXPLO_NUKE:
				/* Total and instant massacre.*/
				if((e->counter)==1){
					/* Draw a filled circle in random explosion color.*/
					zz=MINCOL_EXPLOSION+
					  random()%(MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1);
					/* x^2+y^2==r^2 ... */
					gl_hline((e->x)-(e->type->xrange),
								e->y,
								(e->x)+(e->type->xrange),zz);
				}
				else
				  replace_with_sky((e->x)-(e->type->xrange),e->y,
										 (e->x)+(e->type->xrange),e->y,
										 MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
				for(yy=1;yy<=(e->type->yrange);yy++){
					xx=(int)(sqrt((double)(((e->type->xrange)*(e->type->xrange))
												  -((yy)*(yy)))));
					if((e->counter)==1){
						gl_hline((e->x)-xx,(e->y)-yy,(e->x)+xx,zz);
						gl_hline((e->x)-xx,(e->y)+yy,(e->x)+xx,zz);
						init_particle((e->x)-xx,(e->y)-yy);
						init_particle((e->x)+xx,(e->y)-yy);
						init_particle((e->x)-xx,(e->y)+yy);
						init_particle((e->x)+xx,(e->y)+yy);
					}
					else{
						replace_with_sky((e->x)-xx,(e->y)-yy,
											  (e->x)+xx,(e->y)-yy,
											  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
						replace_with_sky((e->x)-xx,(e->y)+yy,
											  (e->x)+xx,(e->y)+yy,
											  MINCOL_EXPLOSION,MAXCOL_EXPLOSION);
					}
				}
				if((e->counter)>1){
					temp_lower((e->x)-(e->type->xrange),(e->x)+(e->type->xrange),
								  (e->y)+yy);
					/* Give the suckers FULL damage if they are within
					 * range! */
					p=firstplayer;
					while(p!=NULL) {
						if(DISTANCE(e->x,e->y,p->x,p->y)<=(e->type->xrange)) 
						  hurt(p,e->owner,e->type->efficiency,DAMAGE_NUKE);
						p=p->next;
					}
					e->flags &= ~(ENABLED);
				}
				break;
			 case EXPLO_LAVA:
				/* Lava  */
				dot(e->x,e->y,MAXCOL_GROUND-random()%10,e);
				quick_lower((e->x),MAX_Y);
				quick_raise((e->x),(e->y));
				/* Down if pixel below current coords is empty */
				if(!(pixel_type(PIX_GROUND,
									 vga_getpixel(e->x,(e->y)+1))) &&
					((e->y)<MAX_Y))
				  ++(e->y);
				/* Check left or right */
				else if((!(pixel_type(PIX_GROUND,
											 vga_getpixel((e->x)-1+
															  (gfx_counter%2)*2,
															  (e->y))))))
				  e->x=(e->x)-1+(gfx_counter%2)*2;
				/* Go up. */
				else
				  --(e->y);
				/* Stop if there are enough pixels on screen already. */
				if((e->counter)>=100)
				  e->flags &= ~(ENABLED);
				break;
			}
			
			/* Re-init gun(s) that were removed. */
			update_guns_on_xrange(gx1,gx2,BOB_INIT,NONE);
		}
	}
}


/* Turns a bullet into an explosion*/
void init_explosion(struct bullet *b){
	if(b!=NULL){
		b->flags |= BULLET_EXPLODING;
		/* Since explosions do not need to use scaled
		 * coordinates, the coords are scaled back here.
		 * Now it's easier to make moles and stuff :-) */
		b->x=((b->x)>>SCALE);
		b->y=((b->y)>>SCALE);
		/* Reset explosion counter */
		b->counter=0;
		/* Flash color 0 if necessary */
		if((game_flags & EXPLO_FLASH) &&
			(!(b->flags & BULLET_DOES_NOT_FLASH))){
			gl_setpalettecolor(0,63,63,63);
			flash_counter=59;
		}
	}
}


/* Some crap */
/* This doesn't add any visible effect...(?)*/
#define EC (MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1)
void color_fx(){
	int c,l;
	if(game_flags & EXPLO_COLROT){
		/* Rotate explosion colors. NO VISIBLE DIFFERENCE???...FIX! */
		for(c=0;c<EC;c++){
			l=(gfx_counter+c)%EC;
			gl_setpalettecolor(MINCOL_EXPLOSION+c,
									 explosion_color[l].r,
									 explosion_color[l].g,
									 explosion_color[l].b);
		}
	}
	/* FIX ! */
	/* Unnecessary check (at the moment) */
	if(game_flags & EXPLO_FLASH){
		if(flash_counter>19){
			flash_counter-=2;
			gl_setpalettecolor(0,flash_counter,flash_counter,flash_counter);
			/* color 0 originally set in hr_general.c. Should be left
			 * to (15,15,15) */
		}
	}
}


void flite_effect(struct bullet *b){
	if(b!=NULL){
		switch(b->type->flight_effect){
		 case FLIGHT_PARTICLERAIN:
			/* Fire fire fire rain */
			if((((b->x)>>SCALE)+((b->y)>>SCALE))%
				(b->type->trigger_step)==0)
			  INIT_EXPLOSION(b->owner,b->type,
								  ((b->x)>>SCALE),((b->y)>>SCALE),
								  0,0,
								  0,ENABLED|BULLET_EXPLODING);
			break;
		 case FLIGHT_FLOWER:
			/* Fires small shots clockwise/counter-clockwise. */
			if((b->counter)>=(b->type->trigger_step)){
				if((b->counter)%2)
				  fire((b->owner),get_weapon(CHARS2INT("shot")),
						 ((b->x)>>SCALE),((b->y)>>SCALE),
						 ((b->flags & BULLET_RANDOMFLAG)?
						  (((b->counter)*3)%360):(360-((b->counter)*3)%360)),
						 300+random()%200,ENABLED|FIRE_NO_DECREMENT);
			}
			break;
		 case FLIGHT_SHOWER:
			/* Fires small shots upwards.  */
			if((b->counter)>=(b->type->trigger_step)){
				if((b->counter)%2)
				  fire((b->owner),get_weapon(CHARS2INT("shot")),
						 ((b->x)>>SCALE),((b->y)>>SCALE),
						 80+random()%20,
						 50+random()%150,ENABLED|FIRE_NO_DECREMENT);
			}
			break;
		}
	}
}
