/*
 *	Tykkipeli 2
 *
 *	Initialization stuff &
 *	Exit & cleanup stuff...
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <weapons.h>
#include <shields.h>
#include <bullets.h>
#include <explode.h>
#include <outro.h>
#include <info_windows.h>
#include <artificial_stupidity.h>
#include <terrain.h>
#include <control_panel.h>
#include <guns.h>
#include <change_item.h>
#include <disasters.h>

#include <hr_defines.h>
#include <hr_fonts.h>
#include <hr_zones.h>
#include <hr_bobs.h>
#include <hr_buttons.h>
#include <hr_mouse.h>
#include <hr_windows.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_input_handler.h>
#include <hr_ilbm.h>
#include <hr_keyboard.h>
#include <hr_screens.h>


/* Executed when the game is run.*/
/* Screw error surviving :-) */
void startup_stuff() {	
	int a,x=0;
	FILE *fp;
	
	/* Enable automatic mouse setup at mode set. */
	vga_setmousesupport(1);
	
	/* Datapath for Tykkipeli2 specific files.*/
	if(!(tp2_datapath=getenv("TP2_DATAPATH")))
	  printf("TP2_DATAPATH env variable not found.\n");
	/* Nullify some pointers */
	firstplayer=NULL;
	firstweapon=NULL;
	firstwarhead=NULL;
	firstshield=NULL;
	firstbullet=NULL;
	/* These are zeroed here so that create_shields/warheads/whatever
	 * can be used to ADD stuff to existing lists.*/
	weapon_n=0;
	shield_n=0;
	warhead_n=0;
	
	sprintf(buffer,"%s/pixmaps/%s",tp2_datapath,LOGOFILE);
	logo=hr_read_ilbm(buffer,GFX_256COLOR);
	hr_recolor_bitmap(logo,127,NONE);
	/* Create the crosshairs bob for AimAid(tm)*/
	sprintf(buffer,"%s/pixmaps/%s",tp2_datapath,CROSSHAIRS_BITMAP);
	crosshairs=hr_create_bob(0,0,0,6,6,
									 hr_read_ilbm(buffer,GFX_256COLOR),NONE);
	/* Read option screen scrolltext.*/
	sprintf(buffer,"%s/text/%s",tp2_datapath,INSTRUCTIONS);
	if((fp=fopen(buffer,"r"))){
		fseek(fp,0,SEEK_END);
		x=(int)ftell(fp);
		instructions=malloc(x+1);
		fseek(fp,0,0);
		fread(instructions,x,1,fp);
		fclose(fp);
		/* Convert linefeeds etc into spaces */
		for(a=0;a<x;a++){
			if(*(instructions+a)<32)
			  *(instructions+a)=32;
		}
		/* Make sure end-of-string is 0.*/
		*(instructions+x)=0;
	}
	/* Read the "oneliner" messages.*/
	sprintf(buffer,"%s/text/%s",tp2_datapath,ONELINERS);
	oneliner_n=0;
	oneliners=NULL;
	if((fp=fopen(buffer,"r"))) {
		/* Read lines.*/
		while(fgets(buffer,BUFFER_SIZE,fp)){
			++oneliner_n;
			oneliners=realloc(oneliners,sizeof(unsigned char *)*oneliner_n);
			*(oneliners+(oneliner_n-1))=malloc(strlen(buffer)+1);
			strcpy(*(oneliners+(oneliner_n-1)),buffer);
		}
		fclose(fp);
	}
	/* Set screen grabber hotkey */
	hr.def_hotkey=GRAB_KEY;
	hr.def_func=&GRAB_FUNC;
	/* Horrible kludge.*/
	temp_colors=malloc(sizeof(struct hr_color));
	memcpy(temp_colors,hr.def_colors,sizeof(struct hr_color));
#ifdef SOUND_SUPPORT
	init_sounds();
#endif
	create_warheads();
	create_weapons();
	create_special_ws();
	create_shields();
	init_roster();
	init_droids();
}


/* Executed when the game quits */
void exit_stuff() {	
#ifdef SOUND_SUPPORT
	close(audio);
#endif
	if(!(game_flags & QUIET))
	  printf("So many assholes... so few bullets.\n");
}


/* This is called before a game starts.
 * player_n (number of players in game) is calculated here.*/
void pre_game_inits() {
	int r;
	/* Scan all pressed roster buttons! NOTE! Colors are used in order...
	 * So choose the first gun colors well :-) */
	for(r=0;r<ROSTER_SIZE;r++) {
		if(roster[r].flags & PRESSED)
		  create_player(r,roster[r].name,
							 roster[r].bitmap,roster[r].deadmap,ALIVE);
	}
	/* Now create computer players.*/
	for(r=0;r<droids;r++) 
	  create_player(-1,droid_name,droid_bitmap,droid_deadmap,ALIVE|DROID);
}


/* Called after game */
void post_game_stuff() {
	struct player *p,*winner=NULL;
	int r,g,b,x;
	/* Free some stuff.. Most of these are probably freed already...*/
	delete_all_bullets();
	/* Do NOT change any roster data if the game was aborted! */
	if(!(game_flags & GAME_ABORTED)){
		/* Let's find the winner and while we're at it, update some roster data,
		 * but only if the player is human.*/
		p=firstplayer;x=-6666;
		while(p!=NULL) {
			if((p->rounds_won)>x) {
				x=p->rounds_won;
				winner=p;
			}
			else if((p->rounds_won)==x)
			  winner=NULL;
			if(!(p->flags & DROID)) {
				roster[p->rosterindex].games++;
				roster[p->rosterindex].deaths  +=(p->deaths);
				roster[p->rosterindex].kills   +=(p->kills);
				roster[p->rosterindex].suicides+=(p->suicides);
			}
			p=p->next;
		}
		/* If there are equivalent rounds_won values, the player who
		 * killed most and died least wins.*/
		if(winner==NULL)
		  winner=one_with_best_ratio();
		/* Show the outro and update roster data.*/
		if(winner!=NULL) {		
			if(!(winner->flags & DROID))
			  roster[winner->rosterindex].won++;
			gl_getpalettecolor(winner->firstcolor,&r,&g,&b);
			outro(winner->name,r/2,g/2,b/2);
		}
		else
		  outro(NULL,10,10,10);
		/* Enable saving of roster again */
		game_flags &= ~(ROSTER_SAVED);
	}
	/* Zero the abort-flag...*/
	game_flags &= ~(GAME_ABORTED);
	/* Free the mem used by playerz....*/
	delete_all_players();
};


/* Called after round */
void post_round_stuff() {
	struct player *p,*winner=NULL;
	/* Free some mem*/
	hr_delete_all_zones();
	hr_delete_all_buttons();
	/**/
	if(!(game_flags & GAME_ABORTED)){
		/* Give interest to money in bank :-)
		 * If there is a player alive, [s]he is the winner of the round.*/
		p=firstplayer;
		while(p!=NULL) {
			if((p->wealth)<=0)
			  p->wealth=0;
			else
			  (p->wealth)+=((p->wealth)*interest)/100;
			/* There can only be one :-) */
			if((p->flags) & ALIVE)
			  winner=p;
			p=p->next;
		}
		/* If no-one survived, the one with most favorable kill count in current
		 * game is the winner. Note! Amount of suicides is taken into account.*/
		if(winner==NULL)
		  winner=one_with_best_ratio();
		/* Give price money */
		p=firstplayer;
		while(p!=NULL) {
			if(p==winner) {
				(p->rounds_won)++;
				(p->wealth)+=winning_money;
			}
			else
			  (p->wealth)+=winning_money/3;
			p=p->next;
		}
		/* Open nice window and stuff.*/
		post_round_info(winner);
	}
	/* Remove guns*/
   gl_setclippingwindow(0,0,MAX_X,MAX_Y);
	update_guns(BOB_REMOVE,NONE);
   gl_setclippingwindow(0,0,MAX_X,S_HEIGHT-1);
}


/* Called before round */
void pre_round_inits() {
	struct player *p;
	int x;
	
	ammo_money=0;
	shots_fired=0;
	
	p=firstplayer;
	while(p!=NULL) {
		p->flags=DEF_PFLAGS;
		p->old_wealth=(p->wealth);
		p->damage=DEF_DAMAGE;
		p=p->next;
	}
	wind=(windmode*MAX_WIND)-windmode*(random()%(2*MAX_WIND+1));
	/* Game screen resolution! */
	hr_screen_open(G640x480x256,NONE);
	set_game_palette();
	set_player_colors();
	create_control_panel();
	/* Do not create random palettes in case precalced pixmaps
	 * are used.*/
	if(!(game_flags & TERRAIN_PIXMAPS))
	  set_ground_palette();
	if(!(game_flags & SKY_PIXMAPS))
	  set_sky_palette();
	/* Create terrain and sky.*/
	message(1,"... Please wait, creating sky ...");
	create_sky();
	message(1,"... Please wait, creating terrain ...");
	create_terrain(150,MAX_Y-TERRAIN_BORDER,bumpiness);
	/* Init some stuff.*/
   land_total=0;
	flash_counter=0;
	for(x=0;x<S_WIDTH;x++)
	  land_total-=terrain[x]+1;
#if 0
	/* Remove blank areas from within terrain.*/
	/* FIX! If guns are initialized before this loop --> flicker.
	 *      If not --> trouble.*/
	while(collapse());
#endif
	/* Put guns in place.*/
	init_guns();
}


/* Called before a player starts his turn */
void pre_player_inits(struct player *p) {
	struct weapon_list *wl;
	/* If the player is a droid, change equipment.*/
	if(p->flags & DROID)
	  auto_change(p);
	else
	  if((p->active_weapon->default_amount)!=-1)  {
		  /* Activate change_active_item() if the player does not have
			* active_weapons left.*/
		  wl=p->weapons;
		  while((wl->type)!=(p->active_weapon))
			 wl=wl->next;
		  if((wl->amount)==0)
		  	 change_active_item(p,0);
	  }
}


/* Called before a turn */
void pre_turn_inits() {
	struct player *p;
	turn++;
	/*.*/
	p=firstplayer;
	while(p!=NULL){
		/* Unset "turn done"-flag.*/
		p->flags &= ~(TURN_DONE);
		p=p->next;
	}
	/* Modify wind.*/
	if(windmode>0)
	  wind+=windmode*4-random()%(windmode*8+1);
	if(wind<(-windmode*MAX_WIND))
	  wind=-windmode*MAX_WIND;
	if(wind>( windmode*MAX_WIND))
	  wind=windmode*MAX_WIND;
		/* Free some lists.*/
	delete_all_bullets();
	/* Check whether mother nature is angry */
	if((game_flags & DISASTERS) && (random()%10==0))
	  disaster();
}


/* Called after all players have made their moves.*/
/* Disable buttons in control panel... Aesthetics...  */
void post_players_stuff(){
	struct hr_button *bb;
	int bflags;
	bb=hr.firstbutton;
	while(bb!=NULL) {
		bflags=(bb->flags &= ~(ENABLED));
		hr_update_button(0,bb,UPDATE_FLAGS,&bflags);
		bb=bb->next;
	}
}


/* Called after a turn.*/
void post_turn_stuff() {	
	/* aaaaaaaaaabsolutely nothing here yet.*/
}


/* Default values of some global variables */
void defaults() {
	interest=DEF_INTEREST;
	round_n=DEF_ROUNDS;
	start_money=DEF_MONEY;
	winning_money=DEF_WMONEY;
	g=DEF_G;
	wind=DEF_WIND;
	windmode=DEF_WINDMODE;
	bumpiness=DEF_BUMPINESS;
	
	/* NOTE! No checking. So, for example, GAME_ABORTED flag _can_
	 * be given as default, after which 1st played game aborts
	 * immediately :-)*/
	game_flags=DEF_GFLAGS;
	edges=DEF_EDGES;

	droids=DEF_DROIDS;
	droid_sq=DEF_DROID_SQ;
}
