/*
 *	Tykkipeli 2
 *	Game options screen.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <misc.h>
#include <info_windows.h>
#include <option_screen_defines.h>

#include <hr_defines.h>
#include <hr_mouse.h>
#include <hr_bobs.h>
#include <hr_buttons.h>
#include <hr_fonts.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_keyboard.h>
#include <hr_input_handler.h>
#include <hr_screens.h>


/* These are used for the starfield behind the logo.*/
#define OPTION_STAR_N 112

struct option_star{
	int x;	
	int y;
	int xs;
	int ys;
	int color;
};

struct option_star *option_stars;


/* Init the top logo effect. */
void init_options_effect() {
	int s;
	/* Star colors.*/
	for(s=0;s<64;s++)
	  gl_setpalettecolor(192+s,s,s,s);
	/* Scroller color.*/
	gl_setpalettecolor(62,63,0,63);
	/* Get logo palette */
	hr_use_bitmap_palette(logo,128,192,NONE);
	gfx_counter=scrolltext_counter=0;
	gl_fillbox(0,0,320,120,192);
	gl_putboxmask(160-(logo->width)/2,56-(logo->height)/2,
					  logo->width,logo->height,logo->data);
	option_stars=malloc(OPTION_STAR_N*sizeof(struct option_star));
	for(s=0;s<OPTION_STAR_N;s++) {
		(option_stars+s)->x=random()%320;
		(option_stars+s)->y=random()%112;
		(option_stars+s)->xs=10+random()%4;
		(option_stars+s)->ys=1+random()%3;
		(option_stars+s)->color=210+random()%30;
	}
}


/* Update logo effect. Re-inits mouse pointer if necessary.*/
int options_effect() {
   int s,a,x=hr_mouse.x,y=hr_mouse.y;
	if((x<320) && (y<120))
	  hr_update_mouse(NONE,BOB_REMOVE);
	/* Calc s, update scroller */
	s=(++gfx_counter)%8;
	if(s==0) {
		if(*(instructions+(++scrolltext_counter))==0)
		  scrolltext_counter=0;
	}
	vga_waitretrace();
	gl_copybox(1,112,319,8,0,112);
	hr_partial_monowrite(318,112,62,0,*(instructions+scrolltext_counter),
								s,0,s,7,hr.font8x8,NONE);
	/* Stars */
	a=gfx_counter%360;
	for(s=0;s<OPTION_STAR_N;s++)  {
		/* Remove old star */
		if(vga_getpixel((option_stars+s)->x,(option_stars+s)->y)>192)
		  gl_setpixel((option_stars+s)->x,(option_stars+s)->y,192);
		/* Calc new X position */
		(option_stars+s)->x=((option_stars+s)->x+
									 (((option_stars+s)->xs*pcos[a])>>SCALE));
		if((option_stars+s)->x<0)
		  ((option_stars+s)->x)+=320;
		if((option_stars+s)->x>319)
		  (option_stars+s)->x-=320;
		/* Calc new Y position */
		(option_stars+s)->y=((option_stars+s)->y+(option_stars+s)->ys);
		if((option_stars+s)->y>111)
		  ((option_stars+s)->y)-=112;
		/* Draw the dot */
		if(vga_getpixel((option_stars+s)->x,(option_stars+s)->y)==192)
		  gl_setpixel((option_stars+s)->x,
						  (option_stars+s)->y,(option_stars+s)->color);
	}
	if((x<320) && (y<120))
	  hr_update_mouse(NONE,BOB_INIT);
	return 0;
}


/* Creates the option screen */
void create_option_screen() {
	int r,hotkey;
	
	hr_bevelbox(320,0,591,479,hr.def_colors,NONE);				/* Roster */
	hr_bevelbox(592,0,639,479,hr.def_colors,NONE);
	for(r=0;r<ROSTER_SIZE;r++)  {
		/* Create the wide button with player's name on it */
		hr_create_button(100+r,322,2+r*14,589,14+r*14,
							  hr.font8x8,hr.def_colors,
							  roster[r].name,NULL,ENABLED|NO_TAB_CYCLE);
		/* Print the ratio numbers */
		if(roster[r].games>0)
		  sprintf(buffer,"%d",magic_ratio(roster[r].deaths,roster[r].kills,
													 roster[r].suicides));
		else
		  sprintf(buffer,"-");
		/* Create smaller button that opens info window */
		hr_create_button(200+r,594,2+r*14,637,14+r*14,
							  hr.font8x8,hr.def_colors,
							  buffer,"Info",ENABLED|NO_TAB_CYCLE);
		/* textbox(611,2+r*14,637,14+r*14,1,buffer,1); !!!!!!!!!!!!!!*/
	}
	
	/* game options */
  	hr_bevelbox(0,120,179,137,hr.def_colors,NONE);
	hr_monowrite(4,126,1,0,"Rounds",hr.font8x8,NONE);
	hr_create_button(50,86,122,99,135,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(51,164,122,177,135,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);
	
  	hr_bevelbox(0,138,179,185,hr.def_colors,NONE);
	hr_monowrite(4,142,1,0,"Money",hr.font8x8,NONE);
	hr_create_button(52,86,140,99,153,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(53,164,140,177,153,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);
	
	hr_monowrite(4,157,1,0,"Winning $",hr.font8x8,NONE);
	hr_create_button(54,86,155,99,168,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(55,164,155,177,168,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);
	
   hr_monowrite(4,172,1,0,"Interest",hr.font8x8,NONE);
	hr_create_button(56,86,170,99,183,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(57,164,170,177,183,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);
	
	/* game options */
  	hr_bevelbox(0,186,179,365,hr.def_colors,NONE);
	hr_monowrite(4,190,1,0,"Gravity",hr.font8x8,NONE);
	hr_create_button(58,86,188,99,201,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(59,164,188,177,201,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);
	
	hr_monowrite(4,205,1,0,"Wind",hr.font8x8,NONE);
	hr_create_button(60,86,203,99,216,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(61,164,203,177,216,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);
	
	hr_monowrite(4,220,1,0,"Edges",hr.font8x8,NONE);
	hr_create_button(62,86,218,99,231,
						  hr.font8x8,hr.def_colors,
						  "<",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(63,164,218,177,231,
						  hr.font8x8,hr.def_colors,
						  ">",NULL,ENABLED|NO_TAB_CYCLE);	
	
	hr_monowrite(4,235,1,0,"Steepness",hr.font8x8,NONE);
	hr_create_button(64,86,233,99,246,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,ENABLED|NO_TAB_CYCLE);
	hr_create_button(65,164,233,177,246,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,ENABLED|NO_TAB_CYCLE);	
	
	/* Computer player settings */
	hr_bevelbox(0,366,179,398,hr.def_colors,NONE);
	hr_monowrite(4,370,1,0,"Droids",hr.font8x8,NONE);
	hr_create_button(66,86,368,99,381,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,NONE|NO_TAB_CYCLE);
	hr_create_button(67,164,368,177,381,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,NONE|NO_TAB_CYCLE);
	
	hr_monowrite(4,385,1,0,"Droid IQ",hr.font8x8,NONE);
	hr_create_button(68,86,383,99,396,
						  hr.font8x8,hr.def_colors,
						  "-",NULL,NONE|NO_TAB_CYCLE);
	hr_create_button(69,164,383,177,396,
						  hr.font8x8,hr.def_colors,
						  "+",NULL,NONE|NO_TAB_CYCLE);
	
	/* On/off options */
	hr_bevelbox(180,120,319,398,hr.def_colors,NONE);
	hr_create_button(10,182,122,317,133,
						  hr.font8x8,hr.def_colors,
						  "Tracers",NULL,ENABLED);
	hr_create_button(11,182,135,317,146,
						  hr.font8x8,hr.def_colors,
						  "Disasters",NULL,NONE);
	hr_create_button(12,182,148,317,159,
						  hr.font8x8,hr.def_colors,
						  "Particles",NULL,ENABLED);
	hr_create_button(13,182,161,317,172,
						  hr.font8x8,hr.def_colors,
						  "Sound Effects",NULL,NONE);
	hr_create_button(14,182,174,317,185,
						  hr.font8x8,hr.def_colors,
						  "Unsafe storage",NULL,NONE);
   hr_create_button(15,182,187,317,198,
						  hr.font8x8,hr.def_colors,
						  "Suicide Penalty",NULL,NONE);
   hr_create_button(16,182,200,317,211,
						  hr.font8x8,hr.def_colors,
						  "AimAid(tm)",NULL,NONE);
	
   hr_create_button(17,182,213,317,224,
						  hr.font8x8,hr.def_colors,
						  "Alien terrain",NULL,NONE);
   hr_create_button(18,182,226,317,237,
						  hr.font8x8,hr.def_colors,
						  "Alien sky",NULL,NONE);
   hr_create_button(19,182,239,317,250,
						  hr.font8x8,hr.def_colors,
						  "Random order",NULL,NONE);
   hr_create_button(20,182,252,317,263,
						  hr.font8x8,hr.def_colors,
						  "GroundSmoothing",NULL,NONE);
   hr_create_button(21,182,265,317,276,
						  hr.font8x8,hr.def_colors,
						  "Terrain pixmaps",NULL,NONE);
   hr_create_button(22,182,278,317,289,
						  hr.font8x8,hr.def_colors,
						  "Sky pixmaps",NULL,NONE);
   hr_create_button(23,182,291,317,302,
						  hr.font8x8,hr.def_colors,
						  "ExploFlash(tm)",NULL,NONE);
   hr_create_button(24,182,304,317,315,
						  hr.font8x8,hr.def_colors,
						  "ExploColRot(tm)",NULL,NONE);
   hr_create_button(25,182,317,317,328,
						  hr.font8x8,hr.def_colors,
						  "Shopping",NULL,NONE);
	/* Wide buttons */
	hr_bevelbox(0,400,319,479,hr.def_colors,NONE);
	hr_create_button(4,3,402,317,413,
						  hr.font8x8,hr.def_colors,
						  "About Tykkipeli 2",NULL,NONE);
	hotkey='a';hr_update_button(4,NULL,UPDATE_HOTKEY,&hotkey);
	hr_create_button(3,3,415,317,426,
						  hr.font8x8,hr.def_colors,
						  "Save roster",NULL,NONE);
	hotkey='s';hr_update_button(3,NULL,UPDATE_HOTKEY,&hotkey);
	hr_create_button(2,3,428,317,439,
						  hr.font8x8,hr.def_colors,
						  "Hall of Fame",NULL,ENABLED);
	hotkey='h';hr_update_button(2,NULL,UPDATE_HOTKEY,&hotkey);
	hr.default_button=hr_create_button(5,3,441,317,452,
												  hr.font8x8,hr.def_colors,
												  "Back to intro",NULL,ENABLED);
	hotkey='i';hr_update_button(5,NULL,UPDATE_HOTKEY,&hotkey);
	hr_create_button(1,3,454,317,477,
						  hr.font8x8,hr.def_colors,
						  "Begin game","Let's Go!",NONE);
	hotkey='g';hr_update_button(1,NULL,UPDATE_HOTKEY,&hotkey);
}


/* Updates parts of the option screen */
void update_option_screen(int flags) {
	int r,rc,bflags;
	if(flags & OPTION_ROSTER)
	  for(r=0;r<ROSTER_SIZE;r++) {
		  bflags=(((roster[r].flags & PRESSED)?(PRESSED):(NONE))
					 |ENABLED|SWITCH|NO_TAB_CYCLE);
		  hr_update_button(100+r,NULL,UPDATE_FLAGS,&bflags);
	  }
	
	if(flags & OPTION_ONOFF) {
		/* Tracers */
		bflags=(((game_flags & TRACERS)?(PRESSED):(NONE))|ENABLED|SWITCH);
		hr_update_button(10,NULL,UPDATE_FLAGS,&bflags);
#if 0
		/* Disasters */
		bflags=(((game_flags & DISASTERS)?(PRESSED):(NONE))|ENABLED|SWITCH);
		hr_update_button(11,NULL,UPDATE_FLAGS,&bflags);
#endif
		/* Particles*/
		bflags=(((game_flags & PARTICLES)?(PRESSED):(NONE))|ENABLED|SWITCH);
		hr_update_button(12,NULL,UPDATE_FLAGS,&bflags);
#ifdef SOUND_SUPPORT
		/* Sound effects*/
		bflags=(((game_flags & SOUNDEFFECTS)?(PRESSED):(NONE))|ENABLED|SWITCH);
		hr_update_button(13,NULL,UPDATE_FLAGS,&bflags);
#endif
		/* Unsafe Storage*/
		bflags=(((game_flags & ACCIDENTAL_FIRE)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(14,NULL,UPDATE_FLAGS,&bflags);
		/* Suicide penalty*/
		bflags=(((game_flags & SUICIDE_PENALTY)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(15,NULL,UPDATE_FLAGS,&bflags);
		/* AimAid(tm) */
		bflags=(((game_flags & AIM_AID)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(16,NULL,UPDATE_FLAGS,&bflags);
		/* Alien terrain */
		bflags=SWITCH;
		if(!(game_flags & TERRAIN_PIXMAPS))
		  bflags |=ENABLED;
		if(game_flags & ALIEN_TERRAIN)
		  bflags |=PRESSED;
		hr_update_button(17,NULL,UPDATE_FLAGS,&bflags);
		/* Alien sky*/
		bflags=SWITCH;
		if(!(game_flags & SKY_PIXMAPS))
		  bflags |=ENABLED;
		if(game_flags & ALIEN_SKY)
		  bflags |=PRESSED;
		hr_update_button(18,NULL,UPDATE_FLAGS,&bflags);
		/* Random order */
		bflags=(((game_flags & RANDOM_ORDER)?(PRESSED):(NONE))|ENABLED|SWITCH);
		hr_update_button(19,NULL,UPDATE_FLAGS,&bflags);
		/* GroundSmoothing button is enabled only in non-zero-gravity */
		bflags=(((game_flags & GROUNDSMOOTHING)?(PRESSED):(NONE))
				  |SWITCH);
		if(g>0)
		  bflags|=ENABLED;
		hr_update_button(20,NULL,UPDATE_FLAGS,&bflags);
		/* Terrain pixmaps*/
		bflags=(((game_flags & TERRAIN_PIXMAPS)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(21,NULL,UPDATE_FLAGS,&bflags);
		/* Sky pixmaps */
		bflags=(((game_flags & SKY_PIXMAPS)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(22,NULL,UPDATE_FLAGS,&bflags);
		/* ExploFlash*/
		bflags=(((game_flags & EXPLO_FLASH)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(23,NULL,UPDATE_FLAGS,&bflags);
		/* Explosion color rotation*/
		bflags=(((game_flags & EXPLO_COLROT)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(24,NULL,UPDATE_FLAGS,&bflags);
		/* Shopping */
		bflags=(((game_flags & SHOPPING)?(PRESSED):(NONE))
				  |ENABLED|SWITCH);
		hr_update_button(25,NULL,UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_BEGIN) {
		rc=0;
		for(r=0;r<ROSTER_SIZE;r++)
		  if(roster[r].flags & PRESSED) rc++;
		
		bflags=(((rc+droids>1)&&
					((rc+droids)<S_WIDTH/(2*MIN_DISTANCE)))?(ENABLED):(NONE));
		hr_update_button(1,NULL,UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_WIDE) {
		bflags=((game_flags & ROSTER_SAVED)?(NONE):(ENABLED));
		hr_update_button(3,NULL,UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_ROUNDS) {
		sprintf(buffer,"%d",round_n);
		hr_textbox(101,122,162,135,
					  hr.font8x8,hr.def_colors,buffer,PRESSED|ENABLED);
		
		
		bflags=((round_n>1)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(50,NULL,UPDATE_FLAGS,&bflags);
		
		bflags=((round_n<MAX_ROUNDS)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(51,NULL,UPDATE_FLAGS,&bflags);
	}

	if(flags & OPTION_MONEY) {
		bflags=((start_money>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(52,NULL,UPDATE_FLAGS,&bflags);
	   sprintf(buffer,"%d",start_money);
	   hr_textbox(101,140,162,153,
					  hr.font8x8,hr.def_colors,buffer,PRESSED|ENABLED);
		bflags=((start_money<MAX_MONEY)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(53,NULL,UPDATE_FLAGS,&bflags);
		
		bflags=((winning_money>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(54,NULL,UPDATE_FLAGS,&bflags);
	   sprintf(buffer,"%d",winning_money);
	   hr_textbox(101,155,162,168,
					  hr.font8x8,hr.def_colors,buffer,PRESSED|ENABLED);
		bflags=((winning_money<MAX_WMONEY)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(55,NULL,UPDATE_FLAGS,&bflags);
		
		bflags=((interest>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(56,NULL,UPDATE_FLAGS,&bflags);
	   sprintf(buffer,"%d",interest);
	   hr_textbox(101,170,162,183,
					  hr.font8x8,hr.def_colors,buffer,PRESSED|ENABLED);
		bflags=((interest<MAX_INTEREST)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(57,NULL,UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_MISC) {
		bflags=((g>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(58,NULL,UPDATE_FLAGS,&bflags);
	   hr_textbox(101,188,162,201,
					  hr.font8x8,hr.def_colors,gravity_string(),PRESSED|ENABLED);
		bflags=((g<MAX_G)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(59,NULL,UPDATE_FLAGS,&bflags);
		
		bflags=((windmode>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(60,NULL,UPDATE_FLAGS,&bflags);
	   hr_textbox(101,203,162,216,
					  hr.font8x8,hr.def_colors,
					  windtext[windmode],PRESSED|ENABLED);
		bflags=((windmode<MAX_WINDMODE)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(61,NULL,UPDATE_FLAGS,&bflags);
		
		bflags=((edges>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(62,NULL,UPDATE_FLAGS,&bflags);
		hr_textbox(101,218,162,231,
					  hr.font8x8,hr.def_colors,
					  edgetext[edges],PRESSED|ENABLED);
		bflags=((edges<MAX_EDGEMODE)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(63,NULL,UPDATE_FLAGS,&bflags);

		bflags=NO_TAB_CYCLE;
		if(!(game_flags & TERRAIN_PIXMAPS))
		  bflags|=((bumpiness>0)?(ENABLED):(NONE));
		hr_update_button(64,NULL,UPDATE_FLAGS,&bflags);
		hr_textbox(101,233,162,246,
					  hr.font8x8,hr.def_colors,
					  percentage_string(bumpiness,DEF_BUMPINESS),
					  PRESSED|((game_flags & TERRAIN_PIXMAPS)?(NONE):(ENABLED)));
		bflags=NO_TAB_CYCLE;
		if(!(game_flags & TERRAIN_PIXMAPS))
		  bflags|=((bumpiness<=MAX_BUMPINESS)?(ENABLED):(NONE));
		hr_update_button(65,NULL,UPDATE_FLAGS,&bflags);
	}

	if(flags & OPTION_DROIDS) {
		bflags=((droids>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(66,NULL,UPDATE_FLAGS,&bflags);
		sprintf(buffer,"%d",droids);
		hr_textbox(101,368,162,381,
					  hr.font8x8,hr.def_colors,
					  buffer,PRESSED|ENABLED);
		bflags=((droids<MAX_DROIDS)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(67,NULL,UPDATE_FLAGS,&bflags);
		
		bflags=((droid_sq>0)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(68,NULL,UPDATE_FLAGS,&bflags);
		if(droid_sq>0)
		  /* Ugly! percentage_string outputs to buffer[] at the mo...*/
		  percentage_string(droid_sq,DEF_DROID_SQ);
		else
		  sprintf(buffer,"AOLer");
		hr_textbox(101,383,162,396,
					  hr.font8x8,hr.def_colors,
					  buffer,PRESSED|ENABLED);
		bflags=((droid_sq<MAX_DROID_SQ)?(ENABLED):(NONE))|NO_TAB_CYCLE;
		hr_update_button(69,NULL,UPDATE_FLAGS,&bflags);
	}
}


/* This screen lets you change player names, etc etc... */
int option_screen(){
	struct hr_button *ob,*b;
	int begin_game=0,back_to_intro=0,see_hall_of_fame=0,quit=0;
	
   hr_screen_open(G640x480x256,NONE);
	hr_default_palette();
	init_options_effect();
	create_option_screen();	
	update_option_screen(OPTION_ROSTER|OPTION_BEGIN|OPTION_ONOFF|
								OPTION_ROUNDS|OPTION_MONEY|OPTION_MISC|
								OPTION_DROIDS|OPTION_WIDE);
	/* Loop until "Begin game"-button is selected. */
	hr_input_handler(NULL,&options_effect,AMB_RELEASED);
	hr_clearkey();
	while(!begin_game && !back_to_intro &&
			!see_hall_of_fame && !quit){
		ob=hr_input_handler(NULL,&options_effect,LMB_PRESSED|KBD_PRESSED);
		/* Check special keypresses */
		switch(hr_keyboard.lastkey){
		 case 27:
			if(ask_confirmation("Really quit?")){
				if(!(game_flags & ROSTER_SAVED)){
					if(ask_confirmation("Save roster?"))
					  save_roster();
				}
				quit=1;
			}
			break;
		}
		b=hr_input_handler(ob,&options_effect,LMB_RELEASED);
		/* Buttons that respond after the mouse key is released.*/
		if(b!=NULL){
			/* Roster quickinfo buttons */
         if((b->index)>=200)
			  option_quickinfo((b->index)-200);
			/* Roster buttons */
			else if((b->index)>=100)  {
				/* Set the PRESSED flag in roster entry.*/
				if(b->flags & PRESSED)
				  roster[(b->index)-100].flags |= PRESSED;
				else
				  roster[(b->index)-100].flags &= ~(PRESSED);
				update_option_screen(OPTION_BEGIN);
			}
			/* Other buttons */
			else  {
				switch(b->index) {			
				 case 1:	
					begin_game=1;
					break;	
				 case 5:
					back_to_intro=1;
					break;
				 case 3:
					save_roster();
					update_option_screen(OPTION_WIDE);
					break;
#if 0
				 case 4:
					see_about=1;
					break;
#endif
				 case 2:
					see_hall_of_fame=1;
					break;
				 case 10:
					game_flags ^= TRACERS;
					break;
#if 0
				 case 11:
					game_flags ^= DISASTERS;
					break;
#endif
				 case 12:
					game_flags ^= PARTICLES;
					break;
#ifdef SOUND_SUPPORT
				 case 13:
					game_flags ^= SOUNDEFFECTS;
					break;
#endif
				 case 14:
					game_flags ^= ACCIDENTAL_FIRE;
					break;
				 case 15:
					game_flags ^= SUICIDE_PENALTY;
					break;
				 case 16:
					game_flags ^= AIM_AID;
					break;
				 case 17:
					game_flags ^= ALIEN_TERRAIN;
					break;
				 case 18:
					game_flags ^= ALIEN_SKY;
					break;
				 case 19:
					game_flags ^= RANDOM_ORDER;
					break;
				 case 20:
					game_flags ^= GROUNDSMOOTHING;
					break;
				 case 21:
					game_flags ^= TERRAIN_PIXMAPS;
					update_option_screen(OPTION_ONOFF|OPTION_MISC);
					break;
				 case 22:
					game_flags ^= SKY_PIXMAPS;
					update_option_screen(OPTION_ONOFF);
					break;
				 case 23:
					game_flags ^= EXPLO_FLASH;
					update_option_screen(OPTION_ONOFF);
					break;
				 case 24:
					game_flags ^= EXPLO_COLROT;
					update_option_screen(OPTION_ONOFF);
					break;
				 case 25:
					game_flags ^= SHOPPING;
					update_option_screen(OPTION_ONOFF);
					break;
				 case 50:
					--round_n;
					update_option_screen(OPTION_ROUNDS);
					break;
				 case 51:
					++round_n;
					update_option_screen(OPTION_ROUNDS);
					break;
				 case 52:
					start_money-=1000;
					update_option_screen(OPTION_MONEY);
					break;
				 case 53:
					start_money+=1000;
					update_option_screen(OPTION_MONEY);
					break;
				 case 54:
					winning_money-=1000;
					update_option_screen(OPTION_MONEY);
					break;
				 case 55:
					winning_money+=1000;
					update_option_screen(OPTION_MONEY);
					break;
				 case 56:
					interest-=5;
					update_option_screen(OPTION_MONEY);
					break;
				 case 57:
					interest+=5;
					update_option_screen(OPTION_MONEY);
					break;
				 case 58:
					--g;
					update_option_screen(OPTION_MISC|OPTION_ONOFF);
					break;
				 case 59:
					++g;
					update_option_screen(OPTION_MISC|OPTION_ONOFF);
					break;
				 case 60:
					--windmode;
					update_option_screen(OPTION_MISC);
					break;
				 case 61:
					++windmode;
					update_option_screen(OPTION_MISC);
					break;
				 case 62:
					--edges;
					update_option_screen(OPTION_MISC);
					break;
				 case 63:
					++edges;
					update_option_screen(OPTION_MISC);
					break;
				 case 64:
					--bumpiness;
					update_option_screen(OPTION_MISC);
					break;
				 case 65:
					++bumpiness;
					update_option_screen(OPTION_MISC);
					break;
				 case 66:
					--droids;
					update_option_screen(OPTION_DROIDS|OPTION_BEGIN);
					break;
				 case 67:
					++droids;
					update_option_screen(OPTION_DROIDS|OPTION_BEGIN);
					break;
				 case 68:
					--droid_sq;
					update_option_screen(OPTION_DROIDS);
					break;
				 case 69:
					++droid_sq;
					update_option_screen(OPTION_DROIDS);
					break;
				}
			}
		}
	}

	/* Free some space.*/
	hr_delete_all_buttons();
	free(option_stars);

	if(quit)
	  return 3;
	else if(see_hall_of_fame)
	  return 2;
	else if(back_to_intro)
	  return 1;
	else
	  return 0;
}
