/*
 * Use parse_args() as framework on which to build option parsing.
 * Taken from man page
 * GNU                       April 25, 1993                        3
 * GETOPT(3)           Linux Programmer's Manual           GETOPT(3)
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <version.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>

#define OPTION_CHARACTERS "hvqi:r:m:M:g:b:e:d:D:f:w:"


/* Return 1 if the calling proggie should quit. */
/* ADD POINTERZ HERE! */
int parse_args(int argc,unsigned char **argv) {
	int c, option_index=0;
	static struct option long_options[] = {
		  {"help",0, 0,'h'},
		  {"quiet",0,0,'q'},
		  {"interest",1,0,'i'},
		  {"rounds",1,0,'r'},
		  {"money",1,0,'m'},
		  {"wmoney",1,0,'M'},
		  {"gravity",1,0,'g'},
		  {"bumpiness",1,0,'b'},
		  {"edges",1,0,'e'},
		  {"droidsq",1,0,'D'},
		  {"droids",1,0,'d'},
		  {"flags",1,0,'f'},
		  {"wind",1,0,'w'},
		  {"version",0,0,'v'}
	};
	
	while(1) {
		c = getopt_long(argc,(char **)argv,OPTION_CHARACTERS,
							 long_options,&option_index);
		
		/* No more args */
		if(c == -1)
		  break;
		
		switch(c)  {
		 case 'h':
			printf("Usage: %s [options]\n"\
					 "Where long/short options are:\n"\
					 "--help      -h   Prints this help, then exits.\n"\
					 "--version   -v   Prints version string, then exits.\n"\
                "--quiet     -q   Do not print any messages to stdout.\n"\
					 "\nALL FOLLOWING OPTIONS NEED AN ARGUMENT!\n"\
					 "USE WITH CAUTION.\n"\
					 "CHECK THE HEADER FILES FOR INFO ABOUT FLAGS, BOUNDARIES ETC!\n\n"\
					 "--interest  -i   Sets interest.\n"\
					 "--rounds    -r        # of rounds.\n"\
					 "--money     -m        money.\n"\
					 "--wmoney    -M        price money given to winner.\n"\
					 "--gravity   -g        gravity.\n"\
					 "--bumpiness -b        terrain bumpiness.\n"\
					 "--edges     -e        edge mode.\n"\
					 "--droidsq   -D        droid stupidity.\n"\
					 "--droids    -d        number of droids.\n"\
					 "--flags     -f        ON/OFF game flags.\n"\
					 "--wind      -w        wind strength.\n",argv[0]);
			return 1;
			break;
		 case 'v':
			printf("Tykkipeli 2 %s\n",TP2_VERSION);
			return 1;
			break;
		 case 'q':
			game_flags |= QUIET;
			break;
		 case 'i':
			interest=atoi(optarg);
			if((interest<0) || (interest>MAX_INTEREST))
			  return 1;
			break;
		 case 'r':
			round_n=atoi(optarg);
			if((round_n<0) || (round_n>MAX_ROUNDS))
			  return 1;
			break;
		 case 'm':
			start_money=atoi(optarg);
			if((start_money<0) || (start_money>MAX_MONEY))
			  return 1;
			break;
		 case 'M':
			winning_money=atoi(optarg);
			if((winning_money<0) || (winning_money>MAX_WMONEY))
			  return 1;
			break;
		 case 'g':
			g=atoi(optarg);
			if((g<0)||(g>MAX_G))
			  return 1;
			break;
		 case 'b':
			bumpiness=atoi(optarg);
			if((bumpiness<0)||(bumpiness>MAX_BUMPINESS))
			  return 1;
			break;
		 case 'e':
			edges=atoi(optarg);
			if((edges<0)||(edges>MAX_EDGEMODE))
			  return 1;
			break;
		 case 'd':
			droids=atoi(optarg);
			break;
		 case 'D':
			droid_sq=atoi(optarg);
			break;
		 case 'f':
			game_flags=atoi(optarg);
			break;
		 case 'w':
			windmode=atoi(optarg);
			if((windmode<0)||(windmode>MAX_WINDMODE))
			  return 1;
			break;
		 default:
			printf("?? getopt returned character code 0%o ??\n",c);
		}
	}
	return 0;
}
