/*
 * Player stuff
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <tp2.h>
#include <weapons.h>
#include <bullets.h>
#include <shields.h>
#include <misc.h>
#include <control_panel.h>
#include <guns.h>
#include <explode.h>
#include <colors.h>
#include <weapon_defines.h>
#include <shield_defines.h>

#include <hr_defines.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_ilbm.h>


struct shield_list{
	struct shield_list *next;
	struct shield *type;
	int amount;
};

struct weapon_list{
	struct weapon_list *next;
	struct weapon *type;
	int amount;
};

struct player {
	struct player *next;
	unsigned char *name;
	int  flags;
	int  rosterindex;
	int  damage;
	int  old_wealth;
	int  wealth;
	int  rounds_won;
	int  kills;
	int  deaths;
	int  suicides;
	
	struct weapon      *active_weapon;
	struct weapon_list *weapons;
	struct shield      *active_shield;
	struct shield_list *shields;
	
	int  angle;
	int  power;
	int  x;
	int  y;
	int  firstcolor;
	int  britecolor;
	int  darkcolor;
	struct hr_bitmap *bitmap;
	struct hr_bitmap *deadmap;
	struct hr_bitmap *tempmap;
};

struct {
	int flags;
	unsigned char *name;
	unsigned int  games;
	unsigned int  won;
	unsigned int  kills;
	unsigned int  deaths;
	unsigned int  suicides;

	unsigned int default_weapon;
	unsigned int default_shield;
	
	struct hr_bitmap *bitmap;
	struct hr_bitmap *deadmap;
} roster[ROSTER_SIZE];


int magic_ratio(int deaths,int kills,int suicides) {
	return (kills-deaths-suicides*2);
}


/* Reads player colors from file.*/
void set_player_colors(){
	FILE *gunfile;
	struct player *p;
	unsigned char tmp[BUFFER_SIZE];
	int c,plr,rr,gg,bb;
	sprintf(buffer,"%s/players/%s",tp2_datapath,GUN_DEFINITIONS);
	if((gunfile=fopen(buffer,"r"))){
		while(fgets(tmp,BUFFER_SIZE,gunfile)){
			if(strncasecmp("#GUN_COLORS",tmp,11)==0){
				p=firstplayer;
				for(plr=0;plr<player_n;plr++){
					for(c=0;c<COLORS_PER_PLAYER;c++){
						fscanf(gunfile,"%d,%d,%d",&rr,&gg,&bb);
						gl_setpalettecolor((p->firstcolor)+c,rr,gg,bb);
					}
					p=p->next;
				}
				break;
			}
		}
		fclose(gunfile);
	}
	else
	  printf("Error opening %s\n",buffer);
}


/* Initialize roster */
void init_roster() {
	int l,readmode,plr;
	FILE *rosterfile,*gunfile;
	unsigned char tmp[BUFFER_SIZE],guntmp[BUFFER_SIZE],deadtmp[BUFFER_SIZE];
	/**/
	sprintf(buffer,"%s/players/%s",tp2_datapath,ROSTERFILE);
	if((rosterfile=fopen(buffer,"r"))){
		for(plr=0;plr<ROSTER_SIZE;plr++){
			fscanf(rosterfile,"%s %d %d %d %d %d %u %u",
					 tmp,
					 &roster[plr].games,
					 &roster[plr].won,
					 &roster[plr].deaths,
					 &roster[plr].kills,
					 &roster[plr].suicides,
					 &roster[plr].default_weapon,
					 &roster[plr].default_shield);
			/* Replace '_' -> ' ' */
			for(l=0;l<strlen(tmp);l++){
				if(tmp[l]=='_')
				  tmp[l]=' ';
			}
			/* Create player name and init some stuff */
			roster[plr].name=malloc(strlen(tmp)+1);
			strcpy(roster[plr].name,tmp);
			roster[plr].flags=NONE;
			/* Zero these (for now) */
			roster[plr].bitmap=roster[plr].deadmap=NULL;
		}
		fclose(rosterfile);
		/**/
		/* Ok, now read gun file names from GUN_DEFINITIONS
		 * and create bitmaps and deadmaps for roster items.*/
		readmode=plr=0;
		sprintf(buffer,"%s/players/%s",tp2_datapath,GUN_DEFINITIONS);
		if((gunfile=fopen(buffer,"r"))){
			while(fgets(tmp,BUFFER_SIZE,gunfile)&&(readmode<2)){
				switch(readmode){
				 case 0:
					if(strncasecmp("#GUN_IMAGES",tmp,11)==0)
					  readmode=1;
					break;
				 case 1:
					if(strncasecmp("#END_GUN_IMAGES",tmp,15)==0)
					  readmode=2;
					else if(plr<ROSTER_SIZE){
						/* Read gun bitmap and deadmap.*/
						sscanf(tmp,"%s %s",guntmp,deadtmp);
						sprintf(tmp,"%s/pixmaps/%s",tp2_datapath,guntmp);
						roster[plr].bitmap=hr_read_ilbm(tmp,GFX_256COLOR);
						sprintf(tmp,"%s/pixmaps/%s",tp2_datapath,deadtmp);
						roster[plr].deadmap=hr_read_ilbm(tmp,GFX_256COLOR);
						++plr;
					}
					break;
				}
			}
			fclose(gunfile);
		}
		else
		  printf("Error opening %s.\n",buffer);
	}
	else
	  printf("Error opening %s.\n",buffer);	
}


/* Inits some stuff that is necessary for droids to work.*/
void init_droids(){
   FILE *droidfile;
	unsigned char tmp[BUFFER_SIZE];
	unsigned char nametmp[BUFFER_SIZE],guntmp[BUFFER_SIZE],deadtmp[BUFFER_SIZE];
	sprintf(buffer,"%s/players/%s",tp2_datapath,DROID_DEFINITIONS);
	if((droidfile=fopen(buffer,"r"))){
		while(fgets(tmp,BUFFER_SIZE,droidfile)){
			if(strncasecmp("#DROIDS",tmp,7)==0){
				fgets(tmp,BUFFER_SIZE,droidfile);
				sscanf(tmp,"%s %s %s",nametmp,guntmp,deadtmp);
				/* Set droid name */
				droid_name=malloc(strlen(nametmp)+1);
				strcpy(droid_name,nametmp);
				/* Create droid bitmaps.*/
				sprintf(buffer,"%s/pixmaps/%s",tp2_datapath,guntmp);
				droid_bitmap=hr_read_ilbm(buffer,GFX_256COLOR);
				sprintf(buffer,"%s/pixmaps/%s",tp2_datapath,deadtmp);
				droid_deadmap=hr_read_ilbm(buffer,GFX_256COLOR);
			}
		}
		fclose(droidfile);
	}
	else
	  printf("Error opening %s\n",buffer);
}


/* Returns pointer to player with best kill/death/suicide-ratio,
 * or NULL in case of a draw.*/
struct player *one_with_best_ratio() {
	struct player *p,*winner=NULL;
	int c,x;
	p=firstplayer;c=-6666;
	while(p!=NULL) {
		/* The Magical Number... */
		x=magic_ratio(p->deaths,p->kills,p->suicides);
		if(x>c) {
			c=x;
			winner=p;
		}
		else if(x==c)
		  winner=NULL;
		p=p->next;
	}
	return winner;
}


/* Graphical effect when a player dies. It is possible to hurt others
 * * with this final explosion... :-) */
#define DEATH_POWER1 500
#define DEATH_ANGLE2 (45+random()%90)
#define DEATH_POWER2 (300+random()%400)
void death(struct player *p) {
	struct weapon_list *wl;
	/* Round explosion */
	INIT_EXPLOSION(p,&death_s,p->x,p->y,0,0,0,ENABLED|BULLET_EXPLODING);
	/* Send some phaser dots up the air. Looks cool.*/
	fire(p,&death2_s,(p->x),(p->y)-(p->bitmap->height)-1,
		  90,DEATH_POWER1,ENABLED|FIRE_INSTANTANEOUS);
	/* Send some of the poor guys weapons up the air.*/
	if(game_flags & ACCIDENTAL_FIRE) {
		wl=p->weapons;
		while(wl!=NULL){
			if(((wl->amount)!=0) && (random()%10<5))
			  fire(p,wl->type,
					 (p->x)-(p->bitmap->width)/2+random()%(p->bitmap->width),
					 (p->y)-(p->bitmap->height)-1,
					 DEATH_ANGLE2,DEATH_POWER2,ENABLED|FIRE_INSTANTANEOUS);
			wl=wl->next;
		}
	}
}




/* The Pain... DEATH... destruction :-p */
void hurt(struct player *p,struct player *owner,int d,int damagetype) {
	struct bullet *b;
	if((p->flags) & ALIVE) {
		/* Some shields prevent damage even at this point */
		d=shield_modifieddamage(p,d,damagetype);
		/* Damage gun.*/
		(p->damage)-=abs(d);
		/* Ugly shit. FIX!*/
		panel_bars();
		/* Reward? */
		if(owner!=NULL){
			if(p!=owner)
			  owner->wealth+=d;
		}
		/* Death? */
		if((p->damage)<=0){
			++(p->deaths);
			if(owner!=NULL){
				/* $$$ */
				if(p!=owner) {
					owner->kills++;
					owner->wealth+=(2*d);
				}
				/* or Punishment. */
				else {
					if(game_flags & SUICIDE_PENALTY)
					  owner->wealth-=SUICIDE_LOSS;
					owner->suicides++;
				}
			}
			/* Disable bullets that belong to this player and are
			 * not fired yet. */
			b=firstbullet;
			while(b!=NULL) {
				if( ((b->owner)==p) && ((b->counter)<0) )
				  (b->flags) &= ~(ENABLED);
				b=b->next;
			}
			/* Graphical effect */
			death(p);
			/* Toggle off ALIVE */
			p->flags &= ~(ALIVE);
		}
	}
}


/* Deletes all players.*/
void delete_all_players() {
	struct player *op,*p=firstplayer;
	struct weapon_list *w,*w2;
	struct shield_list *s,*s2;
	firstplayer=NULL;
	while(p!=NULL){
		/* No need to free bitmaps/name, they are part of roster.*/
		/* Free the "weapons bought" list */
		w=p->weapons;
		while(w!=NULL){
			w2=w->next;
			free(w);
			w=w2;
		}
		/* Free the "shields bought" list */
		s=p->shields;
		while(w!=NULL){
			s2=s->next;
			free(s);
			s=s2;
		}
		/* Free the bitmaps. hr_delete_bitmap() also deletes the data
		 * p->xmap->data points to.*/
		hr_delete_bitmap(p->bitmap);
		hr_delete_bitmap(p->deadmap);
		hr_delete_bitmap(p->tempmap);
		/* Finally, get rid of the player*/
		op=p;
		p=p->next;
		free(op);
	}
}


/* Creates a player. */
struct player *create_player(int rosterindex,
									  unsigned char *name,
									  struct hr_bitmap *bitmap,
									  struct hr_bitmap *deadmap,
									  int flags)
{
	struct player *p=firstplayer;
	struct weapon *wp;
	struct shield *sp;
	struct weapon_list *wl;
	struct shield_list *sl;
	unsigned char *t;
	/* Create player struct */
	if(p==NULL){
		player_n=0;
		firstplayer=p=malloc(sizeof(struct player));
	}
	else  {
		while(p->next!=NULL)  {
			p=p->next;
		}
		p->next=malloc(sizeof(struct player));
		p=p->next;
	}
	p->next=NULL;
	/* No need to make copy of all of this data.*/
	p->flags=flags;
	p->name=name;
	p->rosterindex=rosterindex;
	/* Copies of these bitmaps are needed, because they are recolored.
	 * This allows one image file to be used for all players, if need be.*/
	t=malloc((bitmap->width)*(bitmap->height));
	p->bitmap=hr_create_bitmap(t,NULL,
										bitmap->width,bitmap->height,
										bitmap->colors,bitmap->flags);
	t=malloc((deadmap->width)*(deadmap->height));
	p->deadmap=hr_create_bitmap(t,NULL,
										 deadmap->width,deadmap->height,
										 deadmap->colors,deadmap->flags);	
	/* Tempmap. This should cover ALL the gfx a shield/whatever
	 * modifies. AT THE MOMENT NOTHING SHOULD MODIFY GFX _UNDER_ A GUN! */
#define TEMPMAP_WIDTH  (2*TEMPMAP_RANGE+1)
#define TEMPMAP_HEIGHT (2*TEMPMAP_RANGE+1)
	t=malloc(TEMPMAP_WIDTH*TEMPMAP_HEIGHT);
	p->tempmap=hr_create_bitmap(t,NULL,
										 TEMPMAP_WIDTH,TEMPMAP_HEIGHT,
										 256,GFX_256COLOR);	 /* hardcoded.. trouble */
	/* Copy bitmaps */
	memcpy(p->bitmap->data,bitmap->data,(bitmap->width)*(bitmap->height));
	memcpy(p->deadmap->data,deadmap->data,(deadmap->width)*(deadmap->height));
	/* Calc first color.*/
	p->firstcolor=MINCOL_GUNS+(player_n*COLORS_PER_PLAYER);
	/* britecolor and darkcolor are set in set_player_colors() */
	p->darkcolor=(p->firstcolor);
	p->britecolor=(p->firstcolor)+1;
	/* Recolors bitmaps */
	hr_recolor_bitmap(p->bitmap,(p->firstcolor)-1,NONE);
	hr_recolor_bitmap(p->deadmap,(p->firstcolor)-1,NONE);
	/* The color palette is not modified yet.*/
	p->angle=DEF_ANGLE;
	p->power=DEF_POWER;
	p->wealth=start_money;
	p->rounds_won=0;
	p->deaths=0;
	p->kills=0;
	p->suicides=0;
	p->active_weapon=NULL;
	p->active_shield=NULL;
	/* Give default weapons, set active weapon.*/
	wl=NULL;
	wp=firstweapon;
	while(wp!=NULL){
		if(wl==NULL)
		  wl=(p->weapons)=malloc(sizeof(struct weapon_list));
		else{
			wl->next=malloc(sizeof(struct weapon_list));
			wl=wl->next;
		}
		wl->next=NULL;
		wl->type=wp;
		/* Get amount from weapon definition. */
		wl->amount=wp->default_amount;
		/* Set active weapon */
		if(wp->index==roster[rosterindex].default_weapon)
		  p->active_weapon=wp;
		wp=wp->next;
	}
	/* Give default shields */
	sl=NULL;
	sp=firstshield;
	while(sp!=NULL){
		if(sl==NULL)
		  sl=(p->shields)=malloc(sizeof(struct shield_list));
		else{
			sl->next=malloc(sizeof(struct shield_list));
			sl=sl->next;
		}
		sl->next=NULL;
		sl->type=sp;
		/* Get amount from weapon definition. */
		sl->amount=sp->default_amount;
		/* Set active_shield */
		if(sp->index==roster[rosterindex].default_shield)
		  p->active_shield=sp;
		/* Set active weapon */
		sp=sp->next;
	}
	/* Make sure these are set to something.*/
	if(p->active_weapon==NULL)
	  p->active_weapon=firstweapon;
	if(p->active_shield==NULL)
	  p->active_shield=firstshield;
	/**/
	++player_n;
	return p;
}


/* Saves the player roster */
void save_roster()  {
	FILE *fp;
	int a,l;
	sprintf(buffer,"%s/players/%s",tp2_datapath,ROSTERFILE);
	if((fp=fopen(buffer,"w"))){
		for(a=0;a<ROSTER_SIZE;a++)   {
			/* Replace ' ' -> '_' */
			sprintf(buffer,"%s",roster[a].name);
			for(l=0;l<strlen(buffer);l++){
			  if(buffer[l]==' ')
				  buffer[l]='_';
			}
			/* Save one roster entry */
			fprintf(fp,"%s\n%d %d %d %d %d\n%u %u\n\n",
					  buffer,
					  roster[a].games,
					  roster[a].won,
					  roster[a].deaths,
					  roster[a].kills,
					  roster[a].suicides,
					  roster[a].default_weapon,
					  roster[a].default_shield);
		}
		fclose(fp);
		game_flags |= ROSTER_SAVED;
	}
}
