/*
 *	Tykkipeli 2
 *	Shield stuff.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <terrain.h>
#include <bullets.h>
#include <guns.h>
#include <weapons.h>
#include <colors.h>
#include <weapon_defines.h>
#include <shield_defines.h>

#include <hr_defines.h>
#include <hr_general.h>


struct shield{
   struct shield  *next;
	int flags;
	unsigned int index;
	unsigned char  *name;
	int efficiency;
	int range;
	int color;
	int cost;
	int default_amount;
};


/* Returns color of shield pixel. SLOW CRAP! FIX! */
#define DISTORTION_OFFSET -30
int shield_color(struct player *p,int x,int y,int flags){
	if(flags & DRAW_SHIELD_FLASH)
	  return MINCOL_SHIELD;
	else if(game_flags & SKY_PIXMAPS)
	  /* If sky pixmaps are used, use solid shield colors.*/
	  return MIN(MAXCOL_SHIELD,(MINCOL_SHIELD+1+(p->active_shield->color)));
	else
	  /* If color slides are used, use distorted sky color.*/
	  return sky_color(x,y+DISTORTION_OFFSET);
}


/* Reads shield definitions from SHIELDDATA.
 * Returns 0 if everything goes ok, non-zero if an error occurs.*/
/* NOTE! Trashes mem if flag/index strings are too long.*/
int create_shields() {
	struct shield *sp;
	unsigned char tmp[BUFFER_SIZE],indextmp[5],flagtmp[16];
	FILE *fp;
	int l,readmode=0;
	sprintf(buffer,"%s/shields/%s",tp2_datapath,SHIELDDATA);
	if((fp=fopen(buffer,"r"))){
		sp=firstshield;	
		while(fgets(tmp,BUFFER_SIZE,fp)){
			if(!readmode){
				if(strncasecmp("#SHIELDS",tmp,8)==0)
				  readmode=1;
			}
			else{
				if(strncasecmp("#ENDSHIELDS",tmp,11)==0){
					readmode=2;
					break;
				}
				else{
					++shield_n;
					if(sp==NULL)
					  sp=firstshield=malloc(sizeof(struct shield));
					else  {
						sp->next=malloc(sizeof(struct shield));
						sp=sp->next;
					}
					sp->next=NULL;
					/**/
					sscanf(tmp,"%s %s %s %d %d %d %d %d",
							 buffer,
							 indextmp,
							 flagtmp,
							 &(sp->efficiency),
							 &(sp->default_amount),
							 &(sp->range),
							 &(sp->color),
							 &(sp->cost));
					/* Create 32bit index number from 4 first chars in
					 * indextmp[].*/
					sp->index=CHARS2INT(indextmp);
					/* Set shield type flags */
					sp->flags = 0;
					for(l=0;l<strlen(flagtmp);l++){
						if(flagtmp[l]!='.')
						  sp->flags |= (1<<l);
					}
					/* Shrink range if necessary. FIX.*/
					if(sp->range>=(TEMPMAP_RANGE-BARREL_OFFSET))
					  sp->range=(TEMPMAP_RANGE-BARREL_OFFSET-1);
					/* Replace '_' -> ' ' */		
					for(l=0;l<strlen(buffer);l++){
						if(buffer[l]=='_')
						  buffer[l]=' ';
					}
					/* Create shield name.*/
					sp->name=malloc(strlen(buffer)+1);
					strcpy(sp->name,buffer);
					/* Check whether color value is too large.
					 * Re-set color if necessary.*/
					if((MINCOL_SHIELD+(sp->color))>MAXCOL_SHIELD)
					  sp->color=(MAXCOL_SHIELD-MINCOL_SHIELD);
				}
			}
		}
		fclose(fp);
		if(readmode!=2)
		  printf("tp2: Error reading %s.\n",SHIELDDATA);
	}
	else
	  printf("tp2: Error opening %s.\n",SHIELDDATA);
	return (readmode-2);
}


/* Returns pointer to shield with given index */
struct shield *get_shield(unsigned int index) {
   struct shield *s=firstshield;
   while(s!=NULL) {
		if((s->index)==index)
		  return s;
		s=s->next;
	}
   return NULL;
}


/* Returns pointer to player who's shield is at (x,y),
 * but only if the shield is not all used up and if the shield
 * does something to bullets.*/
struct player *is_shield(int x,int y){
	struct player *p;
	struct shield_list *sl;
	p=firstplayer;
	while(p!=NULL){
		if(p->flags & ALIVE){
			sl=p->shields;
			while((sl->type)!=(p->active_shield))
			  sl=sl->next;
			if(((sl->amount)>0) &&
				(DISTANCE(p->x,p->y,x,y)<=(p->active_shield->range)) &&
				(!(sl->type->flags & SHIELD_ROCKET)))
			  return p;
		}
		p=p->next;
	}
	return NULL;
}


/* Draws shield belonging to p. */
#define CIRCLE_STEP 4
void draw_addon(struct player *p,int flags){
	int x,y,a,xp,yp,div,l;
	struct shield_list *sl;
	if(p!=NULL){
		if(p->flags & ALIVE){
			/* Draw shield around the gun.*/
			if((flags & DRAW_SHIELD)||
				(flags & DRAW_SHIELD_FLASH)){
				/* Draw only if other flags than SHIELD_ROCKET alone are set.*/
				if(p->active_shield->flags != SHIELD_ROCKET){
					sl=p->shields;
					while((sl->type)!=(p->active_shield))
					  sl=sl->next;
					if((sl->amount)>0){
						x=(p->x);
						y=(p->y)-BARREL_OFFSET;
						div=(1<<SCALE)/(p->active_shield->range);
						for(a=0;a<=45;a+=CIRCLE_STEP){
							xp=pcos[a]/div;
							yp=psin[a]/div;
							if(pixel_type(PIX_SKY,vga_getpixel(x+xp,y+yp)))
							  gl_setpixel(x+xp,y+yp,shield_color(p,x+xp,y+yp,flags));
							if(pixel_type(PIX_SKY,vga_getpixel(x-xp,y+yp)))
							  gl_setpixel(x-xp,y+yp,shield_color(p,x-xp,y+yp,flags));
							if(pixel_type(PIX_SKY,vga_getpixel(x+xp,y-yp)))
							  gl_setpixel(x+xp,y-yp,shield_color(p,x+xp,y-yp,flags));
							if(pixel_type(PIX_SKY,vga_getpixel(x-xp,y-yp)))
							  gl_setpixel(x-xp,y-yp,shield_color(p,x-xp,y-yp,flags));
							/* Swap x and y.*/
							if(pixel_type(PIX_SKY,vga_getpixel(x+yp,y+xp)))
							  gl_setpixel(x+yp,y+xp,shield_color(p,x+yp,y+xp,flags));
							if(pixel_type(PIX_SKY,vga_getpixel(x-yp,y+xp)))
							  gl_setpixel(x-yp,y+xp,shield_color(p,x-yp,y+xp,flags));
							if(pixel_type(PIX_SKY,vga_getpixel(x+yp,y-xp)))
							  gl_setpixel(x+yp,y-xp,shield_color(p,x+yp,y-xp,flags));
							if(pixel_type(PIX_SKY,vga_getpixel(x-yp,y-xp)))
							  gl_setpixel(x-yp,y-xp,shield_color(p,x-yp,y-xp,flags));
						}
					}
				}
			}
			/* Draws a rocket flame under the gun */
			if(flags & DRAW_ROCKET){
				/* FIX!!!! Find better place for the rocket effect! */
				for(l=0;l<3;l++){
					/* Middle of the flame */
					gl_setpixel((p->x),
									(p->y)+BARREL_OFFSET+
									random()%(TEMPMAP_RANGE-BARREL_OFFSET-1),
									MINCOL_EXPLOSION+
									random()%(MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1));
					/* Sides*/
					gl_setpixel((p->x)-1,
									(p->y)+BARREL_OFFSET+
									random()%(TEMPMAP_RANGE-BARREL_OFFSET-5),
									MINCOL_EXPLOSION+
									random()%(MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1));
					gl_setpixel((p->x)+1,
									(p->y)+BARREL_OFFSET+
									random()%(TEMPMAP_RANGE-BARREL_OFFSET-5),
									MINCOL_EXPLOSION+
									random()%(MAXCOL_EXPLOSION-MINCOL_EXPLOSION+1));
				}
			}
			/* Draws barrel of a gun */
			if(flags & DRAW_BARREL){
				/* Only one barrel color in use? FIX! */
				if(p->flags & ALIVE)
				  gl_line((p->x),(p->y)-BARREL_OFFSET,
							 (p->x)+((pcos[p->angle]*BARREL_LEN)>>SCALE),
							 ((p->y)-BARREL_OFFSET-
							  ((psin[p->angle]*BARREL_LEN)>>SCALE)),
							 MINCOL_BARREL);
			}
		}
	}
}


/* When bullet b hits shield of player p,
 * this func is called.
 * Returns 1 if a shield was damaged, 0 otherwise.
 * (This info is used to check if panel bars need updating.)*/
int shield_effect(struct player *p,struct bullet *b){
	struct shield_list *sl;
	int please_update_panel_bars=0;
	double xx,yy,zz;
	if((p!=NULL)&&(b!=NULL)){
		if(!(b->flags & BULLET_PENETRATES_SHIELDS)){
			sl=p->shields;
			while((sl->type)!=(p->active_shield))
			  sl=sl->next;
			if((sl->amount)>0){
				/* Flash the shield. */
				draw_addon(p,DRAW_SHIELD_FLASH);
				/**/
				if(p->active_shield->flags & SHIELD_STOPPER){
					/* Damage shield.*/
					sl->amount-=(b->type->efficiency);
					b->flags |= BULLET_HIT;
					please_update_panel_bars=1;
				}
				/**/
				if(p->active_shield->flags & SHIELD_ABSORB){
					/* This shield does its trick by modifyinf damage later.*/
					sl->amount-=((b->type->efficiency)/4+(b->type->weight));
					b->flags |= BULLET_HIT;
					please_update_panel_bars=1;
				}
				/**/
				if((p->active_shield->flags & SHIELD_REFLECT)||
					(p->active_shield->flags & SHIELD_BOUNCE)){
					/* The heavier the bullet, the more shield energy
					 * is drained.*/
					sl->amount-=((b->type->efficiency)/4+(b->type->weight));
					b->xs = -(b->xs);
					if(p->active_shield->flags & SHIELD_BOUNCE)
					  /* Flip x AND y :-B Cruel.*/
					  b->ys = -(b->ys);
					please_update_panel_bars=1;
				}
				/**/
				else if(p->active_shield->flags & SHIELD_SCATTER){
					/* Scatters bullets to random direction by rotating
					 * speed vectors 0-359 degrees */
					sl->amount-=((b->type->efficiency)/10+(b->type->weight));
					zz=(random()%637)/100.0;
					xx=((b->xs)*cos(zz))-((b->ys)*sin(zz));
					yy=((b->xs)*sin(zz))+((b->ys)*cos(zz));
					b->xs=(int)xx;
					b->ys=(int)yy;
					please_update_panel_bars=1;
				}
			}
			/* Make sure shield conditions don't drop below zero */
			if((sl->amount)<0)
			  sl->amount=0;
		}
	}
	return please_update_panel_bars;
}


/* When a player gets damage, this is checked to see whether
 * the damage should be modified somehow.*/
int shield_modifieddamage(struct player *p,int damage,int damagetype){
	struct shield_list *sl;
	/* Add NULL checks (?).
	 * Not necessary at the moment (only called from within hurt()).*/
	sl=p->shields;
	while((sl->type)!=(p->active_shield))
	  sl=sl->next;
	if((sl->amount)>0){
		switch(damagetype){
		 case DAMAGE_EXPLO:
		 case DAMAGE_DOT:
			/* Absorber only absorbs explosion and "dot" damage*/
			/* Damage shield.*/
			if(sl->type->flags & SHIELD_ABSORB){
				sl->amount-=damage;
				damage=0;
				if((sl->amount)<0)
				  damage=-(sl->amount);
			}
			break;
		 case DAMAGE_DROP:
			/* Damage rocket.*/
			if(sl->type->flags & SHIELD_ROCKET){
				sl->amount-=damage;
				damage=0;
				draw_addon(p,DRAW_ROCKET);
				if((sl->amount)<0)
				  damage=-(sl->amount);
			}
			break;
		}
		/* Make sure shield conditions don't drop below zero */
		if((sl->amount)<0)
		  sl->amount=0;
	}
	return damage;
}
