/*
 *	Tykkipeli 2
 *	Terrain creation
 */
  
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <guns.h>
#include <explode.h>

#include <hr_defines.h>
#include <hr_bobs.h>
#include <hr_general.h>
#include <hr_bitmaps.h>
#include <hr_ilbm.h>
#include <hr_keyboard.h>
#include <info_windows.h>

#include <colors.h>

int collapse();

int terrain[S_WIDTH],tertemp[S_WIDTH];
unsigned char sky_pixmap[MAX_X+1][MAX_Y+1];


/* Returns sky pixel color under coordinates (x,y) */
int sky_color(int x,int y){
	if((x<0) || (x>MAX_X))
	  return 0;
	if(y<=0)
	  return sky_pixmap[x][0];
	else if(y>=MAX_Y)
	  return sky_pixmap[x][MAX_Y];
	else
	  return sky_pixmap[x][y];
}


#define TC (MAXCOL_GROUND-MINCOL_GROUND+1)
void set_ground_palette(){
	int c;
	/* Others...*/
	if(!(game_flags & ALIEN_TERRAIN))
	  switch(random()%4) {
		case 0:
		case 1:
		case 2:
		  /* Green */
		  for(c=0;c<TC;c++)
			 gl_setpalettecolor(MINCOL_GROUND+c,5,(c*32)/TC,10);
		  break;
		case 3:
		  /* Snow */
		  for(c=0;c<TC;c++)
			 gl_setpalettecolor(MINCOL_GROUND+c,
									  50-(c*32)/TC,50-(c*32)/TC,50-(c*32)/TC);
		  break;
	  }
	else
	  switch(random()%2) {
		case 0:
		  /* 128-159 Alien #1 */
		  for(c=0;c<TC;c++)
			 gl_setpalettecolor(MINCOL_GROUND+c,
									  63-(c*32)/TC,0,63-(c*32)/TC);
		  break;
		case 1:
		  /* 128-159 Alien #2 */
		  for(c=0;c<TC;c++)
			 gl_setpalettecolor(MINCOL_GROUND+c,
									  0,33-(c*16)/TC,63-(c*32)/TC);
		  break;
	  }
}


/* Calcs a dithered color slide for sky, or loads image from disk. */
void create_sky() {
	int x,y,xoffset,yoffset,xflip,yflip,x2,y2,x2init,n,l,r,tt;
	int sky_created=0;
	DIR *dirp;
	struct dirent *dp;
	unsigned char tmp[BUFFER_SIZE],*tp,c,darkest;
	struct hr_bitmap *bmap;
	/**/
	while(!sky_created){
		if(!(game_flags & SKY_PIXMAPS)){
			/* The pre-made palette contains a nice color slide.
			 * Now make something creative out of it.*/
			for (y=0;y<=MAX_Y;y++){
				r=MINCOL_SKY+((y*(MAXCOL_SKY-MINCOL_SKY))/MAX_Y);
				for(x=0;x<=MAX_X;x++){
					tt=r;
					/* Y dithering.*/
					if(((y%4)==1)||((y%5)==4))
					  ++tt;
					/* X dithering.*/
					if((y>0) && (x%2) && (sky_pixmap[x][y-1]!=tt))
					  ++tt;
					/* FIXaddFIXadd! */
					/* Color in allower range?*/
					if(tt<MINCOL_SKY)
					  tt=MINCOL_SKY;
					else if(tt>MAXCOL_SKY)
					  tt=MAXCOL_SKY;
					sky_pixmap[x][y]=(unsigned char)tt;
					/* Draw the pixel */
					gl_setpixel(x,y,tt);
				}
			}
			/* That wasn't very creative was it? :-) Ah hell.*/
			sky_created=1;
		}
		/* Read a precalced pixmap from tp2 data directory. */
		else{
			/* Build $TP2_DATAPATH/skies/ to tmp[].*/
			sprintf(tmp,"%s/skies",tp2_datapath);
			/* Open directory for scanning*/
			n=0;
			if((dirp=opendir(tmp))){
				/* Get next dir entry.*/
				while((dp=readdir(dirp))){
					sprintf(buffer,"%s/%s\n",tmp,dp->d_name);
					if(strncasecmp("sky",dp->d_name,3)==0)
					  ++n;
				}
				/* Now n holds the amount of dir entries.
				 * Reset the directory stream and get a random dir entry.*/
				r=random()%n;
				rewinddir(dirp);
				for(l=0;l<=r;l++){
					/* Skip past unwanted entries.*/
					do{
						dp=readdir(dirp);
					}while(strncasecmp("sky",dp->d_name,3)!=0);
				}
				/* Build whole file name, path included, in buffer.*/
				sprintf(tmp,"%s/skies/%s",tp2_datapath,dp->d_name);
				/* See if there's "nx" or "ny" string in filename.*/
				xflip=yflip=1;
				if(strlen(dp->d_name)>4){
					for(l=0;l<strlen(dp->d_name)-1;l++){
						if(((*((dp->d_name)+l))=='n') ||
							((*((dp->d_name)+l))=='N')){
							switch(*((dp->d_name)+l+1)){
							 case 'x':
							 case 'X':
								xflip=0;
								break;
							 case 'y':
							 case 'Y':
								yflip=0;
								break;
							}
						}
					}
				}
				if(xflip)
				  xflip=random()%2;
				if(yflip)
				  yflip=random()%2;
				/* Close dir*/
				closedir(dirp);
				/* Read the file into a bitmap.*/
				bmap=hr_read_ilbm(tmp,GFX_256COLOR);
				/* Recolor the bitmap.*/
				hr_recolor_bitmap(bmap,MINCOL_SKY,GFX_INCLUDEZERO);
				/* Set palette from MINCOL_SKY+1 onwards.*/
				hr_use_bitmap_palette(bmap,MINCOL_SKY,MAXCOL_SKY,
											 GFX_INCLUDEZERO);
				
				/* Set random x/y offsets if sky pixmap is
				 * smaller or larger than screen.*/
				if((x2=(S_WIDTH-(bmap->width))))
				  xoffset=((x2<0)?(1):(-1))*(random()%abs(x2+1));
				else
				  xoffset=0;
				if((y2=((MAX_Y+1)-(bmap->height))))
				  yoffset=((y2<0)?(1):(-1))*(random()%abs(y2+1));
				else
				  yoffset=0;
				
				/* Get darkest color in bitmap.*/
				darkest=hr_color_extreme(NULL,MINCOL_SKY,MAXCOL_SKY,
												 GFX_NOBMAPPALETTE|GFX_DARKEST);
				/* Fill sky_pixmap[][]-array.*/
				/* Scan all pixels on game area. Pixels that are
				 * "outside" the sky pixmap are set to "darkest" color.*/
				if(yflip)
				  y2=((bmap->height)-1)-yoffset;
				else
				  y2=yoffset;
				if(xflip)
				  x2init=((bmap->width)-1)-xoffset;
				else
				  x2init=xoffset;
				for(y=0;y<=MAX_Y;y++){
					x2=x2init;
					tp=((bmap->data)+y2*(bmap->width)+x2);
					/* If y2 is "outside" bmap,
					 * the whole scanline can be set to "darkest" color.*/
					if((y2<0)||(y2>=(bmap->height))){
						for(x=0;x<S_WIDTH;x++)
						  sky_pixmap[x][y]=darkest;
						gl_hline(0,y,MAX_X,darkest);
					}
					/* Normally, pixels must be copied from bmap.*/
					else{
						for(x=0;x<S_WIDTH;x++){
							if((x2>=0)&&
								(x2<(bmap->width)))
							  c=*tp;
							else
							  c=darkest;
							sky_pixmap[x][y]=c;
							gl_setpixel(x,y,c);
							if(xflip){
								--x2;
								--tp;
							}
							else{
								++x2;
								++tp;
							}
						}
					}
					if(yflip)
					  --y2;
					else
					  ++y2;
				}
				/* Free mem. Bmap no longer needed.*/
				hr_delete_bitmap(bmap);
				sky_created=1;
			}
			else
			  /* Toggle off sky pixmap flag.*/
			  game_flags &= ~(SKY_PIXMAPS);
		}
	}
}


/* Colors for sky. This actually does nothing if precalced
 * pixmaps are used. */
void set_sky_palette(){
	int c,x=0,l=0,d,o,rgb_count=0,tc;
#define SC ((MAXCOL_SKY-MINCOL_SKY+1))
	int rr[SC],gg[SC],bb[SC];
	/**/
	while(rgb_count==0){
		rgb_count=0;
		/* Zero the color component arrays */
		for(c=0;c<SC;c++)
		  rr[c]=gg[c]=bb[c]=0;
		/* Natural looking skies. FIX!MORE! */
		if(!(game_flags & ALIEN_SKY)) {
			++rgb_count;
			switch(2) {
			 case 0:
			 case 1:
				/* Blue */
				l=random()%2;
				for(c=0;c<SC;c++) {
					bb[c]=(c*63)/SC;
					/* Lighter blue */
					if(l)
					  gg[c]=(c*32)/SC;
				}
				break;
			}
		}
		/* Random alien sky. Some of them look natural, some don't :-) FIX! */
		else {
			for(x=0;x<3;x++) {
				/* divider */
				d=1+random()%4;
				/* Offset */
				if(d>1)
				  o=random()%(63-(63/d));
				else
				  o=0;
				/* Slide direction */
				l=random()%2;
				/* Either use a color component, or don't.*/
				if(random()%2){
					++rgb_count;
					/* Go through one of the color component arrays,
					 * making a slide.*/
					for(c=0;c<SC;c++) {
						if(l)
						  tc=o+((c*(63/d))/SC);
						else
						  tc=63-o-((c*(63/d))/SC);
						switch(x) {
						 case 0:
							rr[c]=tc;
							break;
						 case 1:
							gg[c]=tc;
							break;
						 case 2:
							bb[c]=tc;
							break;
						}
					}
				}
			}
		}
	}
	/* Now actually set the sky colors.*/
	for(c=0;c<SC;c++)
	  gl_setpalettecolor(MINCOL_SKY+c,rr[c],gg[c],bb[c]);
}


/* Returns 1 if pixel (x,y) is of type "type"...
 *  * Since vga_getpixel(out-of-screen coordinates) seems to return 0,
 *  * no boundary checking is necessary.*/
int pixel_type(int type,int c)   {
	switch(type)   {
	 case PIX_QUERY:
		/* Type PIX_QUERY indicates "gimme type of that pixel".*/
		if((c>=MINCOL_GUNS)&&(c<=MAXCOL_GUNS))
		  return PIX_GUN;
		if((c>=MINCOL_EXPLOSION)&&(c<=MAXCOL_EXPLOSION))
		  return PIX_EXPLOSION;
		if((c>=MINCOL_BULLET)&&(c<=MAXCOL_BULLET))
		  return PIX_BULLET;
		if((c>=MINCOL_GROUND)&&(c<=MAXCOL_GROUND))
		  return PIX_GROUND;
		if((c>=MINCOL_SKY)&&(c<=MAXCOL_SKY))
		  return PIX_SKY;
		if((c>=MINCOL_SHIELD)&&(c<=MAXCOL_SHIELD))
		  return PIX_SHIELD;
		break;
	 case PIX_GUN:
		if(((c>=MINCOL_GUNS)&&(c<=MAXCOL_GUNS)) ||
			((c>=MINCOL_BARREL)&&(c<=MAXCOL_BARREL)))
		  return PIX_GUN;
		break;
	 case PIX_EXPLOSION:
		if((c>=MINCOL_EXPLOSION)&&(c<=MAXCOL_EXPLOSION))
		  return PIX_EXPLOSION;
		break;
	 case PIX_BULLET:
		if((c>=MINCOL_BULLET)&&(c<=MAXCOL_BULLET))
		  return PIX_BULLET;
		break;
	 case PIX_SHIELD:
		if((c>=MINCOL_SHIELD)&&(c<=MAXCOL_SHIELD))
		  return PIX_SHIELD;
		break;
	 case PIX_GROUND:
		if((c>=MINCOL_GROUND)&&(c<=MAXCOL_GROUND))
		  return PIX_GROUND;
		break;
	 case PIX_SKY:
		if((c>=MINCOL_SKY)&&(c<=MAXCOL_SKY))
		  return PIX_SKY;
		break;
	}
	return 0;
}


/* Draws one column, from top of the screen to the terrain surface.*/
void sky_column(int x) {
	int y;
	if(x<0 || x>MAX_X)
	  return;
	else {
		for(y=0;y<terrain[x];y++)
		  gl_setpixel(x,y,sky_color(x,y));
	}
}


/* Draw the whole sky. */
void draw_sky(){
	int x,y;
	for(y=0;y<=MAX_Y;y++) {
		for(x=0;x<=MAX_X;x++)
		  gl_setpixel(x,y,sky_color(x,y));
	}
}


/* Calculates height of terrain after a collapse/etc...*/
void calc_heights(int x1,int x2) {
	int x,y;
	if(x2>MAX_X) x2=MAX_X;
	if(x1<0) x1=0;
	if(x1>x2) {
		x=x1;x1=x2;x2=x;
	}
	for(x=x1;x<=x2;x++) {
		for(y=0;y<=MAX_Y;y++) {
			if(pixel_type(PIX_GROUND,vga_getpixel(x,y))) {
				terrain[x]=y;
				break;
			}
		}
	}
}


/* Creates the terrain array and does other kewl stuff. */
void create_terrain(int y_min, int y_max, int steepness){
	int x,xflip,x2,ys,y_offset,y,c,r,n,l,terrain_created=0;
	struct hr_bitmap *bmap;
	DIR *dirp;
	struct dirent *dp;
	unsigned char tmp[BUFFER_SIZE],*tp;
	/* Fast but not-so-cool terrains.*/
	while(!terrain_created){
		if(!(game_flags & TERRAIN_PIXMAPS)){
			y_min=(y_min<<8);
			y_max=(y_max<<8);
			steepness=steepness<<6;	
			ys=0;
			y=y_min+random()%(abs(y_max-y_min)+1);
			for(x=0;x<=MAX_X;x++) {
				ys+=250-(random()%501);
				if(abs(ys)>steepness)
				  ys/=2;
				y+=ys;
				if(y<y_min) {
					y=y_min;
					ys=random()%33;
				}
				if(y>y_max) {
					y=y_max;
					ys=-(random()%33);
				}
				terrain[x]=(y>>8);		
			}
			/* Draw the terrain slooooowly... */
			for(x=0;x<=MAX_X;x++) {
				/* Normal. */
				for(y=terrain[x];y<=MAX_Y;y++) {
					c=MINCOL_GROUND+((y-terrain[x])*TC)/(MAX_Y-terrain[x]);
					c+=4-random()%9;						/* dithering */
					if(c<MINCOL_GROUND)
					  c=MINCOL_GROUND;
					else if(c>MAXCOL_GROUND)
					  c=MAXCOL_GROUND;
					gl_setpixel(x,y,c);
				}
			}
			terrain_created=1;
		}
		else{
			/* Build $TP2_DATAPATH/terrains/ to tmp[].*/
			sprintf(tmp,"%s/terrains",tp2_datapath);
			/* Open directory for scanning*/
			n=0;
			if((dirp=opendir(tmp))){
				/* Get next dir entry.*/
				while((dp=readdir(dirp))){
					sprintf(buffer,"%s/%s\n",tmp,dp->d_name);
					if(strncasecmp("ter",dp->d_name,3)==0)
					  ++n;
				}
				/* Now n holds the amount of dir entries.
				 * Reset the directory stream and get a random dir entry.*/
				r=random()%n;
				rewinddir(dirp);
				for(l=0;l<=r;l++){
					/* Skip past unwanted entries.*/
					do{
						dp=readdir(dirp);
					}while(strncasecmp("ter",dp->d_name,3)!=0);
				}
				/* Build whole file name, path included, in buffer.*/
				sprintf(tmp,"%s/terrains/%s",tp2_datapath,dp->d_name);
				/* Close dir*/
				closedir(dirp);
				/* Read the file into a bitmap.*/
				bmap=hr_read_ilbm(tmp,GFX_256COLOR);
				/* Recolor the bitmap.*/
				hr_recolor_bitmap(bmap,MINCOL_GROUND-1,NONE);
				/* Set palette from MINCOL_GROUND onwards.*/
				hr_use_bitmap_palette(bmap,MINCOL_GROUND,MAXCOL_GROUND,
											 NONE);
				/* Draw the bitmap. It is scanned in row-major order,
				 * because hr_bitmaps are stored in mem as subsequent
				 * scanlines.
				 * The bitmap is lowered so that it's "base" lies on control panel.
				 * This is so that terrain pixmap's height must not necessarily
				 * match that of game area's. */
				for(x=0;x<=MAX_X;x++){
					terrain[x]=0;
					tertemp[x]=MAX_Y;
					tmp[x]=0;
				}
				y_offset=MAX(0,(MAX_Y+1-(bmap->height)));
				xflip=random()%2;
				for(y=y_offset;y<=MAX_Y;y++){
					/* This stupid multiplication is here to prevent
					 * screwup in case bmap width != screen width. */
					tp=(bmap->data)+(y-y_offset)*(bmap->width);
					for(x=0;x<=MIN(MAX_X,((bmap->width)-1));x++){
						/* Flip the terrain horizontally */
						if(xflip)
						  x2=x;
						else
						  x2=MAX_X-x;
						/* Nonzero bytes are drawn to screen 
						 * Zero bytes are ignored.
						 * This is because
						 * if the pixmap contains rgb values (0,0,0),
						 * recolor_bitmap() does not recolor those pixels
						 * (they are considered transparent).*/
						if(*tp)
						  gl_setpixel(x2,y,*tp);
						else if (tmp[x2]==0){
							tmp[x2]=1;
							terrain[x2]=y+1;
						}
						/* Move the temp pointer forward*/
						tp++;
					}
				}
				/* Free mem. Bmap no longer needed.*/
				hr_delete_bitmap(bmap);
				terrain_created=1;
			}
			else
			  /* Toggle off the terrain pixmap flag.*/
			  game_flags &= ~(TERRAIN_PIXMAPS);
		}
	}
}


/* Removes tracers... SLOW! */
void remove_tracers() {
	int x,y;
	gl_setclippingwindow(0,0,MAX_X,MAX_Y);
	if(game_flags & AIM_AID)
	  hr_update_bob(crosshairs,0,0,NULL,BOB_REMOVEONLY);
	update_guns(BOB_REMOVE,NONE);
	for(x=0;x<=MAX_X;x++) {
		for(y=0;y<terrain[x];y++)
		  gl_setpixel(x,y,sky_color(x,y));
	}
	update_guns(BOB_INIT,NONE);
	if(game_flags & AIM_AID)
	  hr_update_bob(crosshairs,0,0,NULL,BOB_INITONLY);
	gl_setclippingwindow(0,0,MAX_X,S_HEIGHT-1);
}


/* Replaces colors on range c1-c2 with sky pixels.*/
void replace_with_sky(int x1,int y1,int x2,int y2,int c1,int c2)  {
	int x,y,c;
	for(y=y1;y<=y2;y++){
		for(x=x1;x<=x2;x++){
			c=vga_getpixel(x,y);
			if((c>=c1) && (c<=c2))
			  gl_setpixel(x,y,sky_color(x,y));
		}
	}
}


/* "Lower" tertemp[]-array for collapsing...*/
void temp_lower(int x1,int x2,int y) {
	int x;
	x1=MAX(0,x1);
	x2=MIN(MAX_X,x2);
	if(x1>x2){
		x=x1;x1=x2;x2=x;
	}
	for(x=x1;x<=x2;x++){
		if(tertemp[x]<y)
		  tertemp[x]=y;
	}
}
void quick_lower(int x,int y) {
	if((x<0)||(x>MAX_X))
	  return;
	if(tertemp[x]<y)
	  tertemp[x]=y;
}


/* "Raise" terrain[]-array for collapsing...*/
void terrain_raise(int x1,int x2,int y) {
	int x;
	x1=MAX(0,x1);
	x2=MIN(MAX_X,x2);
	if(x1>x2) {
		x=x1;x1=x2;x2=x;
	}
	for(x=x1;x<=x2;x++){
		if(terrain[x]>y)
		  terrain[x]=y;
	}
}
void quick_raise(int x,int y) {
	if((x<0)||(x>MAX_X))
	  return;
	if(terrain[x]>y)
	  terrain[x]=y;
}


/*
 *	Tykkipeli 2
 *	Terrain collapsing
 */
  

/* Terrain Smoothing.
 * Removes "pikes" that are easily created by some weapons.
 * Does nothing in zero gravity.*/
#define SMOOTH_THRESHOLD 2
int smooth(){
	int smoothed_some=0;
	int x,c,p;
	if(g>0){
		if(game_flags & GROUNDSMOOTHING){
			for(x=1;x<(MAX_X-1);x++){
				if((terrain[x]<(terrain[x+1]-SMOOTH_THRESHOLD)) &&
					(terrain[x]<(terrain[x-1]-SMOOTH_THRESHOLD)))
				  {
					  update_guns_on_xrange(x-1,x+1,BOB_REMOVE,NONE);
					  /* Which side? */
					  smoothed_some=1;
					  if(gfx_counter%2)
						 p=1;
					  else
						 p=-1;
					  /* Lower pike */
					  c=gl_getpixel(x,terrain[x]);
					  gl_setpixel(x,terrain[x],sky_color(x,terrain[x]));
					  ++terrain[x];
					  /* Draw pike fragment */
					  --terrain[x+p];
					  gl_setpixel(x+p,terrain[x+p],c);
					  update_guns_on_xrange(x-1,x+1,BOB_INIT,NONE);
				  }
			}
		}
	}
	return smoothed_some;
}


/* Collapses terrain.
 * Does nothing in zero gravity.*/
int collapse() {
	int c,x,y=0,collapsed_some=0,x1,x2;
	if(g>0){
		/* Do some tidying-up and set the "remove guns"-range.
		 * This check is kinda lame - only one range, so two small collapsing
		 * areas on both sides of screen cause ALL guns to be updated...
		 * Anyway, usually this check is good enough and cuts down the amount
		 * of flickering.*/
		x1=MAX_X;
		x2=0;
		for(x=0;x<=MAX_X;x++) {
			if(tertemp[x]>0){
				if(tertemp[x]>MAX_Y)
				  tertemp[x]=MAX_Y;
				if((tertemp[x]>=terrain[x]) && (terrain[x]<=MAX_Y)) {
					collapsed_some=1;
					if((gfx_counter%5)==(x%5)) {
						if(x<x1)
						  x1=x;
						if(x>x2)
						  x2=x;
					}
				}
			}
		}
		/* Remove guns if necessary */
		if(x1<=x2)
		  update_guns_on_xrange(x1,x2,BOB_REMOVE,NONE);
		
		/* Now collapse.*/
		for(x=0;x<=MAX_X;x++) {
			if(tertemp[x]>0){
				if((gfx_counter%5)==(x%5)) {
					if((tertemp[x]>=terrain[x]) && (terrain[x]<=MAX_Y)) {
						/* Move tertemp[] up over ground pixels.*/
						while(pixel_type(PIX_GROUND,vga_getpixel(x,tertemp[x])))
						  tertemp[x]--;
						/* Move terrain[] down until ground or gun is encountered.*/
						c=vga_getpixel(x,terrain[x]);
						while(!(pixel_type(PIX_GROUND,c))  &&
								(terrain[x]<=MAX_Y))
						  {
							  terrain[x]++;
							  c=vga_getpixel(x,terrain[x]);
						  }
						/* Go "up" all the way from tertemp[x] to terrain[x]
						 * and move down pixels.*/
						if((tertemp[x]>=terrain[x]) && (terrain[x]<=MAX_Y)) {
							for(y=tertemp[x];y>=terrain[x];y--){
								c=vga_getpixel(x,y-1);
								if(pixel_type(PIX_GROUND,c))
								  gl_setpixel(x,y,c);
								else{
									if(pixel_type(PIX_GROUND,vga_getpixel(x,y)))
									  gl_setpixel(x,y,sky_color(x,y));
								}
							}
							gl_setpixel(x,terrain[x],sky_color(x,terrain[x]));
							terrain[x]++;
						}
						else
						  /* Mark this column as already collapsed.*/
						  tertemp[x]=0;
					}
				}
			}
		}
		
		/* Re-init guns if they were removed */
		if(x1<=x2)
		  update_guns_on_xrange(x1,x2,BOB_INIT,NONE);
	}
	/* If no collapsing was done, call smooth().*/
	if(collapsed_some)
	  return collapsed_some;
	else
	  return smooth();
}
