/*
 *	Tykkipeli 2
 *	Weapon stuff.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <ctype.h>

#include <defines.h>
#include <tp2.h>
#include <game_screen.h>
#include <players.h>
#include <colors.h>
#include <weapon_defines.h>
#include <explosion_types.h>

#include <hr_defines.h>
#include <hr_general.h>


struct warhead_list{
	struct warhead_list *next;
	struct warhead *type;
};

struct weapon{
	struct weapon *next;
	unsigned int index;
	unsigned char *name;
	int  shots;
	struct warhead_list **warheads;
	int  *speed;
	int  *variance;
	int  *spread;
	int  cost;
	int  default_amount;
};

struct warhead{
	struct warhead *next;
	unsigned int index;
	int  efficiency;
	int  xrange;
	int  yrange;
	int  path;
	int  trigger_step;
	int  explosion_type;
	int  flight_effect;
	int  weight;
	int  color;
};


/* Returns pointer to (first) weapon that has given index,
 * or NULL if no such weapon is found.*/
struct weapon *get_weapon(unsigned int index) {
	struct weapon *w=firstweapon;
	while(w!=NULL) {
		if((w->index)==index)
		  return w;
		w=w->next;
	}
	return NULL; 
}


/* Returns pointer to (first) warhead that has given index,
 * or NULL if no such warhead is found.*/
struct warhead *get_warhead(unsigned int index) {
   struct warhead *w=firstwarhead;
   while(w!=NULL) {
		if((w->index)==index)
		  return w;
		w=w->next;
	}
   return NULL;
}


/* Returns number of warheads in a warhead list.*/
int warheads_in_list(struct warhead_list *whl){
	int heads=0;
	while(whl!=NULL){
		++heads;
		whl=whl->next;
	}
	return heads;
}


/* Reads weapon definitions from WEAPONDATA. */
/* Return values: 0 ok, non-zero - error*/
int create_weapons() {
	struct weapon *wp;
	struct warhead_list *whl;
	FILE *fp;
	unsigned char tmp[BUFFER_SIZE],indextmp[5];
	unsigned char nametmp[BUFFER_SIZE],warheadtmp[BUFFER_SIZE];
	int l,s,readmode=0,shots;
	wp=firstweapon;
	sprintf(buffer,"%s/weapons/%s",tp2_datapath,WEAPONDATA);
	if((fp=fopen(buffer,"r"))){
		while(fgets(tmp,BUFFER_SIZE,fp)&&(readmode<2)){
			switch(readmode){
			 case 0:
				if(strncasecmp("#WEAPONS",tmp,8)==0)
				  readmode=1;
				break;
			 case 1:
				if(strncasecmp("#ENDWEAPONS",tmp,11)==0)
				  readmode=2;
				else{
					++weapon_n;
					/* Create new weapon */
					if(wp==NULL)
					  wp=firstweapon=malloc(sizeof(struct weapon));
					else {			
						wp->next=malloc(sizeof(struct weapon));
						wp=wp->next;
					}
					wp->next=NULL;
					/* Scan everything in one go.*/
					sscanf(tmp,"%s %s %d %d",
							 nametmp,
							 indextmp,
							 &(wp->default_amount),
							 &(wp->cost));
					/* Create weapon index */
					wp->index=CHARS2INT(indextmp);
					/* Create name */
					for(l=0;l<strlen(nametmp);l++){
						if(nametmp[l]=='_')
						  nametmp[l]=' ';
					}
					wp->name=malloc(strlen(nametmp)+1);
					strcpy(wp->name,nametmp);
					/* Now create warheadlists and get number of heads.*/
					wp->warheads=NULL;
					wp->speed=NULL;
					wp->variance=NULL;
					wp->spread=NULL;
					shots=s=0;
					/* Now the warheadlists for shot(s) are built.
					 * wp->warheads points to an array of pointers,
					 * which in turn point to linked warhead_lists.
					 * When a weapon is fired, these lists are scanned
					 * and bullets initialized.*/
					while(fgets(tmp,BUFFER_SIZE,fp)){
						/* Break out of the loop when the first character
						 * on line is a '#'.*/
						if(tmp[0]=='#')
						  break;
						else{
							/* Realloc enough space for pointer array.*/
							++shots;
							wp->warheads=realloc(wp->warheads,
														shots*sizeof(struct warhead_list *));
							wp->speed=realloc(wp->speed,
													shots*sizeof(int));
							wp->variance=realloc(wp->variance,
														shots*sizeof(int));
							wp->spread=realloc(wp->spread,
													 shots*sizeof(int));
							/* Get the string.*/
							sscanf(tmp,"%d %d %d %s",
									 &((wp->speed)[s]),
									 &((wp->variance)[s]),
									 &((wp->spread)[s]),
									 warheadtmp);
							/* Convert the char-values to int and 
							 * create warheadlist for this weapon.*/
							whl=NULL;
							for(l=0;l<strlen(warheadtmp);l++){
								if(whl==NULL)
								  whl=(wp->warheads)[s]=malloc(sizeof(struct warhead_list));
								else{
									whl->next=malloc(sizeof(struct warhead_list));
									whl=whl->next;
								}
								whl->type=get_warhead((int)(warheadtmp[l]));
								whl->next=NULL;
							}
							s++;
						}
					}
					wp->shots=shots;
				}
				break;
			}
		}
		fclose(fp);
		if(readmode!=2)
		  printf("tp2: Error parsing %s.\n",WEAPONDATA);
	}
	else
	  printf("tp2: Error opening %s.\n",WEAPONDATA);
	return (readmode-2);
}


/* This function creates special weapons that are used for particles
 * and player deaths. Note that because these weapons/warheads
 * are not part of weapon/warhead lists, get_weapon() and get_warhead()
 * never return pointers to these. Therefor, indexes can be whatever.*/
void create_special_ws(){
	struct warhead_list *whl;
	int l,s;
	/* round explosion that occurs when a player dies.----------------*/
#define DEATH_EXPLOSION_TYPE 1
	death_s.next=NULL;
	death_s.index=0;
	death_s.efficiency=0;
	death_s.xrange=30;
	death_s.yrange=30;
	death_s.path=PATH_NORMAL;
	death_s.trigger_step=0;
	death_s.explosion_type=DEATH_EXPLOSION_TYPE;
	death_s.flight_effect=0;
	death_s.weight=0;
	death_s.color=0;
	/* Init special weapon for death effect.--------------------------*/
#define DEATH2_HEADS 30
#define DEATH2_WARHEAD_TYPE 'p'
	death2_s.shots=1;
	death2_s.next=NULL;
	death2_s.index=0;
	death2_s.name=NULL;
	death2_s.warheads=malloc(death2_s.shots*sizeof(struct warhead_list *));
	for(s=0;s<death2_s.shots;s++){
		whl=NULL;
		for(l=0;l<DEATH2_HEADS;l++){
			if(whl==NULL)
			  whl=death2_s.warheads[s]=malloc(sizeof(struct warhead_list));
			else{
				whl->next=malloc(sizeof(struct warhead_list));
				whl=whl->next;
			}
			whl->next=NULL;
			whl->type=get_warhead((int)(DEATH2_WARHEAD_TYPE));
		}
	}
	death2_s.speed=malloc(sizeof(int));
	death2_s.variance=malloc(sizeof(int));
	death2_s.spread=malloc(sizeof(int));
	*death2_s.speed=0;
	*death2_s.variance=0;
	*death2_s.spread=180;
	death2_s.cost=0;
	death2_s.default_amount=-1;
	/* Init special warhead for particles.------------------------ */
	particle_s.next=NULL;
	particle_s.index=0;
	particle_s.efficiency=0;
	particle_s.xrange=1;
	particle_s.yrange=1;
	particle_s.path=PATH_NORMAL;
	particle_s.trigger_step=0;
	particle_s.explosion_type=EXPLO_PARTICLE;
	particle_s.flight_effect=0;
	particle_s.weight=20;
	particle_s.color=0;
}


/* Reads warhead data and creates the warhead array */
/* Return values: 0 ok, non-zero - error */
int create_warheads() {
	struct warhead *wp;
	unsigned char tmp[BUFFER_SIZE],indextmp[5];
   FILE *fp;
   int readmode=0;
	wp=firstwarhead;
	sprintf(buffer,"%s/weapons/%s",tp2_datapath,WEAPONDATA);
   if((fp=fopen(buffer,"r"))){
		while(fgets(tmp,BUFFER_SIZE,fp)&&(readmode<2)){
			switch(readmode){
			 case 0:
				if(strncasecmp("#WARHEADS",tmp,9)==0)
				  readmode=1;
				break;
			 case 1:
				if(strncasecmp("#ENDWARHEADS",tmp,12)==0)
				  readmode=2;
				else {
					++warhead_n;
					/* Read warhead infos*/
					if(wp==NULL)
					  wp=firstwarhead=malloc(sizeof(struct warhead));
					else  {			
						wp->next=malloc(sizeof(struct warhead));
						wp=wp->next;
					}
					wp->next=NULL;
					/* Now scan for rest of the info.*/
					sscanf(tmp,"%s %d %d %d %d %d %d %d %d %d",
							 indextmp,
							 &(wp->efficiency),
							 &(wp->xrange),
							 &(wp->yrange),
							 &(wp->path),
							 &(wp->trigger_step),
							 &(wp->explosion_type),
							 &(wp->flight_effect),
							 &(wp->weight),
							 &(wp->color));
					/* Set warhead index*/
					wp->index=indextmp[0];
					/*wp->index=CHARS2INT(indextmp);*/
					/* Check if color value is too large.
					 * If it is, re-set it.*/
					if((MINCOL_BULLET+(wp->color))>MAXCOL_BULLET)
					  wp->color=(MAXCOL_BULLET-MINCOL_BULLET);
				}
				break;
			}
		}
		fclose(fp);
		if(readmode!=2)
		  printf("tp2: Error parsing %s.\n",WEAPONDATA);
	}
	else
	  printf("tp2: Error opening %s.\n",WEAPONDATA);
	return (readmode-2);
}
